<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\Typo3VersionUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * AbstractController for the xBlog xtension
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.17
 * @since 0.3.0
 */
class AbstractBaseController extends ActionController
{

	/**
	 * @var TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
	 */
	public $_oCObj;

	/**
	 * @var TYPO3\CMS\Extbase\Object\ObjectManager
	 */
	private $_oObjectManager;

	/**
	 * @var \TYPO3\CMS\Frontend\Plugin\AbstractPlugin
	 */
	protected $_oPlugin;

	/**
	 * @var TYPO3\CMS\Core\TypoScript\TypoScriptService
	 */
	protected $_oTypoScriptService;

	/**
	 * @var ContentObjectRenderer
	 */
	protected $contentObject;

	/**
	 * TypoScript configuration
	 *
	 * @var array
	 */
	protected $conf;

	/**
	 * @var string Extension key
	 */
	public $extKey = 'xblog';

	/**
	 * elementRepository
	 * 
	 * @var \Netzmacher\Xblog\Domain\Repository\ElementRepository
	 */
	protected $elementRepository = null;

	/**
	 * formRepository
	 * 
	 * @var \Netzmacher\Xblog\Domain\Repository\FormRepository
	 */
	protected $formRepository = null;

	/**
	 * __construct(): Init the repositories
	 * 
	 * @version 0.30.17
	 * @since		0.20.8
	 */
	public function __construct()
	{
		$objectManager = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Object\\ObjectManager' );
		$this->elementRepository = $objectManager->get( 'Netzmacher\\Xblog\\Domain\\Repository\\ElementRepository' );
		$this->formRepository = $objectManager->get( 'Netzmacher\\Xblog\\Domain\\Repository\\FormRepository' );
		$this->tablelayoutRepository = $objectManager->get( 'Netzmacher\\Xblog\\Domain\\Repository\\TablelayoutRepository' );
	}

	/**
	 * Initializes this object
	 *
	 * @return void
	 * @codeCoverageIgnore
	 * @throws InvalidSlotException
	 * @throws InvalidSlotReturnException
	 */
	private function _initObjectContent()
	{
		// @extensionScannerIgnoreLine Seems to be a false positive: getContentObject() is still correct in 9.0
		$this->contentObject = $this->configurationManager->getContentObject();
		//var_dump(__METHOD__, __LINE__, $this->contentObject->data);
	}

	/**
	 * _initObjectCObj( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	private function _initObjectCObj()
	{
		$this->_oCObj = $this->_oObjectManager->get( 'TYPO3\\CMS\\Frontend\\ContentObject\\ContentObjectRenderer' );
		$this->_initObjectCObjData();
	}

	/**
	 * _initObjectCObjData( ) : Is needed, if in TypoScript is used: data = flexform : pi_flexform:...
	 * 													If it isn't set, data will be empty, and data[pi_flexform] too of course.
	 *
	 * @return void
	 * @access private
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	private function _initObjectCObjData()
	{
		$this->_oCObj->data = $this->contentObject->data;
	}

	/**
	 * _initObjectObjectManager( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	private function _initObjectObjectManager()
	{
		$this->_oObjectManager = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Object\\ObjectManager' );
	}

	/**
	 * _initObjectPlugin( ) : Init localisation
	 *
	 * @return void
	 * @access private
	 * @version 0.9,7
	 * @since 0.0.1
	 * @internal #i0062
	 */
	private function _initObjectPlugin()
	{
		$this->_oPlugin = GeneralUtility::makeInstance( 'Netzmacher\\Xblog\\Plugin\\AbstractPlugin' );
		// #t4513, 200202, dwildt, 1+
		$this->_oPlugin->LLkey = 'de';
		$this->_oPlugin->pi_loadLL( 'EXT:xblog/Resources/Private/Language/Paging.xlf' );
//		var_dump(__METHOD__, __LINE__, $this->_oPlugin->LOCAL_LANG, $this->_oPlugin->conf);
	}

	/**
	 * _initObjectTypoScriptService( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	private function _initObjectTypoScriptService()
	{

		switch( true )
		{
			case(Typo3VersionUtility::Get() < 10000000):
				$this->_oTypoScriptService = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Service\\TypoScriptService' );
				return;
			case(Typo3VersionUtility::Get() >= 10000000):
			default:
				$this->_oTypoScriptService = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Core\\TypoScript\\TypoScriptService' );
				return;
		}
	}

	/**
	 * _cObjGetSingle( ) :
	 *
	 * @param array $record
	 * @return string $value
	 * @access protected
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	protected function cObjGetSingle( $record )
	{
		if( empty( $record ) )
		{
			return;
		}

		if( !is_array( $record ) )
		{
			$prompt = 'FATAL ERROR: record isn\'t an array: '
							. serialize( $record ) . '<br />'
							. __METHOD__ . '#' . __LINE__;
			die( $prompt );
		}


		$name = $record[ '_typoScriptNodeValue' ];
		$conf = $this->_oTypoScriptService->convertPlainArrayToTypoScriptArray( $record );

		$value = $this->_oCObj->cObjGetSingle( $name, $conf );

		return $value;
	}

	/**
	 * initObjects( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.3.0
	 * @since 0.0.1
	 */
	protected function initObjects()
	{
		$this->_initObjectObjectManager();
		$this->_initObjectContent();
		$this->_initObjectCObj();
		$this->_initObjectTypoScriptService();
		$this->_initObjectPlugin();
	}

	/**
	 * initialise( ) :
	 *
	 * @return void
	 * @access protected
	 * @version 0.10.1
	 * @since 0.0.1
	 */
	protected function initialise()
	{
		$this->initObjects();
	}

}
