<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\SessionFeUserUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * AbstractController for the xBlog xtension
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.7
 * @since 0.0.1
 */
class AbstractController extends AbstractBaseController
{

	/**
	 * @var ContentObjectRenderer
	 */
	protected $contentObject;

	/**
	 * @var string Extension key
	 */
	public $extKey = 'xblog';

	/**
	 * @var string parameter pointer
	 */
	protected $paramPointer;

	/**
	 * @var array register from TypoScript
	 */
	protected $register;

	/**
	 * _debugRegister( ) :
	 *
	 * @param string $value
	 * @param int $len 
	 * @return string
	 * @access private
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _crop( $value, $len = 50 )
	{
		if( strlen( $value ) <= $len )
		{
			return $value;
		}

		$value = substr( $value, 0, $len ) . '... (#' . strlen( $value ) . ' chars)';
		return $value;
	}

	/**
	 * _debugRegister( ) :
	 *
	 * @return string
	 * @access private
	 * @version 0.8.3
	 * @since 0.5.2
	 */
	private function _debugRegister()
	{
		$debug = $this->settings[ 'flexform' ][ 'pi' ][ 'debug' ][ 'register' ];

		if( !$debug )
		{
			return;
		}
		return $this->_promptRegister();
	}

	/**
	 * _debugSession( ) :
	 *
	 * @return string
	 * @access private
	 * @version 0.8.3
	 * @since 0.0.1
	 */
	private function _debugSession()
	{
		$debug = $this->settings[ 'flexform' ][ 'pi' ][ 'debug' ][ 'session' ];

		if( !$debug )
		{
			return;
		}
		return $this->_promptSessionData();
	}

	/**
	 * _displayEmptyListByParams
	 * 
	 * @return boolean
	 * @version 0.20.24
	 * @since 0.20.24
	 * @internal #t5547
	 */
	private function _displayEmptyListByParams()
	{
		$paramsPi8 = GeneralUtility::_GP( 'tx_xblog_pi8' );
		$swordPi8 = $paramsPi8[ 'sword' ];
		$paramsPi13 = GeneralUtility::_GP( 'tx_xblog_pi13' );
		$swordPi13 = $paramsPi13[ 'sword' ];
		unset( $paramsPi13[ '__referrer' ] );
		unset( $paramsPi13[ '__trustedProperties' ] );
		unset( $paramsPi13[ 'uid' ] );
		unset( $paramsPi13[ 'sword' ] );
		foreach( ( array ) $paramsPi13 as $key => $value )
		{
			if( !empty( $value ) )
			{
				continue;
			}
			unset( $paramsPi13[ $key ] );
		}
		//var_dump( __METHOD__, __LINE__, $paramsPi8, $paramsPi13 );

		switch( true )
		{
			case(!empty( $swordPi8 )):
			case(!empty( $swordPi13 )):
			case(!empty( $paramsPi13 )):
				return false;
			default:
				return true;
		}
	}

	/**
	 * _getXblogMainFirstCallView
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.24
	 * @since 0.20.24
	 */
	private function _getXblogMainFirstCallView( $xBlogMainRow )
	{
		$sheet = 'ctrl';
		$field = 'settings.flexform.pi1.ctrl.firstcallview';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$default = 'list';
		$value = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__, $default );
		return $value;
	}

	/**
	 * _promptMisingTargetXblogAndDie( ) :
	 *
	 * @param integer $uid uid of the target xBlog
	 * @param string $method current method
	 * @param integer $line current line
	 * @return string
	 * @access private
	 * @version 0.24.12
	 * @since 0.24.12
	 */
	private function _promptMisingTargetXblogAndDie( $uidTargetXblog, $method, $line )
	{
		$pid = $this->contentObject->data[ 'pid' ];
		$uid = $this->contentObject->data[ 'uid' ];

		$output = '<div style="background:red; color:white; font-weight:bold; padding:2em; text-align:center;">'
						. '  <h2 style="color:white; font-weight:bold;">'
						. '    ERROR: xBlog linked to a non existing xBlog'
						. '  </h2>'
						. '  <p style="color:white; font-weight:bold;">'
						. '    xBlog with the uid ' . $uid . ' on the page ' . $pid . ' links to an xBlog with the uid ' . $uidTargetXblog . ', <br />'
						. '    but an xBlog with these uid doesn\'t exist. <br />'
						. '    <br />'
						. '    Sorry for the trouble.<br />'
						. '    Error occurs @ ' . $method . '#' . $line . ''
						. '  </p>'
						. '</div>'
		;

		echo $output;
		$prompt = 'FATAL ERROR: xBlog linked to a non existing xBlog';
		throw new \RuntimeException( $prompt, 1615737276 );
	}

	/**
	 * _promptRegister( ) :
	 *
	 * @return string
	 * @access private
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _promptRegister()
	{
//var_dump( __METHOD__, __LINE__, $this->settings[ 'flexform' ][ 'pi' ][ 'debug' ][ 'register' ] );
		$register = ( array ) $GLOBALS[ 'TSFE' ]->register;
		ksort( $register );

		$output = null;

		foreach( $register as $key => $value )
		{
			$value = $this->_crop( $value );
			$output = $output
							. '<tr>'
							. '<td>'
							. $key
							. '</td>'
							. '<td>'
							. htmlentities( $value )
							. '</td>'
							. '</tr>'
			;
		}

		if( $output )
		{
			$output = ''
							. '<h2>'
							. 'Debug Register'
							. '</h2>'
							. '<table style="font-size:.85rem;">'
							. $output
							. '</table>'
			;
		}

		return $output;
	}

	/**
	 * _promptSessionData():
	 *
	 * @return string
	 * @access private
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _promptSessionData()
	{

		$arrSessionData = SessionFeUserUtility::Get( 'tx_xblog' );

		if( empty( $arrSessionData ) )
		{
			return ''
							. '<h2>'
							. 'Debug Sessiondata'
							. '</h2>'
							. '<p>'
							. 'Session is empty. '
							. '<span style="background:white;color:red;">'
							. 'It is recommended to load this page twice to see current session data.'
							. '</span>'
							. '</p>'
			;
		}

		$strSessionData = ''
						. '<h2>'
						. 'Debug Sessiondata'
						. '</h2>'
						. '<pre style="font-size:.85rem;">'
						. var_export( $arrSessionData, true )
						. '</pre>'
						. '<p>'
						. '<span style="background:white;color:red;">'
						. 'It is recommended to load this page twice to see current session data.'
						. '</span>'
						. '</p>'
		;

		//var_dump( __METHOD__, __LINE__, SessionFeUserUtility::Get( 'tx_xblog' ) );
		return $strSessionData;
	}

	/**
	 * _setSessionContinue
	 * 
	 * @param array $arrSessionData
	 * @param string $key
	 * @param string $value
	 * @return boolean
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	private function _setSessionContinue( &$arrSessionData, $key, $value ): bool
	{
		$pos = strpos( $key, '__' );
		switch( true )
		{
			case($pos === ( int ) 0):
				return true;
			case($value === NULL):
				unset( $arrSessionData[ $key ] );
				return true;
			default:
				return false;
		}
	}

	/**
	 * _actionIsSingle():
	 *
	 * @param		string		$showUid
	 * @return boolean		true, if param action is single
	 * @access protected
	 * @version 0.30.7
	 * @since 0.9.7
	 */
	protected function actionIsSingle( $showUid = 'showUid' )
	{
		$piVars = GeneralUtility::_GP( 'tx_xblog_pi1' );
//		if( empty( $piVars ) )
//		{
//			$piVars = GeneralUtility::_GP( 'tx_xblog_pi15' );
//		}
		//var_dump( __METHOD__, __LINE__, $showUid, $piVars );
		switch( true )
		{
			case(isset( $piVars[ $showUid ] )):
			case($piVars[ 'action' ] == 'single'):
				return true;
			case(!isset( $piVars[ 'action' ] )):
			default:
				return false;
		}
	}

	/**
	 * debugDatesheet( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.7.0
	 * @since 0.7.0
	 * @internal #i0036
	 */
	protected function debugDatesheet()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugForm( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.8.26
	 * @since 0.8.26
	 * @internal #i0115
	 */
	protected function debugForm()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugImage( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.7.0
	 * @since 0.7.0
	 * @internal #i0034
	 */
	protected function debugImage()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugList( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	protected function debugMain()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugMap( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.7.0
	 * @since 0.7.0
	 * @internal #i0037
	 */
	protected function debugMap()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugMedia( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.7.0
	 * @since 0.7.0
	 * @internal #i0035
	 */
	protected function debugMedia()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugMini( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	protected function debugMini()
	{
		$debugIsEnabled = $this->settings[ 'flexform' ][ 'pi' ][ 'debug' ][ 'register' ];

		if( !$debugIsEnabled )
		{
			return;
		}

		return $this->_promptRegister();
		//return $this->cObjGetSingle( $this->settings[ 'views' ][ 'list' ][ 'debug' ] );
	}

	/**
	 * debugPaging( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.8.3
	 * @since 0.0.1
	 */
	protected function debugPaging()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * debugSearch( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.7.0
	 * @since 0.7.0
	 * @internal #i0038
	 */
	protected function debugSearch()
	{
		$content = null;
		$content = $content
						. $this->_debugSession()
						. $this->_debugRegister()
		;
		return $content;
	}

	/**
	 * displayEmptyList
	 * 
	 * @param array $row Current row of the xBlogMain
	 * @return boolean
	 * @version 0.20.24
	 * @since 0.20.24
	 */
	protected function displayEmptyList( $row )
	{
		$firstCallView = $this->_getXblogMainFirstCallView( $row );
		switch( true )
		{
			case($firstCallView == 'empty'):
				return $this->_displayEmptyListByParams();
			case($firstCallView == 'list'):
			default:
				return false;
		}
	}

	/**
	 * getPointerFromSession( ) :
	 *
	 * @return integer	$pointer
	 * @internal Must correspond with PointerUserfunc::_getPointerFromSession
	 * @access private
	 * @version 0.20.15
	 * @since 0.3.0
	 */
	protected function getPointerFromSession()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$arrSessionData = SessionFeUserUtility::Get( 'tx_xblog' );

		$pointer = $arrSessionData[ $uid ][ 'pointer' ];
//		var_dump(__METHOD__, __LINE__, $arrSessionData, $arrSessionData[ $uid ], $pointer);
		return $pointer;
	}

	/**
	 * getSessionData
	 * 
	 * @param string $list_type: name of the current plugin like xblog_pi8
	 * @param int $uid: uid of the current plugin
	 * @return array
	 * @version 0.9.10
	 * @since 0.8.26
	 */
	protected function getSessionData( $list_type = null, $uid = null )
	{
		if( empty( $uid ) )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}

		if( empty( $list_type ) )
		{
			$plugin = 'tx_' . $this->contentObject->data[ 'list_type' ];
		}
		else
		{
			$plugin = 'tx_' . $list_type;
		}

		$sessionData = SessionFeUserUtility::Get( $plugin );
		return $sessionData[ $uid ];
	}

	/**
	 * evalTargetXblog( ) :
	 *
	 * @param integer $uid	uid of the target xBlog
	 * @param string $method current method
	 * @param integer $line current line
	 * @return string
	 * @access protected
	 * @version 0.24.12
	 * @since 0.24.12
	 */
	protected function evalTargetXblog( $uid, $method, $line )
	{
		if( $uid < 1 )
		{
			return;
		}
		$row = SqlUtility::getRecord( 'tt_content', $uid );
		if( $row )
		{
			return true;
		}
		$this->_promptMisingTargetXblogAndDie( $uid, $method, $line );
	}

	/**
	 * getPageIdOfForeignListview: Returns the uid of the listview page, if page
	 * 		isn't the current page
	 * 
	 * @return integer
	 * @version 0.21.8
	 * @since 0.21.8
	 */
	protected function getPageIdOfForeignListview()
	{
		$xBlogMainRow = $this->contentObject->data;
		$PageIdOfListview = $this->getXblogMainPageIdOfListview( $xBlogMainRow );

		switch( true )
		{
			case( empty( $PageIdOfListview ) ):
			case( $PageIdOfListview == $this->contentObject->data[ 'uid' ] ):
				return;
			default:
				// follow the workflow
				break;
		}


		$row = SqlUtility::getTtcontentRowByUid( $PageIdOfListview );
		$pid = $row[ 'pid' ];

		return $pid;
	}

	/**
	 * getXblogMainPageIdOfListview
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.21.8
	 * @since 0.21.8
	 */
	protected function getXblogMainPageIdOfListview( $xBlogMainRow )
	{
		$sheet = 'ctrl';
		$field = 'settings.flexform.pi1.ctrl.plugin.list';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$aspectratio = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		return $aspectratio;
	}

	/**
	 * _getXblogMainAspectratio
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.57
	 * @since 0.20.57
	 */
	protected function getXblogMainAspectratio( $xBlogMainRow )
	{
		$sheet = 'list';
		$field = 'settings.flexform.pi1.list.aspectratio';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$default = '';
		$value = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__, $default );
		return $value;
	}

	/**
	 * _getXblogMainBackgroundchange
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.53
	 * @since 0.10.1
	 */
	protected function getXblogMainBackgroundchange( $xBlogMainRow )
	{
		$sheet = 'list';
		$field = 'settings.flexform.pi1.list.backgroundchange';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$default = '';
		$value = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__, $default );
		return $value;
	}

	/**
	 * _getXblogMainLocaltable
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.14
	 * @since 0.10.1
	 */
	protected function getXblogMainLocaltable( $xBlogMainRow )
	{
		$sheet = 'table';
		$field = 'settings.flexform.pi.table.table';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$localtable = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		return $localtable;
	}

	/**
	 * _getXblogMainPages
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.14
	 * @since 0.10.1
	 */
	protected function getXblogMainPages( $xBlogMainRow )
	{
		return $xBlogMainRow[ 'pages' ];
	}

	/**
	 * getXblogMainSocialmediaServices
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.19
	 * @since 0.20.19
	 */
	protected function getXblogMainSocialmediaServices( $xBlogMainRow )
	{
		$sheet = 'socialmedia';
		$field = 'settings.flexform.pi1.socialmedia.services';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$default = '';
		$value = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__, $default );
		return $value;
	}

	/**
	 * getXblogMainResultsPerPage
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.15
	 * @since 0.20.15
	 */
	protected function getXblogMainResultsPerPage( $xBlogMainRow )
	{
		$sheet = 'table';
		$field = 'settings.flexform.pi.table.max';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$resultsPerPage = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		return $resultsPerPage;
	}

	/**
	 * _getXblogMainRecursive
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.14
	 * @since 0.10.1
	 */
	protected function getXblogMainRecursive( $xBlogMainRow )
	{
		return $xBlogMainRow[ 'recursive' ];
	}

	/**
	 * getXblogMainRow
	 * 
	 * @param string	$tab		flexform tab
	 * @param string	$field	flexform field
	 * @return array
	 * @version 0.20.14
	 * @since 0.20.14
	 */
	protected function getXblogMainRow( $tab, $field )
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		// 201203, dwildt, -
		//$xBlogMainUid = $this->getFlexformValue( __METHOD__, __LINE__, $xmlFlexform, $tab, $field );
		$xBlogMainUid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $tab, $field, __METHOD__, __LINE__ );
		$xBlogMainRow = SqlUtility::getRecord( 'tt_content', $xBlogMainUid );

		return $xBlogMainRow;
	}

	/**
	 * _getXblogMainShowUid
	 * 
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return string
	 * @version 0.20.53
	 * @since 0.10.1
	 */
	protected function getXblogMainShowUid( $xBlogMainRow )
	{
		$sheet = 'profile';
		$field = 'settings.flexform.pi1.profile.showUid';
		$xmlFlexform = $xBlogMainRow[ 'pi_flexform' ];
		$default = 'showUid';
		$value = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__, $default );
		return $value;
	}

	/**
	 * _isOnerecordonly( ) :
	 *
	 * @return boolean
	 * @access protected
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	protected function _isOnerecordonly()
	{

		$mode = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_record_mode' ];
		//var_dump( __METHOD__, __LINE__, $mode );

		switch( $mode )
		{
			case('first'):
			case('selected'):
				return true;
			case(''):
				return false;
			default:
				break;
		}

		$prompt = 'FATAL ERROR: $mode isn\'t defined: "' . $mode . '"';
		throw new \RuntimeException( $prompt, 1616147904 );
	}

	/**
	 * _isOnerecordonlyInSingleView( ) :
	 *
	 * @return boolean
	 * @access protected
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	protected function _isOnerecordonlyInSingleView()
	{
		if( !$this->_isOnerecordonly() )
		{
			return false;
		}

		$view = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_record_view' ];
		//var_dump( __METHOD__, __LINE__, $mode );

		switch( $view )
		{
			case('list'):
				return false;
			case('single'):
				return true;
			default:
				break;
		}

		$prompt = 'FATAL ERROR: $view isn\'t defined: "' . $view . '"';
		throw new \RuntimeException( $prompt, 1616150162 );
	}

	/**
	 * oddDivsAddDiv( ) : In the case of a column layout, the error may occur 
	 * that a closing div-tag is missing if the number of records does not suffice 
	 * to fill all the columns.
	 * This workaround counts the div-tags. And if they are odd it adds a 
	 * closing div-tag.
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.9.15
	 */
	protected function oddDivsAddDiv( $content )
	{
		$matches = [];

		$regExpr = '/<\/?\bdiv\b[^>]*>/m';
		preg_match_all( $regExpr, $content, $matches, PREG_SET_ORDER, 0 );

		$amountOfDivTags = count( array_keys( $matches ) );

		if( ($amountOfDivTags % 2) != 0 )
		{
			$content = $content . '</div>';
		}

		return $content;
	}

	/**
	 * setSessionData
	 * 
	 * @param array $pluginData: data of the current form
	 * @param array $sessionData: current session data
	 * @param string $list_type: name of the current plugin like xblog_pi8
	 * @param int $uid: uid of the current plugin
	 * @return array
	 * @version 0.20.17
	 * @since 0.8.26
	 * @internal #t5532
	 */
	protected function setSessionData( $pluginData, $sessionData, $list_type = null, $uid = null )
	{

		if( empty( $uid ) )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}

		if( empty( $list_type ) )
		{
			$plugin = 'tx_' . $this->contentObject->data[ 'list_type' ];
		}
		else
		{
			$plugin = 'tx_' . $list_type;
		}

		$arrSessionData = SessionFeUserUtility::Get( $plugin );

		foreach( ( array ) $pluginData as $key => $value )
		{
			if( $this->_setSessionContinue( $sessionData, $key, $value ) )
			{
				continue;
			}

			if( $value === '' )
			{
				if( isset( $arrSessionData[ $uid ][ $key ] ) )
				{
					unset( $arrSessionData[ $uid ][ $key ] );
				}
				if( empty( $arrSessionData[ $uid ] ) )
				{
					unset( $arrSessionData[ $uid ] );
				}
				continue;
				//var_dump( __METHOD__, __LINE__, $value );
			}
			$arrSessionData[ $uid ][ $key ] = $value;
		}
		SessionFeUserUtility::Set( $plugin, $arrSessionData );

		//SessionFeUserUtility::Set( $plugin, $sessionDataUid );

		return $sessionData;
	}

	/**
	 * singlePluginRequirement( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.30.7
	 * @since 0.0.1
	 * @internal #t4480
	 */
	protected function singlePluginRequirement()
	{
		$piVars = GeneralUtility::_GP( 'tx_xblog_pi1' );
//		if( empty( $piVars ) )
//		{
//			$piVars = GeneralUtility::_GP( 'tx_xblog_pi15' );
//		}
		$uid = $this->contentObject->data[ 'uid' ];

		//var_dump( __METHOD__, __LINE__, $piVars[ 'plugin' ], $uid );
		// 200102, dwildt, +
		switch( true )
		{
			case(!isset( $piVars[ 'plugin' ] )):
				return true;
			case(( int ) $piVars[ 'plugin' ] == $uid):
				return true;
			default:
				return false;
		}
	}

	/**
	 * displayInSingleView( ): 
	 *
	 * @return	boolean	
	 * @version 0.21.5
	 * @since 0.21.5
	 */
	protected function displayInSingleView()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$sheet = 'ctrl';
		$field = 'settings.flexform.pi1.ctrl.display';
		$default = 'listsingle';
		$displayMode = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__, $default );
		switch( true )
		{
			case($displayMode == "list"):
			case($displayMode == "notsingle"):
				return false;
			case(empty( $displayMode )):
			case($displayMode == "listsingle"):
				return true;
		}

		$prompt = 'FATAL ERROR: value for switch is not defined: "' . $displayMode . '"<br />'
						. __METHOD__ . '#' . __LINE__ . ' @virü3uhjdfs';
		die( $prompt );
	}

	/**
	 * uidOfDestXblogDoesMatch( ): 
	 *
	 * @return	boolean	
	 * @version 0.21.5
	 * @since 0.21.5
	 */
	protected function uidOfDestXblogDoesMatch()
	{
		$uidCurrentXblog = $this->contentObject->data[ 'uid' ];

		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$sheet = 'ctrl';
		$field = 'settings.flexform.pi1.ctrl.plugin.single';
		$uidDestXblog = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		if( empty( $uidDestXblog ) )
		{
			return true;
		}

		if( $uidCurrentXblog == $uidDestXblog )
		{
			return true;
		}

		return false;
	}

	/**
	 * unsetSession( ): 
	 *
	 * @param string	$plugin
	 * @return	string		user || ses
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	protected function unsetSession( $plugin )
	{
		$sessionData = [
		];
		SessionFeUserUtility::Set( $plugin, $sessionData );
	}

	/**
	 * urlWiUid( ) :
	 *
	 * @return string
	 * @access protected
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	protected function urlWiUid()
	{
		$args = $this->request->getArguments();
		$uid = ( int ) $args[ 'uid' ];
		//var_dump( __METHOD__, __LINE__, $uid );

		if( $uid )
		{
			return true;
		}
		return false;
	}

	/**
	 * viewAssignSessionData( ) :
	 *
	 * @return void
	 * @access protected
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	protected function viewAssignSessionData()
	{
		$pluginData = $this->request->getArguments();
		$sessionData = $this->getSessionData();
		$this->setSessionData( $pluginData, $sessionData );
		$sessionData = $this->getSessionData();

//		var_dump( __METHOD__, __LINE__, $pluginData );
//		var_dump( __METHOD__, __LINE__, $sessionData );
		foreach( ( array ) $sessionData as $key => $value )
		{
			$this->view->assign( $key, $value );
		}
	}

}
