<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class PagingUtility
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.10
 * @since 0.30.9
 */
class AbstractPaging extends AbstractController
{

	/**
	 * @var string	:	xblog_pi1 || xblog_pi3
	 */
	private $_listType;

	/**
	 * RenderPaging( ) :
	 *
	 * @param string	$position		: default | bottom | none | top
	 * @return string	HTML paging
	 * @access private
	 * @version 0.30.10
	 * @since 0.30.9
	 */
	public function RenderPaging( $position )
	{
		$this->_listType = $this->contentObject->data[ 'list_type' ];

		if( $this->_isOnerecordonly() )
		{
			return;
		}

		$tmplPaging = $this->_renderPagingGetTmplPaging();

		switch( true )
		{
			case($tmplPaging == 'none'):
			case(!$this->_renderPagingIsNeeded() ):
				return;
			case($tmplPaging == 'bottom' && $position == 'bottom'):
			case($tmplPaging == 'default' && ($position == 'top' || $position == 'bottom')):
			case($tmplPaging == 'top' && $position == 'top'):
				// follow the workflow
				break;
			default:
				return;
		}

		$this->_renderPagingPiVars();
		$paging = $this->_renderPagingHtml();

		return $paging;
	}

	/**
	 * _renderPagingCalcProperPointer
	 * 
	 * @internal Should correspond with PagingController::_renderPagingCalcProperPointer
	 * @return integer	pointer
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingCalcProperPointer()
	{
		$params = $this->_renderPagingParamsCleanedUp();

		if( empty( $params ) )
		{
			return null;
		}

// #t6183, 210626, dwildt
//		if( !isset( $params[ 'pointer' ] ) )
//		{
//			$pointer = $this->getPointerFromSession();
//			return $pointer;
//		}

		$pointer = ( int ) $params[ 'pointer' ];
		return $pointer;
	}

	/**
	 * _renderPagingGetTmplPaging
	 * 
	 * @return string	
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingGetTmplPaging()
	{
		switch( $this->_listType )
		{
			case('xblog_pi3'):
				$tmplPaging = 'default';
				break;
			case('xblog_pi1'):
//			case('xblog_pi15'):
			default:
				$tmplPaging = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi1.tmpl.paging' ];
				break;
		}

		return $tmplPaging;
	}

	/**
	 * _renderPagingHtml( ) :
	 *
	 * @return string	HTML paging
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingHtml()
	{
		static $paging = null;

		if( $paging[ $this->_listType ] )
		{
			return $paging[ $this->_listType ];
		}

		$this->_renderPagingHtmlRegister();
		$this->_renderPagingHtmlPointer();
		$this->_renderPagingHtmlInternal();
		$paging[ $this->_listType ] = $this->_renderPagingHtmlResult();

		//var_dump(__METHOD__, __LINE__, $this->settings[ 'paging' ]);
		//var_dump(__METHOD__, __LINE__, $this->_oPlugin->internal);

		return $paging[ $this->_listType ];
	}

	/**
	 * _renderPagingHtmlInternal( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingHtmlInternal()
	{
		switch( $this->_listType )
		{
			case('xblog_pi3'):
				$this->_oPlugin->internal[ 'dontLinkActivePage' ] = $this->settings[ 'paging' ][ 'init' ][ 'dontLinkActivePage' ];
				$this->_oPlugin->internal[ 'maxPages' ] = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi3.paging.resultsatatime' ];
				$this->_oPlugin->internal[ 'pagefloat' ] = $this->settings[ 'paging' ][ 'init' ][ 'pagefloat' ];
				$this->_oPlugin->internal[ 'res_count' ] = $GLOBALS[ 'TSFE' ]->register[ 'xBlogPi3DestXblogSelectCount' ];
				$this->_oPlugin->internal[ 'results_at_a_time' ] = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi.table.max' ];
				$this->_oPlugin->internal[ 'showFirstLast' ] = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi3.paging.showfirstlast' ];
				$this->_oPlugin->internal[ 'showRange' ] = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi3.paging.showrange' ];
				break;
			case('xblog_pi1'):
//			case('xblog_pi15'):
			default:
				$this->_oPlugin->internal[ 'dontLinkActivePage' ] = $this->settings[ 'paging' ][ 'init' ][ 'dontLinkActivePage' ];
				$this->_oPlugin->internal[ 'maxPages' ] = $this->settings[ 'paging' ][ 'init' ][ 'maxPages' ];
				$this->_oPlugin->internal[ 'pagefloat' ] = $this->settings[ 'paging' ][ 'init' ][ 'pagefloat' ];
				$this->_oPlugin->internal[ 'res_count' ] = $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'countItems' ] );
				$this->_oPlugin->internal[ 'results_at_a_time' ] = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi.table.max' ];
				$this->_oPlugin->internal[ 'showFirstLast' ] = $this->settings[ 'paging' ][ 'init' ][ 'showFirstLast' ];
				$this->_oPlugin->internal[ 'showRange' ] = $this->settings[ 'paging' ][ 'init' ][ 'showRange' ];
				break;
		}
	}

	/**
	 * _renderPagingHtmlPointer( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingHtmlPointer()
	{
		$pointer = $this->_renderPagingCalcProperPointer();
		$this->_oPlugin->piVars[ 'tx_xblog_pi1[pointer]' ] = $pointer;
	}

	/**
	 * _renderPagingHtmlResult( ) :
	 *
	 * @return string	HTML paging
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingHtmlResult()
	{
		switch( $this->_listType )
		{
			case('xblog_pi3'):
				$paging = $this->_oPlugin->pi_list_browseresults(
								$GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi3.paging.showresultcount' ]
								, $this->settings[ 'paging' ][ 'init' ][ 'tableParams' ]
								, $this->settings[ 'paging' ][ 'wrap' ]
								, 'tx_xblog_pi1[pointer]'
								, $this->settings[ 'paging' ][ 'init' ][ 'hscText' ]
								, $this->settings[ 'paging' ][ 'init' ][ 'forceOutput' ]
				);
				break;
			case('xblog_pi1'):
//			case('xblog_pi15'):
			default:
				$paging = $this->_oPlugin->pi_list_browseresults(
								$this->settings[ 'paging' ][ 'init' ][ 'showResultCount' ]
								, $this->settings[ 'paging' ][ 'init' ][ 'tableParams' ]
								, $this->_renderPagingHtmlWrap()
								, 'tx_xblog_pi1[pointer]'
								, $this->settings[ 'paging' ][ 'init' ][ 'hscText' ]
								, $this->settings[ 'paging' ][ 'init' ][ 'forceOutput' ]
				);
				break;
		}

		return $paging;
	}

	/**
	 * _renderPagingHtmlRegister( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingHtmlRegister()
	{
		$GLOBALS[ 'TSFE' ]->register[ 'xBlogPiTable' ] = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi.table.table' ];
	}

	/**
	 * _renderPagingHtmlWrap( ) :
	 *
	 * @return array
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingHtmlWrap()
	{
		return [
				'activeLinkWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'activeLinkWrap' ]
				, 'browseBoxWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'browseBoxWrap' ]
				, 'browseLinksWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'browseLinksWrap' ]
				, 'disabledLinkWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'disabledLinkWrap' ]
				, 'inactiveLinkWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'inactiveLinkWrap' ]
				, 'showResultsNumbersWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'showResultsNumbersWrap' ]
				, 'showResultsWrap' => $this->settings[ 'paging' ][ 'wrap' ][ 'showResultsWrap' ]
		];
	}

	/**
	 * _renderPagingIsNeeded( ) :
	 *
	 * @return	boolean
	 * @access private
	 * @version 0.30.9
	 * @since 0.30.9
	 */
	private function _renderPagingIsNeeded()
	{
		switch( $this->_listType )
		{
			case('xblog_pi3'):
				$res_count = $GLOBALS[ 'TSFE' ]->register[ 'xBlogPi3DestXblogSelectCount' ];
				break;
			case('xblog_pi1'):
//			case('xblog_pi15'):
			default:
				// next line MUST CALLED AFTER $this->_getContent()
				$res_count = $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'countItems' ] );
				break;
		}
		$results_at_a_time = $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi.table.max' ];

		if( $res_count > $results_at_a_time )
		{
			return true;
		}

		return false;
	}

	/**
	 * _renderPagingParamsCleanedUp
	 * 
	 * @internal Should correspond with PagingController::_renderPagingParamsCleanedUp
	 * @return array	params without action=list and controller=Main
	 * @version 0.30.7
	 * @since 0.20.18
	 */
	private function _renderPagingParamsCleanedUp()
	{
		$params = GeneralUtility::_GP( 'tx_xblog_pi1' );
//		if( empty( $params ) )
//		{
//			$params = GeneralUtility::_GP( 'tx_xblog_pi15' );
//		}

// #t6183, 210626, dwildt
//		if( isset( $params[ 'action' ] ) )
//		{
//			if( $params[ 'action' ] == 'list' )
//			{
//				unset( $params[ 'action' ] );
//			}
//		}
//		if( isset( $params[ 'controller' ] ) )
//		{
//			if( $params[ 'controller' ] == 'Main' )
//			{
//				unset( $params[ 'controller' ] );
//			}
//			if( $params[ 'controller' ] == 'MainUncached' )
//			{
//				unset( $params[ 'controller' ] );
//			}
//		}

		return $params;
	}

	/**
	 * _renderPagingPiVars
	 * 
	 * @return void
	 * @version 0.30.10
	 * @since 0.30.10
	 */
	private function _renderPagingPiVars()
	{
		if( !isset( $GLOBALS[ 'TSFE' ]->register[ 'pi13__params' ] ) )
		{
			return;
		}
		$params = explode( '&', $GLOBALS[ 'TSFE' ]->register[ 'pi13__params' ] );
		foreach( $params as $param )
		{
			list($key, $value) = explode( '=', $param );
			$this->_oPlugin->piVars[ $key ] = $value;
		}
		$this->_oPlugin->piVars[ 'tx_xblog_pi1' ][ 'action' ] = 'result';
		$this->_oPlugin->piVars[ 'tx_xblog_pi1' ][ 'controller' ] = 'Main';
		$this->_oPlugin->piVars = array_filter( $this->_oPlugin->piVars );
	}

}
