<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\Controller;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class PagingUtility
 *
 * @package			TYPO3
 * @subpackage	xblog
 * @author			Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version			0.30.17
 * @since				0.30.17
 */
class AbstractTablelayout extends AbstractPaging
{

	/**
	 * TablelayoutClass
	 * 
	 * @return	string	$class
	 * @version	0.30.17
	 * @since		0.30.17
	 */
	protected function TablelayoutClass()
	{
		$class = '';

		if( $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tablelayout_hoverstate' ] )
		{
			$class = $class . ' hover';
		}
		if( !$GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tablelayout_stripes' ] )
		{
			$class = $class . ' unstriped';
		}
		if( $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tablelayout_scrolling' ] )
		{
			$class = $class . ' table-scroll';
		}
		if(!$class){
			return;
		}
		$class = trim($class);
		$class = ' class="' . $class .'"';
		return $class;
	}

	/**
	 * TablelayoutGetRows
	 * 
	 * @return	array		$rows
	 * @version	0.30.17
	 * @since		0.30.17
	 */
	protected function TablelayoutGetRows()
	{
		$xblogUid = $this->contentObject->data[ 'uid' ];
		$rows = $this->tablelayoutRepository->getRowsByXblogUid( $xblogUid );
		$this->_tablelayoutGetRowsPromptEmpty( $rows );
		return $rows;
	}

	/**
	 * TablelayoutThead
	 * 
	 * @param		array		$rows
	 * @return	void
	 * @version	0.30.17
	 * @since		0.30.17
	 */
	protected function TablelayoutThead( $rows )
	{
		$arrTh = [];
		foreach( $rows as $row )
		{
			$width = $this->TablelayoutTheadWidth( $row );
			$arrTh[] = '      <th' . $width . '>' . $row[ 'title' ] . '</th>';
		}
		$thead = '
  <thead>
    <tr>
' . implode( PHP_EOL, $arrTh ) . '
    </tr>
  </thead>
';
		return $thead;
//		var_dump( __METHOD__, __LINE__, $thead, $rows );
//		die();
	}

	/**
	 * TablelayoutThead
	 * 
	 * @param		array		$rows
	 * @return	void
	 * @version	0.30.17
	 * @since		0.30.17
	 */
	protected function TablelayoutTheadWidth( $row )
	{
		if( empty( $row[ 'width' ] ) )
		{
			return;
		}
		$width = ' width="' . $row[ 'width' ] . '"';
		return $width;
	}

	/**
	 * _tablelayoutGetRowsPromptEmpty
	 * 
	 * @param		array		$rows
	 * @return	void
	 * @version	0.30.17
	 * @since		0.30.17
	 */
	private function _tablelayoutGetRowsPromptEmpty( $rows )
	{
		if( !empty( $rows ) )
		{
			return;
		}

		$prompt = 'xBlog template layout is "table". '
						. 'But any column hasn\'t any setup!'
		;
		var_dump( __METHOD__, __LINE__, $prompt );
	}

}
