<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\RegisterUtility;
use Netzmacher\Xblog\Utility\SessionForFormsUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering the search form
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.8
 * @since 0.8.26
 */
class FormController extends AbstractController
{

	/**
	 * @var string path to the language file
	 */
	public $sLanguageFilePath = 'Resources/Private/Language/Form.xlf';

	/**
	 * _flexformValuesToRegister( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	private function _flexformValuesToRegister()
	{
		$this->_flexformValuesToRegisterPi1();
		$this->_flexformValuesToRegisterPi13();

//		var_dump(__METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register);
//		die();
	}

	/**
	 * _flexformValuesToRegisterPi1( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	private function _flexformValuesToRegisterPi1()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$tab = 'ctrl';
		$field = 'settings.flexform.pi13.ctrl.plugin.list';
		$uid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $tab, $field, __METHOD__, __LINE__ );
		$this->evalTargetXblog( $uid, __METHOD__, __LINE__ );

		if( $uid < 1 )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}
		$plugin = 'pi1';
		RegisterUtility::setRegister( $uid, $plugin );
	}

	/**
	 * _flexformValuesToRegisterPi12( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.30.7
	 * @since 0.22.0
	 */
	private function _flexformValuesToRegisterPi13()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$plugin = 'pi13';
		RegisterUtility::setRegister( $uid, $plugin );

		$GLOBALS[ 'TSFE' ]->register[ 'pi13__pi1_uid' ] = $GLOBALS[ 'TSFE' ]->register[ 'pi1__uid' ];
		$GLOBALS[ 'TSFE' ]->register[ 'pi13__pi1_pid' ] = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pid' ];
		$GLOBALS[ 'TSFE' ]->register[ 'pi13__pi1_table' ] = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_table' ];
	}

	/**
	 * _getFlexformFormRows
	 * 
	 * @param string localtable
	 * @param string list of pages
	 * @param integer depth
	 * @return object area
	 * @version 0.9.8
	 * @since 0.8.26
	 */
	private function _getFlexformFormRows( $localtable, $pages, $depth )
	{
		$elements = NULL;

		$xblogUid = $this->contentObject->data[ 'uid' ];

		$rows = $this->formRepository->getRowsByXblogUid( $xblogUid );

		foreach( ( array ) $rows as $row )
		{
			$arrRowUids[] = $row[ 'uid' ];
		}

		if( !is_array( $arrRowUids ) )
		{
			return;
		}

		foreach( ( array ) $arrRowUids as $rowUid )
		{
			$elements[ $rowUid ] = $this->elementRepository->GetElementsByRow( $rowUid, $localtable, $pages, $depth );
		}

		return $elements;
	}

	/**
	 * _getFormRows( ) :
	 *
	 * @param		array		$xBlogMainRow		Row of a main xBlog
	 * @return	array		$formRows
	 * @access	private
	 * @version 0.30.8
	 * @since		0.30.8
	 */
	private function _getFormRows( $xBlogMainRow )
	{
		$localtable = $this->getXblogMainLocaltable( $xBlogMainRow );
		$pages = $this->getXblogMainPages( $xBlogMainRow );
		$recursive = $this->getXblogMainRecursive( $xBlogMainRow );
		$formRows = $this->_getFlexformFormRows( $localtable, $pages, $recursive );

		return $formRows;
	}

	/**
	 * _getKeysFromRowsColumns( ) : Get the name of the columns of the form rows
	 *
	 * @param		array		$xBlogMainRow		Rows of a main xBlog
	 * @return	array		$arrKeysOfColumns
	 * @access	private
	 * @version 0.30.8
	 * @since		0.30.8
	 */
	private function _getKeysFromRowsColumns( $formRows )
	{
		$arrKeysOfColumns = [];
		foreach( $formRows as $formColumn )
		{
			foreach( $formColumn as $values )
			{
				$arrKeysOfColumns[] = $values[ 'name' ];
			}
		}
		// Remove empty keys
		$arrKeysOfColumns = array_filter( $arrKeysOfColumns );

		return $arrKeysOfColumns;
	}

	/**
	 * _getSessionUid
	 * 
	 * @return	integer	$sessionUid
	 * @version 0.30.10
	 * @since		0.30.10
	 */
	private function _getSessionUid()
	{
		if( !empty( $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi13.handon.plugin' ] ) )
		{
			return $GLOBALS[ 'TSFE' ]->register[ 'settings.flexform.pi13.handon.plugin' ];
		}

		return $data = $this->contentObject->data[ 'uid' ];
	}

	/**
	 * _initInCaseOfParamByGET( ) : Sets the GET param uid for the xBlogForm
	 *
	 * @param array		$xBlogMainRow		Rows of a main xBlog
	 * @return void
	 * @access private
	 * @version 0.30.10
	 * @since 0.30.8
	 */
	private function _initInCaseOfParamByGET( $formRows )
	{
		$paramsPi13 = GeneralUtility::_GP( 'tx_xblog_pi13' );
		if( !is_array( $paramsPi13 ) )
		{
			return;
		}

		$this->_initPostParam( $formRows, $paramsPi13 );
		$this->_initRegisterParam();
	}

	/**
	 * _initPostParam( ) :	Sets POST variables. They are needed for empty the form.
	 * 											They are needed by _initRegisterParam().
	 *
	 * @param array		$formRows		:	Rows of a main xBlog
	 * @param array		$paramsPi13	:	GET params of tx_blog_pi13
	 * @return void
	 * @access private
	 * @version 0.30.10
	 * @since 0.30.10
	 */
	private function _initPostParam( $formRows, $paramsPi13 )
	{
		$matchIsTrue = false;

		$formKeys = $this->_getKeysFromRowsColumns( $formRows );
		foreach( array_keys( $paramsPi13 ) as $param )
		{
			if( !in_array( $param, $formKeys ) )
			{
				continue;
			}
			$value = $paramsPi13[ $param ];
			$_POST[ 'tx_xblog_pi13' ][ $param ] = $value;
			$matchIsTrue = true;
		}

		if( $matchIsTrue )
		{
			$uid = $this->contentObject->data[ 'uid' ];
			$_POST[ 'tx_xblog_pi13' ][ 'uid' ] = $uid;
		}
	}

	/**
	 * _initRegsiterParam( ): registers additionalParams. xBlog needs it for a proper paging.
	 *
	 * @return void
	 * @access private
	 * @version 0.30.10
	 * @since 0.30.10
	 */
	private function _initRegisterParam()
	{
		$additionalParams = false;
		unset( $GLOBALS[ 'TSFE' ]->register[ 'pi13__params' ] );

		if( !isset( $_POST[ 'tx_xblog_pi13' ] ) )
		{
			return;
		}
		$arrParams = $_POST[ 'tx_xblog_pi13' ];
		unset( $arrParams[ '__referrer' ] );
		unset( $arrParams[ '__trustedProperties' ] );
		$arrParams = array_filter( $arrParams );

		foreach( $arrParams as $key => $value )
		{
			$additionalParams = $additionalParams . '&tx_xblog_pi13[' . $key . ']=' . $value;
		}

		$GLOBALS[ 'TSFE' ]->register[ 'pi13__params' ] = $additionalParams;
	}

	/**
	 * _setSwordFromPi8
	 * 
	 * @return string
	 * @version 0.9.10
	 * @since 0.9.10
	 * @internal #t4734
	 */
	private function _setSwordFromPi8( $sessionData )
	{
		$pluginPi8Data = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'tx_xblog_pi8' );

		if( empty( $pluginPi8Data ) )
		{
			return $sessionData;
		}

		$data = $this->contentObject->data;
		$formUid = $data[ 'uid' ];
		$aSword[ 'sword' ] = $pluginPi8Data[ 'sword' ];

		$this->_setSwordFromPiSession( $aSword, $formUid );
		$this->_setSwordFromPi8Session( $pluginPi8Data );

		$sessionData = $this->getSessionData( 'xblog', $formUid );

		$xBlogPi8Handon = $pluginPi8Data[ 'xBlogPi8Handon' ];
		switch( true )
		{
			case( empty( $xBlogPi8Handon ) ):
			case( $xBlogPi8Handon != $formUid ):
				return $sessionData;
		}

		$sessionData[ 'sword' ] = $pluginPi8Data[ 'sword' ];
		return $sessionData;
	}

	/**
	 * _setSwordFromPi8Session
	 * 
	 * @param array $pluginData GET/POST-data 
	 * @return void
	 * @version 0.9.10
	 * @since 0.9.10
	 * @internal #t4734
	 */
	private function _setSwordFromPi8Session( $pluginData )
	{
		$uid = $pluginData[ 'uid' ];
		$sessionData = $this->getSessionData( 'xblog_pi8', $uid );
		$this->setSessionData( $pluginData, $sessionData, 'xblog_pi8', $uid );
	}

	/**
	 * _setSwordFromPiSession
	 * 
	 * @param array $aSword array with the element sword
	 * @param int $uid Uid of the xBlogForm 
	 * @return void
	 * @version 0.9.10
	 * @since 0.9.10
	 * @internal #t4734
	 */
	private function _setSwordFromPiSession( $aSword, $formUid )
	{
		$sessionData = $this->getSessionData( 'xblog', $formUid );
		$this->setSessionData( $aSword, $sessionData, 'xblog', $formUid );
	}

	/**
	 * _viewAssignList( ) :
	 *
	 * @param array		$xBlogMainRow		Row of a main xBlog
	 * @return void
	 * @access private
	 * @version 0.30.8
	 * @since 0.8.26
	 */
	private function _viewAssignList( $xBlogMainRow )
	{
		$formRows = $this->_getFormRows( $xBlogMainRow );
		if( empty( $formRows ) )
		{
			return;
		}

		$this->_initInCaseOfParamByGET( $formRows );

		$this->cObjGetSingle( $this->settings[ 'views' ][ 'list' ][ 'register' ] );

		// :TODO: #t4439
		//$this->cObjGetSingle( $this->settings[ 'views' ][ 'list' ][ 'session' ][ 'setForm' ] );

		$sessionUid = $this->_getSessionUid();
//		var_dump( __METHOD__, __LINE__, $sessionUid );
		SessionForFormsUtility::setValues( $sessionUid );
		$sessionData = SessionForFormsUtility::getValues( $sessionUid );
		$sessionData = $this->_setSwordFromPi8( $sessionData );

		$this->view->assignMultiple(
						array(
								'formRows' => $formRows
								, 'pageUid' => $GLOBALS[ 'TSFE' ]->register[ 'pi13__pi1_pid' ]
								, 'sessionData' => $sessionData
//								, 'tt_content' => $data
//								, 'debug' => $this->debugSearch()
						)
		);
		$this->viewAssignSessionData();
	}

	/**
	 * listAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 0.20.14
	 * @since 0.8.26
	 *
	 */
	public function listAction()
	{
		$this->initialise();

		$this->_flexformValuesToRegister();

		$xBlogMainRow = $this->getXblogMainRow( 'ctrl', 'settings.flexform.pi13.ctrl.plugin.list' );
		$showUid = $this->getXblogMainShowUid( $xBlogMainRow );

		if( $this->actionIsSingle( $showUid ) )
		{
			return;
		}
		$this->_viewAssignList( $xBlogMainRow );
	}

}
