<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FeUserUtility;
use Netzmacher\Xblog\Utility\LanguageUtility;
use Netzmacher\Xblog\Utility\RegisterUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\UriUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering records
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.7
 * @since 0.0.1
 */
class MainController extends AbstractTablelayout
{

  /**
   * @var string path to the language file
   */
  public $sLanguageFilePath = 'Resources/Private/Language/Main.xlf';

  /**
   * @var string
   */
  private static $_LLLpath = 'LLL:EXT:xblog/Resources/Private/Language/Main.xlf:';

  /**
   * _extDependency( ) :
   *
   * @return void
   * @access private
   * @version 0.30.4
   * @since 0.10.6
   * @internal #t5317
   */
  private function _extDependency()
  {
    $prompt = ''
      . $this->_extDependencyCopyright()
      . $this->_extDependencyRxShariff()
    ;
    return $prompt;
  }

  /**
   * _extDependencyCopyright( ) :
   *
   * @return void
   * @access private
   * @version 0.30.4
   * @since 0.30.4
   */
  private function _extDependencyCopyright()
  {
    switch (true)
    {
      case( \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('copyright') ):
        $prompt = $this->_extDependencyCopyrightTyposcript();
        return $prompt;
      case(!\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('copyright') ):
      default:
        $prompt = 'The extension Copyright (copyright) is needed. Please install this extension.<br />';
        return $prompt;
    }
  }

  /**
   * _extDependencyCopyrightTyposcript( ) :
   *
   * @return void
   * @access private
   * @version 0.30.4
   * @since 0.30.4
   */
  private function _extDependencyCopyrightTyposcript()
  {
    $tsPath                              = 'lib.contentElement.partialRootPaths.21';
    $neededPartial                       = 'EXT:copyright/Resources/Private/Partials/';
    $libContentelementPartialrootpaths21 = $GLOBALS['TSFE']->tmpl->setup['lib.']['contentElement.']['partialRootPaths.']['21'];
    if ($libContentelementPartialrootpaths21 == $neededPartial)
    {
      return;
    }
    $prompt = 'The TypoScript template of the extension Copyright (copyright) is missing.<br />'
      . 'Please include the static template.<br />'
      . '<br />'
      . 'If static tempate is included, you will have this proper configuration:<br />'
      . $tsPath . ' = ' . $neededPartial . '<br />'
    ;
    return $prompt;
  }

  /**
   * _extDependencyRxShariff( ) :
   *
   * @return void
   * @access private
   * @version 0.20.19
   * @since 0.20.19
   * @internal #t5537
   */
  private function _extDependencyRxShariff()
  {
    $xBlogMainRow                 = $this->contentObject->data;
    $xBlogMainSocialmediaServices = $this->getXblogMainSocialmediaServices($xBlogMainRow);
    //var_dump( __METHOD__, __LINE__, $position, $xBlogMainSocialmediaServices );

    if (empty($xBlogMainSocialmediaServices))
    {
      return;
    }


    switch (true)
    {
      case( \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('rx_shariff') ):
        return;
      case(!\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('rx_shariff') ):
      default:
        $prompt = 'You are using the socialmedia property of the xBlogMain.<br />'
          . 'Heise Shariff (rx_shariff) is needed. Please install this extension.<br />'
          . 'Or disable the socialmedia property.<br />'
        ;
        return $prompt;
    }
  }

  /**
   * _flexformValuesToRegister( ) :
   *
   * @return void
   * @access private
   * @version 0.23.0
   * @since 0.23.0
   */
  private function _flexformValuesToRegister()
  {
    $this->_flexformValuesToRegisterPi1();
//		var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register );
//		die();
  }

  /**
   * _flexformValuesToRegisterPi1( ) :
   *
   * @return void
   * @access private
   * @version 0.30.7
   * @since 0.23.0
   */
  private function _flexformValuesToRegisterPi1()
  {
    $uid    = $this->contentObject->data['uid'];
    $plugin = 'pi1';
    RegisterUtility::setRegister($uid, $plugin);

    $this->_flexformValuesToRegisterPi1PidXblogList();
  }

  /**
   * _flexformValuesToRegisterPi1PidXblogList( ) :
   *
   * @return void
   * @access private
   * @version 0.30.7
   * @since 0.30.7
   */
  private function _flexformValuesToRegisterPi1PidXblogList()
  {
    $uidXblogList = $GLOBALS['TSFE']->register['settings.flexform.pi1.ctrl.plugin.list'];
    switch (true)
    {
      case(empty($uidXblogList)):
        // This xBlog
        $GLOBALS['TSFE']->register['pi1__pidXblogList'] = $this->contentObject->data['pid'];
        break;
      case($uidXblogList >= 1):
        // Another xBlog
        $row                                            = SqlUtility::getTtcontentRowByUid($uidXblogList);
        $GLOBALS['TSFE']->register['pi1__pidXblogList'] = $row['pid'];
        break;
      default:
        $prompt                                         = 'Undefined key in switch: "' . $uidXblogList . '" @' . __METHOD__ . '#' . __LINE__;
        die($prompt);
    }
  }

  /**
   * _getContent( ) :
   *
   * @return string
   * @access private
   * @version 0.20.24
   * @since 0.20.24
   * @internal #t5547
   */
  private function _getContent()
  {
    switch (true)
    {
      case($this->displayEmptyList($this->contentObject->data)):
        $content = LanguageUtility::getLabelInContextofBEOrFe(self::$_LLLpath . 'content.displayEmptyList');
        break;
      default:
        $content = $this->cObjGetSingle($this->settings['views']['list']['content']);
        $content = $this->oddDivsAddDiv($content);
        break;
    }
    return $content;
  }

  /**
   * _getXBlogTeaserUidsFromSession
   * 
   * @return void
   * @version 0.23.8
   * @since 0.23.6
   */
  private function _getXBlogTeaserUidsFromSession()
  {
    $sessionUid                                   = 'xBlogTeaser';
    $sessionData                                  = $this->getSessionData('xblog', $sessionUid);
    $GLOBALS['TSFE']->register['xBlogTeaserUids'] = $sessionData['xBlogTeaserUids'][$this->contentObject->data['pid']];
  }

  /**
   * _initCObj( ) :
   *
   * @return void
   * @access private
   * @version 0.0.5
   * @since 0.0.5
   */
  private function _initCObj()
  {
    $this->_oPlugin->cObj = $this->contentObject;
  }

  /**
   * _isTablelayout( ) :
   *
   * @return boolean
   * @access private
   * @version 0.30.17
   * @since		0.30.17
   */
  private function _isTablelayout()
  {
    $externalTemplate = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate'];
    if ($externalTemplate != 'tablelayout')
    {
      return false;
    }
    return true;
  }

  /**
   * _piWrapClassList( ) :
   * 
   * @param string showUid	like: showUid, calUid, newsUid, ...
   * @param string mode			list || single
   *
   * @return string
   * @access private
   * @version 0.20.53
   * @since 0.20.53
   */
  private function _piWrapClass($showUid, $mode)
  {
    $mainClass = $this->cObjGetSingle($this->settings['general']['constants']['css']['list']['cssPlugin']);

    return ''
      . $mainClass . ' '
      . $mode
      . $this->_piWrapClassProfile($showUid) // Is needed for Javascript
//						. $this->_piWrapClassBackgroundChange()  // 201124, dwildt: isn't needed any longer. :ToDo: remove line and method
    ;
  }

  /**
   * _piWrapClassBackgroundChange( ) :
   * 
   * @return string
   * @access private
   * @version 0.20.59
   * @since 0.20.59
   */
  private function _piWrapClassBackgroundChange()
  {
    $backgroundchange = $this->getXblogMainBackgroundchange($this->contentObject->data);

    if (empty($backgroundchange))
    {
      return;
    }

    return ' backgroundchange';
  }

  /**
   * _piWrapClassList( ) :
   * 
   * @param string showUid	like: showUid, calUid, newsUid, ...
   *
   * @return string
   * @access private
   * @version 0.20.59
   * @since 0.20.59
   */
  private function _piWrapClassProfile($showUid)
  {
    if (empty($showUid))
    {
      return;
    }

    // Remove Uid from the showUid
    $length  = strlen($showUid) - strlen('Uid');
    $profile = ' ' . substr($showUid, 0, $length);

    return $profile;
  }

  /**
   * _redirectToSinglepublish
   * 
   * @return  void
   * @version 0.31.3
   * @since   0.31.3
   */
  private function _redirectToSinglepublish()
  {
    $showUidParam = $this->getXblogMainShowUid($this->contentObject->data);
    $showUidValue = (int) $_GET['tx_xblog_pi1'][$showUidParam];
    $action       = 'singlepublish';
    $controller   = null;
    $extension    = null;
    $parameter    = [
      $showUidParam => $showUidValue
    ];

    $this->redirect($action, $controller, $extension, $parameter);
  }

  /**
   * _removeXBlogTeaserUidsFromSession
   * 
   * @return void
   * @version 0.23.6
   * @since 0.23.6
   */
  private function _removeXBlogTeaserUidsFromSession()
  {
    unset($GLOBALS['TSFE']->register['xBlogTeaserUids']);
    $sessionUid      = 'xBlogTeaser';
    $sessionData     = $this->getSessionData('xblog', $sessionUid);
    $xBlogTeaserUids = [
      'xBlogTeaserUids' => [
        $this->contentObject->data['pid'] => ''
      ]
    ];
    $this->setSessionData($xBlogTeaserUids, $sessionData, 'xblog', $sessionUid);
  }

  /**
   * _renderJavascriptRegisterAspectratio( ) :
   * 
   * @param string $aspectratio
   *
   * @return void
   * @access private
   * @version 0.20.57
   * @since 0.20.57
   */
  private function _renderJavascriptRegisterAspectratio($aspectratio)
  {
    return;

    // #v0451, 201120, dwildt: lines below are working,
    // but register can't called Configuration/TypoScript/Base/page/jsFooterInline.ts
//		if( !$aspectratio )
//		{
//			return;
//		}
//		
//		$register = 'jss-aspectratio-' . $aspectratio;
//		$GLOBALS[ 'TSFE' ]->register[ $register ] = 1;
//
//		//var_dump( __METHOD__, __LINE__, $register );
//		//die();
  }

  /**
   * _renderJavascriptXblogBubble( ) : Prompts variables for JavaScript. See
   * 		* Resources/Public/JavaScript/xBlogBubble.js
   * 
   * @return string $script
   * @access private
   * @version 0.20.57
   * @since 0.20.57
   */
  private function _renderJavascriptXblogBubble()
  {
    $bgcolor  = $this->settings['general']['constants']['xblogbubble']['colors']['background'];
    $radius   = $this->settings['general']['constants']['xblogbubble']['vars']['radius'];
    $txtcolor = $this->settings['general']['constants']['xblogbubble']['colors']['text'];
    $distance = $this->settings['general']['constants']['xblogbubble']['vars']['distance'];

    // :ToDo: 201124, dwildt: run code below only, if xBlog template Bubble is selected

    $script = '  
			<script>
				var ' . $this->_renderJavascriptXblogBubbleColors('xBlogBubbleBgColors', $bgcolor) . ';
				var ' . $this->_renderJavascriptXblogBubbleColors('xBlogBubbleColors', $txtcolor) . ';
				var xBlogBubbleDistance = ' . $distance . ';
				var xBlogBubbleRadius   = ' . $radius . ';
			</script>
						';

//		var_dump( __METHOD__, __LINE__, 'Please inspect the HTML source code for looking the script!', $script );
//		die();

    return $script;
  }

  /**
   * _renderJavascriptXblogBubble( ) :
   * 
   * @param string $param			name of the variable
   * @param array $arrColors	array with colors
   *
   * @return string $paramColor variable = [color1, color2, ...]
   * @access private
   * @version 0.24.11
   * @since 0.20.57
   */
  private function _renderJavascriptXblogBubbleColors($param, $arrColors)
  {
    if (!is_array($arrColors))
    {
      return;
    }

    foreach ($arrColors AS $color)
    {
      $_arrColors[] = $color;
    }
    // Remove empty items
    $_arrColors = array_filter($_arrColors);
    $jsonColors = json_encode($_arrColors);

    $paramColor = $param . ' = ' . $jsonColors;

    return $paramColor;
  }

  /**
   * _resetRegisterList
   * 
   * @return void
   * @internal #i0044
   * @version 0.20.41
   * @since 0.20.41
   */
  private function _resetRegisterList()
  {
    unset($GLOBALS['TSFE']->register['recordCycle']);
  }

  /**
   * _setPointerInSession
   * 
   * @return void
   * @version 0.21.5
   * @since 0.21.5
   */
  private function _resetRegisterSingle()
  {
    unset($GLOBALS['TSFE']->register['xBlogMainSingle']);
  }

  /**
   * _setController( ) : 
   *
   * @return void
   * @access public
   * @version 0.30.7
   * @since 0.30.7
   *
   */
  private function _setController()
  {
    $uidXBlogSingle  = $GLOBALS['TSFE']->register['settings.flexform.pi1.ctrl.plugin.single'];
    $uidXBlogCurrent = $this->contentObject->data['uid'];

    if (empty($uidXBlogSingle))
    {
      $uidXBlogSingle = $uidXBlogCurrent;
    }

    switch (true)
    {
      case($uidXBlogSingle == -1):
        // A third party plugin should used for the single view
        return;
//			case($uidXBlogCurrent == $uidXBlogSingle):
//				// The current xBlog should used for the single view
//				$this->_setControllerPi15( $uidXBlogSingle );
//				return;
//			default:
//				$this->_setControllerPi15( $uidXBlogSingle );
//				return;
    }
  }

//	/**
//	 * _setControllerPi15( ) : 
//	 *
//	 * @param integer	$uidXBlogSingle
//	 * @return void
//	 * @access public
//	 * @version 0.30.7
//	 * @since 0.30.7
//	 *
//	 */
//	private function _setControllerPi15( $uidXBlogSingle )
//	{
//		$row = SqlUtility::getTtcontentRowByUid( $uidXBlogSingle );
//		if( $row[ 'list_type' ] != 'xblog_pi15' )
//		{
//			// The destination xBlog isn't an uncached xBlog
//			return;
//		}
//
//		// The destination xBlog is an uncached xBlog
//		$GLOBALS[ 'TSFE' ]->register[ 'xBlogUncachedController' ] = 'MainUncached';
//		return;
//	}

  /**
   * _setPointerInSession
   * 
   * @return void
   * @version 0.30.7
   * @since 0.20.15
   */
  private function _setPointerInSession()
  {
    $uid         = $this->contentObject->data['uid'];
    $sessionData = $this->getSessionData('xblog', $uid);
    $params      = GeneralUtility::_GP('tx_xblog_pi1');
//		if( empty( $params ) )
//		{
//			$params = GeneralUtility::_GP( 'tx_xblog_pi15' );
//		}
    $aPointer    = ['pointer' => $params['pointer']];
    $this->setSessionData($aPointer, $sessionData, 'xblog', $uid);
  }

  /**
   * _viewAssignError( ) :
   *
   * @return void
   * @access private
   * @version 0.10.6
   * @since 0.10.6
   */
  private function _viewAssignError($prompt)
  {
    $prompt = '
			<div data-alert class="alert-box alert">
				<h1>Oops, an error occured :(</h1>
				' . $prompt . '<br />
				<br />
				Sorry for the trouble.<br />
				This is a message of the Extension xBlog.<br />
				<br />
				' . __METHOD__ . '#' . __LINE__ . '<br />
				<a href="#" class="close">&times;</a>
			</div>'
    ;
    $this->view->assignMultiple(
      array(
        'content' => $prompt
      )
    );
  }

  /**
   * _viewAssignList( ) :
   * 
   * @param string showUid like showUid, calUid, newsUid, ...
   *
   * @return void
   * @access private
   * @version 0.23.6
   * @since 0.0.1
   */
  private function _viewAssignList($showUid)
  {
    $this->_initCObj();

    $this->_setPointerInSession();

    $this->_flexformValuesToRegister();

    if ($this->_isOnerecordonlyInSingleView())
    {
      // BE AWARE: List.html will taken!!!
      $this->_viewAssignSingle();
      return;
    }

    if ($this->_isTablelayout())
    {
      $this->_viewAssignTable($showUid);
      return;
    }

    $this->_getXBlogTeaserUidsFromSession();

    $aspectratio       = $this->getXblogMainAspectratio($this->contentObject->data);
    $pageIdForeignList = $this->getPageIdOfForeignListview();
    $this->_renderJavascriptRegisterAspectratio($aspectratio);

    // $showUid must be last value!
    // #i0083, 201219, dwildt: 1-/+
    //$piWrapClass = trim( $aspectratio . ' ' . $showUid );
    $piWrapClass = $showUid;

    // MUST CALLED before $this->RenderPaging(). See below.
    $content = $this->_getContent();

    $this->view->assignMultiple(
      array(
        'tt_content'        => $this->contentObject->data
        , 'content'           => $content
        , 'debug'             => $this->debugMain()
        // #t4744, 200316, 1+
        , 'externalTemplate'  => $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate']
        , 'jsVarsXblogbubble' => $this->_renderJavascriptXblogBubble()
        , 'pageIdForeignList' => $pageIdForeignList
        , 'pagingOnBottom'    => $this->RenderPaging('top')
        , 'pagingOnTop'       => $this->RenderPaging('top')
        , 'piWrapClass'       => $this->_piWrapClass($piWrapClass, 'list')
        , 'publishIconAdd'    => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['add'])
        , 'publishIconCopy'   => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['copy'])
        , 'publishIconDelete' => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['delete'])
        , 'publishIconEdit'   => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['edit'])
        , 'profile'           => $showUid
      )
    );

    $this->_removeXBlogTeaserUidsFromSession();
  }

  /**
   * _viewAssignSingle( ) :
   *
   * @return void
   * @access private
   * @version 0.23.0
   * @since 0.0.1
   */
  private function _viewAssignSingle()
  {
    $this->_flexformValuesToRegister();

    $showUid           = $this->getXblogMainShowUid($this->contentObject->data);
    $this->cObjGetSingle($this->settings['register']['flexform']['pi1']['single']);
    $pageIdForeignList = $this->getPageIdOfForeignListview();

    $this->_getXBlogTeaserUidsFromSession();

    //var_dump( __METHOD__, __LINE__, $this->cObjGetSingle( $this->settings[ 'views' ][ 'single' ][ 'content' ] ) );
    $this->view->assignMultiple(
      array(
        'tt_content'        => $this->contentObject->data
        , 'content'           => $this->cObjGetSingle($this->settings['views']['single']['content'])
        , 'debug'             => $this->debugMain()
        // #t4744, 200316, 1+
        , 'externalTemplate'  => $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate']
        , 'pageIdForeignList' => $pageIdForeignList
        , 'piWrapClass'       => $this->_piWrapClass($showUid, 'single')
        , 'publishIconAdd'    => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['add'])
        , 'publishIconCopy'   => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['copy'])
        , 'publishIconDelete' => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['delete'])
        , 'publishIconEdit'   => $this->cObjGetSingle($this->settings['general']['constants']['publish']['icons']['edit'])
        , 'profile'           => $showUid
      )
    );

    $this->_removeXBlogTeaserUidsFromSession();
  }

  /**
   * _viewAssignTable( ) :
   * 
   * @param string showUid like showUid, calUid, newsUid, ...
   *
   * @return void
   * @access private
   * @version 0.23.6
   * @since 0.0.1
   */
  private function _viewAssignTable($showUid)
  {
    $tablelayoutRows                              = $this->TablelayoutGetRows();
    $thead                                        = $this->TablelayoutThead($tablelayoutRows);
    $tableclass                                   = $this->TablelayoutClass();
    $GLOBALS['TSFE']->register['tablelayoutRows'] = $tablelayoutRows;

    $this->_getXBlogTeaserUidsFromSession();

    $pageIdForeignList = $this->getPageIdOfForeignListview();

    $piWrapClass = $showUid;

    // MUST CALLED before $this->RenderPaging(). See below.
    $content = $this->_getContent();

//		var_dump(__METHOD__, __LINE__, $tablelayoutRows, $content);
//		die();

    $this->view->assignMultiple(
      array(
        'tt_content'        => $this->contentObject->data
        , 'content'           => $content
        , 'debug'             => $this->debugMain()
        , 'externalTemplate'  => $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate']
        , 'pageIdForeignList' => $pageIdForeignList
        , 'pagingOnBottom'    => $this->RenderPaging('top')
        , 'pagingOnTop'       => $this->RenderPaging('top')
        , 'piWrapClass'       => $this->_piWrapClass($piWrapClass, 'list')
        , 'profile'           => $showUid
        , 'tableClass'        => $tableclass
        , 'tableRows'         => $content
        , 'tableHead'         => $thead
      )
    );

    $this->_removeXBlogTeaserUidsFromSession();
  }

  /**
   * listAction( ) : 
   *
   * @return void
   * @access public
   * @version 0.0.1
   * @since 0.0.1
   *
   */
  public function listAction()
  {
    if (FeUserUtility::FeUserUid())
    {
      $arguments = [
        'pointer' => UriUtility::GetXblogPointer()
      ];
        
      $this->redirect('listpublish', null, null, $arguments);
      return;
    }

    $this->initialise();

    $prompt = $this->_extDependency();
    if ($prompt)
    {
      $this->_viewAssignError($prompt);
      return;
    }

    $this->_setController();

    $showUid = $this->getXblogMainShowUid($this->contentObject->data);
    switch (true)
    {
      case($this->actionIsSingle($showUid)):
        // BE AWARE: List.html will taken!!!
        $this->_viewAssignSingle();
        break;
      default:
        $this->_viewAssignList($showUid);
        $this->_resetRegisterList();
        break;
    }
    return;
  }

  /**
   * listpublishAction( ) : 
   *
   * @return  void
   * @access  public
   * @version 0.31.3
   * @since   0.31.3
   */
  public function listpublishAction()
  {
    //var_dump(__METHOD__, __LINE__, time());
    $this->initialise();

    $prompt = $this->_extDependency();
    if ($prompt)
    {
      $this->_viewAssignError($prompt);
      return;
    }

    $this->_setController();

    $showUid = $this->getXblogMainShowUid($this->contentObject->data);
    switch (true)
    {
      case($this->actionIsSingle($showUid)):
        // BE AWARE: List.html will taken!!!
        $this->_viewAssignSingle();
        break;
      default:
        $this->_viewAssignList($showUid);
        $this->_resetRegisterList();
        break;
    }
    return;
  }

  /**
   * resultAction( ) : 
   *
   * @return void
   * @access public
   * @version 0.30.10
   * @since 0.30.10
   *
   */
  public function resultAction()
  {
    $this->listAction();
  }

  /**
   * singleAction( ) : 
   *
   * @return void
   * @access public
   * @version 0.0.1
   * @since 0.0.1
   *
   */
  public function singleAction()
  {
    $this->initialise();

    if (FeUserUtility::FeUserUid())
    {
      $this->_redirectToSinglepublish();
      return;
    }


    if (!$this->singlePluginRequirement())
    {
      return;
    }

    $prompt = $this->_extDependency();
    if ($prompt)
    {
      $this->_viewAssignError($prompt);
      return;
    }

    switch (true)
    {
      case(!$this->displayInSingleView()):
        // Display nothing
        return;
      case(!$this->uidOfDestXblogDoesMatch()):
        $showUid = $this->getXblogMainShowUid($this->contentObject->data);
        $this->_viewAssignList($showUid);
        break;
      default:
        $this->_viewAssignSingle();
        $this->_resetRegisterSingle();
        break;
    }

    return;
  }

  /**
   * singlepublishAction( ) : 
   *
   * @return  void
   * @access  public
   * @version 0.31.3
   * @since   0.31.3
   *
   */
  public function singlepublishAction()
  {
    $this->initialise();

    if (!$this->singlePluginRequirement())
    {
      return;
    }

    $prompt = $this->_extDependency();
    if ($prompt)
    {
      $this->_viewAssignError($prompt);
      return;
    }

    switch (true)
    {
      case(!$this->displayInSingleView()):
        // Display nothing
        return;
      case(!$this->uidOfDestXblogDoesMatch()):
        $showUid = $this->getXblogMainShowUid($this->contentObject->data);
        $this->_viewAssignList($showUid);
        break;
      default:
        $this->_viewAssignSingle();
        $this->_resetRegisterSingle();
        break;
    }

    return;
  }

}
