<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\RegisterUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering a paging
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.9
 * @since 0.0.5
 */
class PagingController extends AbstractPaging
{

	/**
	 * @var string path to the language file
	 * @internal #i0050
	 */
	public $sLanguageFilePath = 'EXT:xblog/Resources/Private/Language/Paging.xlf';

	/**
	 * _calcProperPointer
	 * 
	 * @internal Should correspond with MainController::_calcProperPointer
	 * @return integer	pointer
	 * @version 0.20.17
	 * @since 0.20.17
	 */
	private function _calcProperPointer()
	{
		$params = $this->_paramsCleanedUp();

		if( empty( $params ) )
		{
			return null;
		}

		if( !isset( $params[ 'pointer' ] ) )
		{
			$pointer = $this->getPointerFromSession();
		}
		else
		{
			$pointer = ( int ) $params[ 'pointer' ];
		}

		return $pointer;
	}

	/**
	 * _flexformValuesToRegister( ) :
	 *
	 * @return void
	 * @access private
	 * @version 030.8
	 * @since 030.8
	 */
	private function _flexformValuesToRegister()
	{
		$this->_flexformValuesToRegisterPi1();
		$this->_flexformValuesToRegisterPi3();

		// var_dump(__METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register);
	}

	/**
	 * _flexformValuesToRegisterPi1( ) :
	 *
	 * @return void
	 * @access private
	 * @version 030.8
	 * @since 030.8
	 */
	private function _flexformValuesToRegisterPi1()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$tab = 'ctrl';
		$field = 'settings.flexform.pi3.ctrl.plugin.list';
		$uid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $tab, $field, __METHOD__, __LINE__ );
		$this->evalTargetXblog( $uid, __METHOD__, __LINE__ );

		if( $uid < 1 )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}
		$plugin = 'pi1';
		RegisterUtility::setRegister( $uid, $plugin );
	}

	/**
	 * _flexformValuesToRegisterPi12( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.30.8
	 * @since 030.8
	 */
	private function _flexformValuesToRegisterPi3()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$plugin = 'pi3';
		RegisterUtility::setRegister( $uid, $plugin );

		$GLOBALS[ 'TSFE' ]->register[ 'pi3__pi1_uid' ] = $GLOBALS[ 'TSFE' ]->register[ 'pi1__uid' ];
		$GLOBALS[ 'TSFE' ]->register[ 'pi3__pi1_pid' ] = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pid' ];
		$GLOBALS[ 'TSFE' ]->register[ 'pi3__pi1_table' ] = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_table' ];
	}

	/**
	 * _initCObj( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.0.5
	 * @since 0.0.5
	 */
	private function _initCObj()
	{
		$this->_oPlugin->cObj = $this->contentObject;
	}

	/**
	 * _initLL( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.8.1
	 * @since 0.8.1
	 * @internal #i0050
	 * @todo 190401: method should moved to AbstractController
	 * 
	 */
	private function _initLL()
	{
		$this->_oPlugin->pi_loadLL( $this->sLanguageFilePath );
	}

	/**
	 * _paramsCleanedUp
	 * 
	 * @internal Should correspond with MainController::_calcProperPointer
	 * @return array	params without action=list and controller=Main
	 * @version 0.30.7
	 * @since 0.20.17
	 */
	private function _paramsCleanedUp()
	{
		$params = GeneralUtility::_GP( 'tx_xblog_pi1' );
//		if( empty( $params ) )
//		{
//			$params = GeneralUtility::_GP( 'tx_xblog_pi15' );
//		}

		if( isset( $params[ 'action' ] ) )
		{
			if( $params[ 'action' ] == 'list' )
			{
				unset( $params[ 'action' ] );
			}
		}
		if( isset( $params[ 'controller' ] ) )
		{
			if( $params[ 'controller' ] == 'Main' )
			{
				unset( $params[ 'controller' ] );
			}
//			if( $params[ 'controller' ] == 'MainUncached' )
//			{
//				unset( $params[ 'controller' ] );
//			}
		}

		return $params;
	}

	/**
	 * _setPointerInSession
	 * 
	 * @return void
	 * @version 0.20.17
	 * @since 0.20.15
	 */
	private function _setPointerInSession()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$sessionData = $this->getSessionData( 'xblog', $uid );
		$pointer = $this->_calcProperPointer();
		$aPointer = [ 'pointer' => $pointer ];
		$this->setSessionData( $aPointer, $sessionData, 'xblog', $uid );

		$sessionData = $this->getSessionData( 'xblog', $uid );
	}

	/**
	 * _viewAssignContent( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.30.9
	 * @since 0.0.5
	 * @internal #i0050
	 */
	private function _viewAssignList()
	{
		$this->_initCObj();

		$this->_setPointerInSession();

		$this->_initLL();
		$this->view->assignMultiple(
						array(
								'sessionBefore' => $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'session' ][ 'getPointer' ] )
								, $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'register' ] )
								, 'content' => $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'content' ] )
								, 'paging' => $this->RenderPaging( 'top' )
								, 'debug' => $this->debugPaging()
								, 'sessionSet' => $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'session' ][ 'setPointer' ] )
								, 'sessionAfter' => $this->cObjGetSingle( $this->settings[ 'views' ][ 'paging' ][ 'session' ][ 'getPointer' ] )
						)
		);

	}

	/**
	 * listAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 0.30.8
	 * @since 0.0.5
	 * @internal #t5547
	 */
	public function listAction()
	{
		$this->initialise();

		$this->_flexformValuesToRegister();

		$xBlogMainRow = $this->getXblogMainRow( 'ctrl', 'settings.flexform.pi3.ctrl.plugin.list' );
		$showUid = $this->getXblogMainShowUid( $xBlogMainRow );

		switch( true )
		{
			case($this->actionIsSingle( $showUid )):
			case($this->displayEmptyList( $xBlogMainRow )):
				// return nothing
				break;
			default:
				$this->_viewAssignList();
				break;
		}
		return;
	}

}
