<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\RegisterUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering records
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.24.6
 * @since 0.8.15
 */
class RSSController extends AbstractController
{

	/**
	 * @var string path to the language file
	 */
	public $sLanguageFilePath = 'Resources/Private/Language/';

	/**
	 * _flexformValuesToRegister( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	private function _flexformValuesToRegister()
	{
		$this->_flexformValuesToRegisterPi1();
		$this->_flexformValuesToRegisterPi12();
//		var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register );
//		die();
	}

	/**
	 * _flexformValuesToRegisterPi1( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	private function _flexformValuesToRegisterPi1()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$tab = 'ctrl';
		$field = 'settings.flexform.pi1.ctrl.plugin.single';
		$uid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $tab, $field, __METHOD__, __LINE__ );
		$this->evalTargetXblog( $uid, __METHOD__, __LINE__ );

		if( $uid < 1 )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}
		$plugin = 'pi1';
		RegisterUtility::setRegister( $uid, $plugin );
	}

	/**
	 * _flexformValuesToRegisterPi12( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	private function _flexformValuesToRegisterPi12()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$plugin = 'pi12';
		RegisterUtility::setRegister( $uid, $plugin );
	}

	/**
	 * _typolinkToPage( ) :
	 *
	 * @return string
	 * @access private
	 * @version 0.24.6
	 * @since 0.24.6
	 */
	private function _typolinkToPage()
	{
		$pid = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_rss_pidlist' ];
		if( empty( $pid ) )
		{
			return;
		}

		$conf = [];
		$conf[ 'parameter' ] = $pid;
		$conf[ 'forceAbsoluteUrl' ] = true;
		$conf[ 'returnLast' ] = 'url';
		$conf[ 'useCacheHash' ] = 1;
		$str = NULL;
		$typoLink = $this->_oCObj->typoLink( $str, $conf );
//		var_dump( __METHOD__, __LINE__, $conf, $typoLink );
//		die();
		return $typoLink;
	}

	/**
	 * _viewAssignList( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.22.0
	 * @since 0.8.15
	 */
	private function _viewAssignList()
	{
		$this->_flexformValuesToRegister();

		$this->view->assignMultiple(
						array(
								'content' => $this->cObjGetSingle( $this->settings[ 'content' ] )
								, 'debug' => $this->debugMain()
								, 'rssdescription' => $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_rss_description' ]
								, 'rsslanguage' => $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_rss_language' ]
								, 'rsstitle' => $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_rss_title' ]
								, 'rssurl' => $this->_typolinkToPage()
						)
		);
	}

	/**
	 * listAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 0.22.0
	 * @since 0.8.15
	 *
	 */
	public function feedAction()
	{
		$this->initialise();
		$this->_viewAssignList();
	}

}
