<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\RegisterUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering records
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.25.1
 * @since 0.9.15
 */
class SliderController extends AbstractController
{

	/**
	 * @var string path to the language file
	 */
	public $sLanguageFilePath = 'Resources/Private/Language/';

	/**
	 * _flexformValuesToRegister( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.23.0
	 * @since 0.23.0
	 */
	private function _flexformValuesToRegister()
	{
		$this->_flexformValuesToRegisterPi1();
		$this->_flexformValuesToRegisterPi11();
//		var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register );
//		die();
	}

	/**
	 * _flexformValuesToRegisterPi1( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.25.1
	 * @since 0.23.0
	 */
	private function _flexformValuesToRegisterPi1()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$tab = 'ctrl';
		$field = 'settings.flexform.pi1.ctrl.plugin.single';
		$uid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $tab, $field, __METHOD__, __LINE__ );
		$this->evalTargetXblog( $uid, __METHOD__, __LINE__ );

		if( $uid < 1 )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}
		$plugin = 'pi1';
		RegisterUtility::setRegister( $uid, $plugin );
		$this->_flexformValuesToRegisterPi1Cleanup();
	}

	/**
	 * _flexformValuesToRegisterPi1Cleanup( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	private function _flexformValuesToRegisterPi1Cleanup()
	{
		unset( $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_record_mode' ] );
		unset( $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_record_view' ] );
	}

	/**
	 * _flexformValuesToRegisterPi12( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.23.0
	 * @since 0.23.0
	 */
	private function _flexformValuesToRegisterPi11()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$plugin = 'pi11';
		RegisterUtility::setRegister( $uid, $plugin );
	}

	/**
	 * _viewAssignList( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.9.15
	 * @since 0.9.15
	 */
	private function _viewAssignList()
	{
		$this->_flexformValuesToRegister();
		$content = $this->cObjGetSingle( $this->settings[ 'CONTENT' ][ 'list' ] );

		//var_dump( __METHOD__, __LINE__, $this->contentObject->data );
		//var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate' ] );

		$this->view->assignMultiple(
						array(
								'tt_content' => $this->contentObject->data
								, 'content' => $this->oddDivsAddDiv( $content )
								, 'debug' => $this->debugMain()
								// #t4744, 200316, 1+
								, 'externalTemplate' => $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate' ]
						)
		);
//		var_dump( __METHOD__, __LINE__, $this->cObjGetSingle( $this->settings[ 'views' ][ 'list' ][ 'content' ] ));
//		die();
	}

	/**
	 * listAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 0.9.15
	 * @since 0.9.15
	 *
	 */
	public function listAction()
	{

		$this->initialise();
//		var_dump(__METHOD__, __LINE__, $this->settings);
//		var_dump(__METHOD__, __LINE__, $this->settings['xblogId']);
//		var_dump(__METHOD__, __LINE__, $this->contentObject->data);
//		var_dump(__METHOD__, __LINE__, $this->cObjGetSingle( $this->settings['tables']['pages'] ));
		$this->_viewAssignList();
		// #i0061, dwildt, 1- //unset( $GLOBALS[ 'TSFE' ]->register );
	}

}
