<?php

namespace Netzmacher\Xblog\Controller;

use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\RegisterUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering records
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.23.8
 * @since 0.10.1
 */
class TeaserController extends AbstractController
{


	/**
	 * _flexformValuesToRegister( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.23.8
	 * @since 0.23.8
	 */
	private function _flexformValuesToRegister()
	{
		$this->_flexformValuesToRegisterPi1();
		$this->_flexformValuesToRegisterPi14();
//		var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register );
//		die();
	}

	/**
	 * _flexformValuesToRegisterPi1( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.23.8
	 * @since 0.23.8
	 */
	private function _flexformValuesToRegisterPi1()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$tab = 'table';
		$field = 'settings.flexform.pi14.table.xblog';
		$uid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $tab, $field, __METHOD__, __LINE__ );
		$this->evalTargetXblog( $uid, __METHOD__, __LINE__ );

		if( $uid < 1 )
		{
			$uid = $this->contentObject->data[ 'uid' ];
		}
		$plugin = 'pi1';
		RegisterUtility::setRegister( $uid, $plugin );
	}

	/**
	 * _flexformValuesToRegisterPi12( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.23.8
	 * @since 0.23.8
	 */
	private function _flexformValuesToRegisterPi14()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$plugin = 'pi14';
		RegisterUtility::setRegister( $uid, $plugin );
	}
	
	/**
	 * _removeXBlogTeaserUidsFromSession
	 * 
	 * @return void
	 * @version 0.23.6
	 * @since 0.23.6
	 */
	private function _removeXBlogTeaserUidsFromSession()
	{
		unset( $GLOBALS[ 'TSFE' ]->register[ 'xBlogTeaserUids' ] );
		$sessionUid = 'xBlogTeaser';
		$sessionData = $this->getSessionData( 'xblog', $sessionUid );
		$xBlogTeaserUids = [
				'xBlogTeaserUids' => [
						$this->contentObject->data[ 'pid' ] => ''
				]
		];
		$this->setSessionData( $xBlogTeaserUids, $sessionData, 'xblog', $sessionUid );
	}

	/**
	 * _setPointerInSession
	 * 
	 * @return void
	 * @version 0.23.8
	 * @since 0.23.6
	 */
	private function _setXBlogTeaserUidsInSession()
	{
		$sessionUid = 'xBlogTeaser';
		$sessionData = $this->getSessionData( 'xblog', $sessionUid );
		$xBlogTeaserUids = [
				'xBlogTeaserUids' => [
						$this->contentObject->data[ 'pid' ] => $GLOBALS[ 'TSFE' ]->register[ 'xBlogTeaserUids' ]
				]
		];
		$this->setSessionData( $xBlogTeaserUids, $sessionData, 'xblog', $sessionUid );
		$sessionData = $this->getSessionData( 'xblog', $sessionUid );
	}

	/**
	 * _viewAssignList( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.9.11
	 * @since 0.0.1
	 */
	private function _viewAssignList()
	{
		$this->_flexformValuesToRegister();
		$this->_removeXBlogTeaserUidsFromSession();

		$maincolumn = $this->cObjGetSingle( $this->settings[ 'views' ][ 'list' ][ 'content' ][ 'maincolumn' ] );
		$marginalcolumn = $this->cObjGetSingle( $this->settings[ 'views' ][ 'list' ][ 'content' ][ 'marginalcolumn' ] );


		$this->view->assignMultiple(
						array(
//								'data' => $this->contentObject->data
								'maincolumn' => $maincolumn
								, 'marginalcolumn' => $marginalcolumn
						)
		);
		$this->_setXBlogTeaserUidsInSession();
		//var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register );
		//die();
	}

	/**
	 * listAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 0.0.1
	 * @since 0.0.1
	 *
	 */
	public function listAction()
	{

		$this->initialise();
//		var_dump( __METHOD__, __LINE__, $this->settings );
//		var_dump(__METHOD__, __LINE__, $this->contentObject->data);
		$this->_viewAssignList();
	}

}
