<?php

namespace Netzmacher\Xblog\Domain\Repository;

use Netzmacher\Xblog\Utility\TcaUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;

/* * *
 *
 * This file is part of the Xblog extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019-2021 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * The repository for Elements
 * 
 * @version 0.9.8
 * @since 0.8.26
 */
class ElementRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
{

	/**
	 * @var string
	 */
	private $_localtable = null;

	/**
	 * @param int	$rowUid
	 * @param string $localtable
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.8.26
	 */
	public function GetElementsByRow( $rowUid, $localtable, $pages, $depth ): array
	{
		$this->_localtable = $localtable;

		$arrTypes = [
				'fieldToArray' => [
						'checkbox',
						'radio'
				],
				'fieldToObject' => [
						'select',
						'selectperiod'
				]
		];

		$result = $this->_getElementRowsByUid( $rowUid );

		$rows = [];
		while( $row = $result->fetch() )
		{
			foreach( $arrTypes[ 'fieldToArray' ] as $type )
			{
				$row = $this->_addOptionsAsArray( $row, $type );
				$row = $this->_foreigntableAddOptionsAsArray( $row, $type, $pages, $depth );
				$row = $this->_localtableAddOptionsAsArray( $row, $type, $pages, $depth );
			}
			foreach( $arrTypes[ 'fieldToObject' ] as $type )
			{
				$row = $this->_addOptionsAsObject( $row, $type );
				$row = $this->_foreigntableAddOptionsAsObject( $row, $type, $pages, $depth );
				$row = $this->_localtableAddOptionsAsObject( $row, $type, $pages, $depth );
			}
//			var_dump( __METHOD__, __LINE__, $row );
			$rows[] = $row;
		}

		return $rows;
	}

	/**
	 * @param array $row
	 * @param string $type
	 * @return array
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	private function _addOptionsAsArray( $row, $type ): array
	{
		if( $row[ 'type' ] != $type )
		{
			return $row;
		}
		$arrField = explode( PHP_EOL, $row[ $type ] );
		foreach( $arrField as $key => $csvProperties )
		{
			$arrProperties = explode( '|', trim( $csvProperties ) );
			$arrNewField[ $key ] = $arrProperties;
			if( !isset( $arrNewField[ $key ][ 1 ] ) )
			{
				$arrNewField[ $key ][ 1 ] = $arrNewField[ $key ][ 0 ];
			}
		}
		unset( $row[ $type ] );
//		var_dump( __METHOD__, __LINE__, $row, $arrNewField );
//		die();
		$row[ $type ] = $arrNewField;
		return $row;
	}

	/**
	 * @param array $row
	 * @param string $type
	 * @return array
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	private function _addOptionsAsObject( $row, $type ): array
	{
		$checked = null;
		if( $row[ 'type' ] != $type )
		{
			return $row;
		}
		$arrField = explode( PHP_EOL, $row[ $type ] );
		foreach( $arrField as $csvProperties )
		{
			$arrProperties = explode( '|', trim( $csvProperties ) );
			if( !isset( $arrProperties[ 1 ] ) )
			{
				$arrProperties[ 1 ] = $arrProperties[ 0 ];
			}
			if( isset( $arrProperties[ 2 ] ) )
			{
				$checked = $arrProperties[ 1 ];
			}
			$option = new \stdClass();
			$option->key = $arrProperties[ 1 ];
			$option->value = $arrProperties[ 0 ];
			$options[] = $option;
		}
		unset( $row[ $type ] );
		$row[ $type ] = $options;
		if( $checked )
		{
			$row[ $type . 'checked' ] = $checked;
		}
		//var_dump( __METHOD__, __LINE__, $row );
		return $row;
	}

	/**
	 * @param array $row
	 * @param string $type
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.8.26
	 */
	private function _foreigntableAddOptionsAsArray( $row, $type, $pages, $depth ): array
	{
		$arrNewField = null;

		if( $row[ 'type' ] != $type )
		{
			return $row;
		}
		if( $row[ 'source' ] != 'foreigntable' )
		{
			return $row;
		}

		$column = $row[ 'table' ];
		$tableRows = $this->_foreigntableRows( $column, $pages, $depth );
//		var_dump( __METHOD__, __LINE__, $tableRows );
//		return $row;

		foreach( $tableRows as $tableRow )
		{
			$arrNewField[ $tableRow[ 'key' ] ][ 0 ] = $tableRow[ 'value' ];
			$arrNewField[ $tableRow[ 'key' ] ][ 1 ] = $tableRow[ 'key' ];
		}
		$row[ '_options' ] = $arrNewField;
		return $row;
	}

	/**
	 * @param array $row
	 * @param string $type
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.8.26
	 */
	private function _foreigntableAddOptionsAsObject( $row, $type, $pages, $depth ): array
	{
		if( $row[ 'type' ] != $type )
		{
			return $row;
		}
		if( $row[ 'source' ] != 'foreigntable' )
		{
			return $row;
		}

		//var_dump(__METHOD__, __LINE__, $row );
		$column = $row[ 'table' ];
		$tableRows = $this->_foreigntableRows( $column, $pages, $depth );
		$row = $this->_firstOptionItemAsObject( $row );

		foreach( $tableRows as $tableRow )
		{
			$option = new \stdClass();
			$option->key = $tableRow[ 'key' ];
			$option->value = $tableRow[ 'value' ];
			$options[] = $option;
		}
		$row[ '_options' ] = $options;
		//var_dump( __METHOD__, __LINE__, $row );
		return $row;
	}

	/**
	 * @param string $column
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.8.26
	 */
	private function _foreigntableRows( $column, $pages, $depth ): array
	{
		$foreigntable = TcaUtility::getColumnConfigForeigntable( $this->_localtable, $column );

		//var_dump(__METHOD__, __LINE__, $table );
		$label = $GLOBALS[ 'TCA' ][ $foreigntable ][ 'ctrl' ][ 'label' ];
		$rows = [];
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( $foreigntable )->createQueryBuilder();
		$queryBuilder
						->select( 'uid AS key', $label . ' AS value' )
						->from( $foreigntable )
//						->orderBy( 'sorting' )
		;

		$pids = SqlUtility::getTreeList( $pages, $depth );
		if( !empty( $pids ) )
		{
			$queryBuilder
							->andWhere( $queryBuilder->expr()->in( 'pid', $queryBuilder->createNamedParameter( $pids, Connection::PARAM_INT_ARRAY ) ) );
		}

//		var_dump( __METHOD__, __LINE__, $queryBuilder->getParameters(), $queryBuilder->getSQL() );
//		die();
		
		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row;
		}

		return $rows;
	}

	/**
	 * @param int	$rowUid
	 * @param string list of pages
	 * @param integer depth
	 * @return object resource
	 * @version 0.9.8
	 * @since 0.9.2
	 */
	private function _getElementRowsByUid( $rowUid ): object
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_xblog_domain_model_element' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_element' )
						->where(
										$queryBuilder->expr()->eq( 'tx_xblog_domain_model_form', $queryBuilder->createNamedParameter( $rowUid ) )
						)
		;

		$queryBuilder->orderBy( 'sorting' );

		$result = $queryBuilder->execute();

		return $result;
	}

	/**
	 * @param array $row
	 * @param string $type
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.9.2
	 */
	private function _localtableAddOptionsAsArray( $row, $type, $pages, $depth ): array
	{
		$arrNewField = null;

		if( $row[ 'type' ] != $type )
		{
			return $row;
		}
		if( $row[ 'source' ] != 'localtable' )
		{
			return $row;
		}

		$field = $row[ 'field' ];
		$fieldRows = $this->_localtableRows( $this->_localtable, $field, $pages, $depth );
//		var_dump( __METHOD__, __LINE__, $fieldRows );
//		return $row;

		foreach( $fieldRows as $fieldRow )
		{
			$arrNewField[ $fieldRow[ 'key' ] ][ 0 ] = $this->_renderValueByEval( $this->_localtable, $field, $fieldRow[ 'value' ] );
			$arrNewField[ $fieldRow[ 'key' ] ][ 1 ] = $fieldRow[ 'key' ];
		}
		$row[ '_options' ] = $arrNewField;
		//var_dump( __METHOD__, __LINE__, $row );
		return $row;
	}

	/**
	 * @param array $row
	 * @param string $type
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.9.2
	 */
	private function _localtableAddOptionsAsObject( $row, $type, $pages, $depth ): array
	{
		if( $row[ 'type' ] != $type )
		{
			return $row;
		}
		if( $row[ 'source' ] != 'localtable' )
		{
			return $row;
		}

		$field = $row[ 'field' ];
		$fieldRows = $this->_localtableRows( $this->_localtable, $field, $pages, $depth );

		$row = $this->_firstOptionItemAsObject( $row );

		foreach( $fieldRows as $fieldRow )
		{
			$option = new \stdClass();
			$option->key = $fieldRow[ 'key' ];
			$option->value = $this->_renderValueByEval( $this->_localtable, $field, $fieldRow[ 'value' ] );
			$options[] = $option;
		}
		$row[ '_options' ] = $options;

		return $row;
	}

	/**
	 * @param array $row
	 * @return array $row
	 * @version 0.9.3
	 * @since 0.9.2
	 */
	private function _firstOptionItemAsObject( $row )
	{
//		var_dump( __METHOD__, __LINE__, $row );
//		die();

		switch( true )
		{
			case($row[ 'displaytitlefirstoption' ] && $row[ 'required' ]):
				$row[ '_prependOptionLabel' ] = ' Admin: disable required!';
				return $row;
			case($row[ 'required' ]):
				return $row;
			case($row[ 'displaytitlefirstoption' ]):
				$row[ '_prependOptionLabel' ] = $row[ 'title' ];
				return $row;
			default:
				$row[ '_prependOptionLabel' ] = '';
				return $row;
		}
	}

	/**
	 * @param string $table
	 * @param string $field
	 * @param string list of pages
	 * @param integer depth
	 * @return array
	 * @version 0.9.8
	 * @since 0.9.2
	 */
	private function _localtableRows( $table, $field, $pages, $depth ): array
	{
		$rows = [];
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( $table )->createQueryBuilder();
		$queryBuilder
						->select( $field . ' AS key', $field . ' AS value' )
						->from( $table )
//						->orderBy( 'sorting' )
		;
		
		$pids = SqlUtility::getTreeList( $pages, $depth );
		if( !empty( $pids ) )
		{
			$queryBuilder
							->andWhere( $queryBuilder->expr()->in( 'pid', $queryBuilder->createNamedParameter( $pids, Connection::PARAM_INT_ARRAY ) ) );
		}

//		var_dump( __METHOD__, __LINE__, $pids );
//		var_dump( __METHOD__, __LINE__, $queryBuilder->getParameters(), $queryBuilder->getSQL() );
//		die();

		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row;
		}

		return $rows;
	}

	/**
	 * @param string table
	 * @param string field
	 * @param string value
	 * @return string
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	private function _renderValueByEval( $table, $field, $value ): string
	{
		$value = TcaUtility::getColumnValueRenderedByEval( $table, $field, $value );
		return $value;
	}

}
