<?php

namespace Netzmacher\Xblog\Flexform;

use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * AbstractController for the xBlog xtension
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.31.0
 * @since 0.0.1
 */
class ItemsprocFlexform
{

	/**
	 * GetPagesWiCaddyPi1( ) : 
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.31.0
	 * @since 0.30.0
	 */
	public function GetPagesWiCaddyPi1( &$params )
	{
//		$labelAtPage = $GLOBALS[ 'LANG' ]->sL( 'LLL:EXT:xblog/Resources/Private/Language/FlexForms/Base.xlf:label.atpage' );
		$params[ 'items' ][] = [ '', '', '' ];

		$rows = SqlUtility::getPagesWiPluginCaddyPi1();
		$pages = $this->_getPagesFromRows( $rows );

		foreach( ( array ) $rows as $row )
		{
			$pageUid = $row[ 'pid' ];

			$pageRow = $pages[ $pageUid ];
			if( empty( $pageRow ) )
			{
				continue;
			}
			if( !$GLOBALS[ 'BE_USER' ]->doesUserHaveAccess( $pageRow, 1 ) )
			{
				continue;
			}

//			$header = trim( $row[ 'header' ], '[]' )
//							. ' (' . $row[ 'uid' ] . ') ' . $labelAtPage . ' ' . $pageRow[ 'title' ] . ' (' . $row[ 'pid' ] . ')'
//			;

			$header = $pageRow[ 'title' ] . ' (' . $row[ 'pid' ] . ')';

			$params[ 'items' ][] = [
					$header
					, $row[ 'pid' ]
					, 'typo3conf/ext/caddy/ext_icon.svg'
			];
		}
	}


	/**
	 * GetPagesWiPublish( ) : 
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	public function GetPagesWiPublishPi1( &$params )
	{
//		$labelAtPage = $GLOBALS[ 'LANG' ]->sL( 'LLL:EXT:xblog/Resources/Private/Language/FlexForms/Base.xlf:label.atpage' );
		$params[ 'items' ][] = [ '', '', '' ];

		$rows = SqlUtility::getPagesWiPluginPublish();
		$pages = $this->_getPagesFromRows( $rows );

		foreach( ( array ) $rows as $row )
		{
			$pageUid = $row[ 'pid' ];

			$pageRow = $pages[ $pageUid ];
			if( empty( $pageRow ) )
			{
				continue;
			}
			if( !$GLOBALS[ 'BE_USER' ]->doesUserHaveAccess( $pageRow, 1 ) )
			{
				continue;
			}

//			$header = trim( $row[ 'header' ], '[]' )
//							. ' (' . $row[ 'uid' ] . ') ' . $labelAtPage . ' ' . $pageRow[ 'title' ] . ' (' . $row[ 'pid' ] . ')'
//			;

			$header = $pageRow[ 'title' ] . ' (' . $row[ 'pid' ] . ')';

			$params[ 'items' ][] = [
					$header
					, $row[ 'pid' ]
					, 'typo3conf/ext/publish/ext_icon.svg'
			];
		}
	}

	/**
	 * GetXBlogDest( ) : Returns the uid of the destination xBlog. xBlog must 
	 * have selected the table from the relation xBlog
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.8.16
	 * @since 0.8.10
	 */
	public function GetXBlogDest( &$params )
	{
		$tableXBlogRelation = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '', '' ];

		$rows = SqlUtility::getTtcontentXBlogs();

		foreach( ( array ) $rows as $row )
		{
			$tableXBlogDest = $this->_getXBlogSrceTable( $row[ 'uid' ] );
			if( $tableXBlogDest != $tableXBlogRelation )
			{
				continue;
			}
			$params[ 'items' ][] = [
					$row[ 'header' ]
					, $row[ 'uid' ]
					, 'typo3/sysext/core/Resources/Public/Icons/T3Icons/mimetypes/mimetypes-x-content-plugin.svg'
			];
		}
	}

	/**
	 * GetRecordsForXBlogSinglerecord( ) : 
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.8.16
	 * @since 0.8.10
	 */
	public function GetRecordsForXBlogSinglerecord( &$params )
	{
		//$params[ 'items' ][] = [ '', '', '' ];

		$pages = $params[ 'flexParentDatabaseRow' ][ 'pages' ];
		$recursive = $params[ 'flexParentDatabaseRow' ][ 'recursive' ];

		$tableSheet = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'table' ][ 'lDEF' ];
		$table = $tableSheet[ 'settings.flexform.pi.table.table' ][ 'vDEF' ][ 0 ];

		$rows = SqlUtility::getTableUidsTitle( $table, $pages, $recursive );

		$iconfile = TcaUtility::getCtrlIconfile( $table );

		foreach( ( array ) $rows as $row )
		{
			$params[ 'items' ][] = [
					$row[ 'title' ]
					, $row[ 'uid' ]
					, $iconfile
			];
		}

		return $params;
	}

	/**
	 * RelatedTablesFromXBlogSrce( ) : Returns a list of related tables of the 
	 * selected table of the source xBlog
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.9.3
	 * @since 0.8.10
	 */
	public function RelatedTablesFromXBlogSrce( &$params )
	{
		$params[ 'items' ][] = [ '', '', '' ];

		$uid = $this->_getXBlogSrceUid( $params );
		if( empty( $uid ) )
		{
			return $params;
		}
		$srceTable = $this->_getXBlogSrceTable( $uid );

		foreach( $GLOBALS[ 'TCA' ][ $srceTable ][ 'columns' ] as $table => $properties )
		{
			// #t4446, 200117, 1-/+
			//if( !isset( $properties[ 'config' ][ 'MM' ] ) )
			if( !isset( $properties[ 'config' ][ 'foreign_table' ] ) )
			{
				continue;
			}

			$params[ 'items' ][] = [
					TcaUtility::getCtrlTitle( $table )
					, $table
					, TcaUtility::getCtrlIconfile( $table )
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCAXblogFormList( ) : Get a list of database items from the $TCA
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.24.0
	 * @since 0.2.0
	 */
	public function TCAXblogFormList( &$params )
	{
		//var_dump( __METHOD__, __LINE__, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		var_dump( __METHOD__, __LINE__, $params );
//		die();

		$labelAtPage = $GLOBALS[ 'LANG' ]->sL( 'LLL:EXT:xblog/Resources/Private/Language/FlexForms/Base.xlf:label.atpage' );

		$ctrlLDEF = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'ctrl' ][ 'lDEF' ];

		$pid = $params[ 'flexParentDatabaseRow' ][ 'pid' ];
		$pidOnly = false;
		if( isset( $ctrlLDEF[ 'settings.flexform.ctrl.plugin.xblogfromthispageonly' ][ 'vDEF' ] ) )
		{
			$pidOnly = $ctrlLDEF[ 'settings.flexform.ctrl.plugin.xblogfromthispageonly' ][ 'vDEF' ];
		}
		if( !$pidOnly )
		{
			$pid = 0;
		}

		$uid = $params[ 'flexParentDatabaseRow' ][ 'uid' ];
		$rows = SqlUtility::getXblogsForm( $pid, $uid );
		$pages = $this->_getPagesFromRows( $rows );

		foreach( ( array ) $rows as $fields )
		{
			$pageUid = $fields[ 'pid' ];

			$pageRow = $pages[ $pageUid ];
			if( empty( $pageRow ) )
			{
				continue;
			}
			if( !$GLOBALS[ 'BE_USER' ]->doesUserHaveAccess( $pageRow, 1 ) )
			{
				continue;
			}

			$header = trim( $fields[ 'header' ], '[]' );

			$items[] = [
					$header . ' (' . $fields[ 'uid' ] . ') ' . $labelAtPage . ' ' . $pageRow[ 'title' ] . ' (' . $fields[ 'pid' ] . ')'
					, $fields[ 'uid' ]
			];
		}

		if( empty( $items ) )
		{
			return $params;
		}

		array_multisort( array_column( $items, 0 ), SORT_ASC, $items );

		$params[ 'items' ] = array_merge( $params[ 'items' ], $items );

		return $params;
	}


	/**
	 * TCAXblogList( ) : Get a list of database items from the $TCA
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.30.10
	 * @since 0.2.0
	 */
	public function TCAXblogList( &$params, $withEmptyItem = true, $onlyUncached = false )
	{
		//var_dump( __METHOD__, __LINE__, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		var_dump( __METHOD__, __LINE__, $params );
//		die();
		if( $withEmptyItem )
		{
			$params[ 'items' ][] = [ '', '' ];
		}

		$labelAtPage = $GLOBALS[ 'LANG' ]->sL( 'LLL:EXT:xblog/Resources/Private/Language/FlexForms/Base.xlf:label.atpage' );

		$ctrlLDEF = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'ctrl' ][ 'lDEF' ];
		$tableLDEF = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'table' ][ 'lDEF' ];

		$pid = $params[ 'flexParentDatabaseRow' ][ 'pid' ];
		$pidOnly = false;
		if( isset( $ctrlLDEF[ 'settings.flexform.ctrl.plugin.xblogfromthispageonly' ][ 'vDEF' ] ) )
		{
			$pidOnly = $ctrlLDEF[ 'settings.flexform.ctrl.plugin.xblogfromthispageonly' ][ 'vDEF' ];
		}
		if( !$pidOnly )
		{
			$pid = 0;
		}

		// #v0599, 210106, dwildt, 1-/4+
		//$rows = SqlUtility::getXblogs( $pid );
		$uid = $params[ 'flexParentDatabaseRow' ][ 'uid' ];
		$table = $tableLDEF[ 'settings.flexform.pi.table.table' ][ 'vDEF' ][ 0 ];
		$rows = SqlUtility::getXblogs( $pid, $uid, $table, $onlyUncached );
		$pages = $this->_getPagesFromRows( $rows );

		foreach( ( array ) $rows as $fields )
		{
			$pageUid = $fields[ 'pid' ];

			$pageRow = $pages[ $pageUid ];
			if( empty( $pageRow ) )
			{
				continue;
			}
			if( !$GLOBALS[ 'BE_USER' ]->doesUserHaveAccess( $pageRow, 1 ) )
			{
				continue;
			}

			$header = trim( $fields[ 'header' ], '[]' );

			$items[] = [
					$header . ' (' . $fields[ 'uid' ] . ') ' . $labelAtPage . ' ' . $pageRow[ 'title' ] . ' (' . $fields[ 'pid' ] . ')'
					, $fields[ 'uid' ]
			];
		}

		if( empty( $items ) )
		{
			return $params;
		}

		array_multisort( array_column( $items, 0 ), SORT_ASC, $items );

		$params[ 'items' ] = array_merge( $params[ 'items' ], $items );

		return $params;
	}

	/**
	 * TCAXblogListUncached( ) : 
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.30.7
	 * @since 0.30.7
	 */
	public function TCAXblogListOnlyUncached( &$params )
	{
		$withEmptyItem = true;
		$onlyUncached = true;
		$params = $this->TCAXblogList( $params, $withEmptyItem, $onlyUncached );
		return $params;
	}

	/**
	 * TCAXblogList( ) : Get a list of database items from the $TCA
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.20.74
	 * @since 0.20.74
	 */
	public function TCAXblogListWithoutEmptyItem( &$params )
	{
		$withEmptyItem = false;

		$params = $this->TCAXblogList( $params, $withEmptyItem );

		return $params;
	}

	/**
	 * TCATableRecords( ) : Get a list of records of the current table
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	public function TCATableRecords( &$params )
	{

		$table = $this->_getTableForTCATableFields( $params );

		if( empty( $table ) )
		{
			return $params;
		}

		$rows = SqlUtility::getTableUidsTitle( $table );

		foreach( ( array ) $rows as $fields )
		{
//			var_dump( __METHOD__, __LINE__, $fields );
//			die();

			$items[] = [
					// #i0105, 191005, dwildt., 1-/+
					//$fields[ $label ] . ' (' . $fields[ 'uid' ] . ')'
					$fields[ 'title' ]
					, $fields[ 'uid' ]
			];
		}

		if( empty( $items ) )
		{
			return $params;
		}

		if( empty( TcaUtility::getCtrlSortbyOrDefaultsortby( $table ) ) )
		{
			array_multisort( array_column( $items, 0 ), SORT_ASC, $items );
		}

		$params[ 'items' ] = array_merge( $params[ 'items' ], $items );

		return $params;
	}

	/**
	 * TCATables( ) : Get a list of databse items from the $TCA
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.2.0
	 * @since 0.0.1
	 */
	public function TCATables( &$params )
	{
//		var_dump( __METHOD__, __LINE__, $GLOBALS[ 'BE_USER' ] );
//		die();
//		var_dump( __METHOD__, __LINE__, array_keys( $params ), $params['flexParentDatabaseRow'] );
//		die();
		$params[ 'items' ][] = [ '', '', '' ];

		foreach( array_keys( $GLOBALS[ 'TCA' ] ) as $table )
		{
			if( !$this->_userHasAccess( $table ) )
			{
				continue;
			}
			$params[ 'items' ][] = [
					TcaUtility::getCtrlTitle( $table )
					, $table
					, TcaUtility::getCtrlIconfile( $table )
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFields( ) : Get a list of database items from the $TCA
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.10.1
	 * @since 0.2.0
	 */
	public function TCATableFields( &$params )
	{
//		var_dump( __METHOD__, __LINE__, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		die();
		$params[ 'items' ][] = [ '', '' ];
		$params[ 'items' ][] = [ 'uid', 'uid' ];

		$table = $this->_getTableForTCATableFields( $params );
		if( empty( $table ) )
		{
			return $params;
		}

		$params = $this->_addSortingField( $params, $table );

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsFAL( ) : Get a list of fields from the $TCA, which has the
	 * 												property: foreign_table = sys_file_reference
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.8.10
	 * @since 0.8.10
	 */
	public function TCATableFieldsFAL( &$params )
	{
//		var_dump( __METHOD__, __LINE__, $table, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		die();
		$table = $this->_getTableForTCATableFields( $params );

		if( empty( $table ) )
		{
			$params[ 'items' ][] = [ '', '' ];
			return $params;
		}

//		var_dump( __METHOD__, __LINE__, $table, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		die();
		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'foreign_table' ] ) )
			{
				continue;
			}
			if( $column[ 'config' ][ 'foreign_table' ] != 'sys_file_reference' )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsSysCategory( ) : Get a list of fields from the $TCA, which has the
	 * 																property: foreign_table = sys_file_reference
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.8.10
	 * @since 0.8.10
	 */
	public function TCATableFieldsSysCategory( &$params )
	{
//		var_dump( __METHOD__, __LINE__, $table, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		die();
		$table = $this->_getTableForTCATableFields( $params );

		if( empty( $table ) )
		{
			$params[ 'items' ][] = [ '', '' ];
			return $params;
		}

//		var_dump( __METHOD__, __LINE__, $table, $params, $params[ 'config' ][ 'parameters' ][ 'xblog' ] );
//		die();
		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'foreign_table' ] ) )
			{
				continue;
			}
			if( $column[ 'config' ][ 'foreign_table' ] != 'sys_category' )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsCheck( )
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public function TCATableFieldsCheck( &$params )
	{
		$table = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'type' ] ) )
			{
				continue;
			}
			if( $column[ 'config' ][ 'type' ] != 'check' )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsInput( ) : Get a list of fields from the $TCA, which has the
	 * type input
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.8.10
	 * @since 0.8.10
	 */
	public function TCATableFieldsInput( &$params )
	{
		$table = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'type' ] ) )
			{
				continue;
			}
			if( $column[ 'config' ][ 'type' ] != 'input' )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsText( )
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public function TCATableFieldsText( &$params )
	{
		$table = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'type' ] ) )
			{
				continue;
			}
			if( $column[ 'config' ][ 'type' ] != 'text' )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsWiForeigntableWiMM( ) : Get a list of fields from the $TCA, which has the
	 * property: foreign_table = sys_file_reference
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.8
	 */
	public function TCATableFieldsWiForeigntableWiMM( &$params )
	{
		$table = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'MM' ] ) )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsWiForeigntable( ) : Get a list of fields from the $TCA, which has the
	 * property: foreign_table = sys_file_reference
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.12
	 */
	public function TCATableFieldsWiForeigntable( &$params )
	{
		$table = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'foreign_table' ] ) )
			{
				continue;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * TCATableFieldsTstamp( ) : Get a list of fields from the $TCA, which has the
	 * property: foreign_table = sys_file_reference
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.8.10
	 * @since 0.8.10
	 */
	public function TCATableFieldsTstamp( &$params )
	{
		$table = $this->_getTableForTCATableFields( $params );

		$params[ 'items' ][] = [ '', '' ];

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] as $columns => $column )
		{
			if( !isset( $column[ 'config' ][ 'eval' ] ) )
			{
				continue;
			}
			switch( true )
			{
				case( strpos( $column[ 'config' ][ 'eval' ], 'date' ) !== false ):
				case( strpos( $column[ 'config' ][ 'eval' ], 'datetime' ) !== false ):
				case( strpos( $column[ 'config' ][ 'eval' ], 'time' ) !== false ):
					// follow the workflow
					break;
				default:
					continue 2;
			}
			$label = trim( $GLOBALS[ 'LANG' ]->sL( $column[ 'label' ] ), ':' );
			if( empty( $label ) )
			{
				$label = $columns;
			}
			else
			{
				$label = $label . ' (' . $columns . ')';
				//$label = $columns . ' (' . $label . ')';
			}

			$params[ 'items' ][] = [
					$label
					, $columns
			];
		}

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * _addSortingField( ) : Get a list of database items from the $TCA
	 *
	 * @return array	item array for a flexform
	 * @access private
	 * @version 0.20.70
	 * @since 0.20.70
	 */
	private function _addSortingField( $params, $table )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'sortby' ] ) )
		{
			return $params;
		}

		$column = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'sortby' ];

		$params[ 'items' ][] = [
				$GLOBALS[ 'LANG' ]->sL( 'LLL:EXT:xblog/Resources/Private/Language/FlexForms/Base.xlf:label.sorting' ) . ' (' . $column . ')'
				, $column
		];

		return $params;
	}

	/**
	 * _getPagesFromRows()
	 *
	 * @param array	rows 
	 * @return array rows
	 * @access private
	 * @version 0.10.1
	 * @since 0.8.10
	 */
	private function _getPagesFromRows( $rows )
	{
		$aPidList = NULL;
		$sPidList = NULL;
		$pages = NULL;

		if( empty( $rows ) )
		{
			return;
		}

		foreach( $rows as $row )
		{
			$aPidList[] = $row[ 'pid' ];
		}
		$sPidList = implode( ',', ( array ) $aPidList );
		$rows = SqlUtility::getPagesFromList( $sPidList );

		foreach( $rows as $key => $row )
		{
			$pages[ $row[ 'uid' ] ] = $row;
		}

		return $pages;
	}

	/**
	 * _getTableForTCATableFields():
	 *
	 * @param array	current params
	 * @return string	path to the icon file of the given table
	 * @access private
	 * @version 0.10.1
	 * @since 0.8.10
	 */
	private function _getTableForTCATableFields( $params )
	{
		$table = '';

		//var_dump( __METHOD__, __LINE__, array_keys( $params ), $params['flexParentDatabaseRow']['pi_flexform']['data']['table']['lDEF']['settings.flexform.pi.table.table']['vDEF'], $params[ 'row' ][ 'settings.flexform.pi.table.table' ][ 0 ] );
		//var_dump( __METHOD__, __LINE__, array_keys( $params ), $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ], $params[ 'row' ] );
		//die();
		switch( true )
		{
			// xBlogMain
			case(isset( $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'table' ][ 'lDEF' ][ 'settings.flexform.pi.table.table' ][ 'vDEF' ] )):
				$table = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'table' ][ 'lDEF' ][ 'settings.flexform.pi.table.table' ][ 'vDEF' ];
				break;
			// xBlogRelation
			case(isset( $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'ctrl' ][ 'lDEF' ][ 'settings.flexform.pi10.ctrl.plugin.relation' ][ 'vDEF' ] )):
				$table = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'ctrl' ][ 'lDEF' ][ 'settings.flexform.pi10.ctrl.plugin.relation' ][ 'vDEF' ];
				break;
			// xBlogTeaser
			case(isset( $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'table' ][ 'lDEF' ][ 'settings.flexform.pi14.table.xblog' ][ 'vDEF' ] )):
				$xBlogUid = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'table' ][ 'lDEF' ][ 'settings.flexform.pi14.table.xblog' ][ 'vDEF' ];
				if( is_array( $xBlogUid ) )
				{
					$xBlogUid = $xBlogUid[ 0 ];
				}
				if( !empty( $xBlogUid ) )
				{
					$table = SqlUtility::getXBlogSrceTable( $xBlogUid );
				}
				break;
			default:
//				$prompt = 'FATAL ERROR: ' . __METHOD__ . ' (#' . __LINE__ . '): flexform data is missing.';
//				die( $prompt );
				return '';
		}

		if( is_array( $table ) )
		{
			$table = $table[ 0 ];
		}

		return $table;
	}

	/**
	 * _getXBlogSrceTable():
	 *
	 * @param integer	uid
	 * @return string table from xBlog source
	 * @access private
	 * @version 0.8.10
	 * @since 0.8.10
	 */
	private function _getXBlogSrceTable( $xBlogSrceUid )
	{
		$xBlogSrceTable = SqlUtility::getXBlogSrceTable( $xBlogSrceUid );
		return $xBlogSrceTable;
	}

	/**
	 * _getXBlogSrceUid():
	 *
	 * @return integer	uid of the xBlog source
	 * @access private
	 * @version 0.8.10
	 * @since 0.8.10
	 */
	private function _getXBlogSrceUid( $params )
	{
		$xBlogSrceUid = $params[ 'flexParentDatabaseRow' ][ 'pi_flexform' ][ 'data' ][ 'ctrl' ][ 'lDEF' ][ 'settings.flexform.pi10.ctrl.plugin.srce' ][ 'vDEF' ];
		if( is_array( $xBlogSrceUid ) )
		{
			$xBlogSrceUid = $xBlogSrceUid[ 0 ];
		}

		return $xBlogSrceUid;
	}

	/**
	 * _userHasAccess( ) : Get a list of database items from the $TCA
	 * 
	 * @param	string	table
	 * @param	string	field
	 *
	 * @return boolean
	 * @access private
	 * @version 0.20.66
	 * @since 0.20.66
	 */
	private function _userHasAccess( $table, $field = null )
	{
		switch( true )
		{
			case(empty( $field )):
				return $this->_userHasAccessTable( $table );
			default:
				return $this->_userHasAccessTableField( $table, $field );
		}
	}

	/**
	 * _userHasAccessTable( )
	 * 
	 * @param	string	table
	 * @param	string	field
	 *
	 * @return boolean
	 * @access private
	 * @version 0.20.66
	 * @since 0.20.66
	 */
	private function _userHasAccessTable( $table )
	{
		if( $GLOBALS[ 'BE_USER' ]->check( 'tables_select', $table ) )
		{
			return true;
		}
		return false;
	}

	/**
	 * _userHasAccessTableField( ): Checking if a table/field pair is allowed explicitly through the “Allowed Excludefields”:
	 * 
	 * @param	string	table
	 * @param	string	field
	 *
	 * @return boolean
	 * @access private
	 * @version 0.20.66
	 * @since 0.20.66
	 */
	private function _userHasAccessTableField( $table, $field )
	{
		if( $GLOBALS[ 'BE_USER' ]->check( 'non_exclude_fields', $table . ':' . $field ) )
		{
			return true;
		}
		return false;
	}

}
