<?php

namespace Netzmacher\Xblog\Frontend\DataProcessing;

use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
use TYPO3\CMS\Frontend\ContentObject\DataProcessorInterface;

/* * *************************************************************
 *
 *  The MIT License (MIT)
 *
 *  Copyright (c) 2019-2021 - Dirk Wildt, http://wildt.at.die-netzmacher.de
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 *
 * ************************************************************* */

/**
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xBlog
 *
 * @version 0.6.0
 * @since 0.6.0
 */
class BrigdeTtcontent implements DataProcessorInterface
{

	/**
	 * Assign xBlog values to tt_content fields
	 *
	 * @param ContentObjectRenderer $cObj The data of the content element or page
	 * @param array $contentObjectConfiguration The configuration of Content Object
	 * @param array $processorConfiguration The configuration of this processor
	 * @param array $processedData Key/value store of processed data (e.g. to be passed to a Fluid View)
	 * @return array the processed data as key/value store
	 * 
	 * @access public
	 * @version 0.6.5
	 * @since 0.6.5
	 */
	public function process( ContentObjectRenderer $cObj, array $contentObjectConfiguration, array $processorConfiguration, array $processedData )
	{
		if( isset( $processorConfiguration[ 'if.' ] ) && !$cObj->checkIf( $processorConfiguration[ 'if.' ] ) )
		{
			return $processedData;
		}

		$fieldNames = $processorConfiguration[ 'fieldNames.' ];
		foreach( array_keys( $fieldNames ) as $fieldName )
		{
			if( trim( $fieldName, '.' ) != $fieldName )
			{
				continue;
			}
				// Assign xBlog value to tt_content field
			$processedData = $this->_assignToCobjData( $cObj, $fieldNames, $processedData, $fieldName );
		}

		return $processedData;
	}

	/**
	 * _assignToCobjData: Assign xBlog values to tt_content fields
	 *
	 * @param ContentObjectRenderer $cObj The data of the content element or page
	 * @param array $fieldNames 
	 * @param array $processedData Key/value store of processed data (e.g. to be passed to a Fluid View)
	 * @param string $fieldName
	 * @return array the processed data as key/value store
	 * 
	 * @access private
	 * @version 0.6.5
	 * @since 0.6.5
	 */
	private function _assignToCobjData( ContentObjectRenderer $cObj, array $fieldNames, array $processedData, $fieldName )
	{
			// Don't overwrite existing fields
		if( isset( $processedData[ 'data' ][ $fieldName ] ) )
		{
			return $processedData;
		}
		
			// Assign xBlog value to tt_content field
		$processedData[ 'data' ][ $fieldName ] = $cObj->stdWrapValue( $fieldName, $fieldNames );
		//var_dump(__METHOD__, __LINE__, $fieldNames, $fieldName, $processedData[ 'data' ][ $fieldName ]);
		return $processedData;
	}

}
