<?php

namespace Netzmacher\Xblog\Hooks;

/**
 * This file is part of the "xBlog" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use Netzmacher\Xblog\Utility\ExternalTemplateUtility;
use TYPO3\CMS\Backend\Utility\BackendUtility as BackendUtilityCore;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\StringUtility;

/**
 * Userfunc to render alternative label for media elements
 * 
 * @version 0.9.11
 * @since 0.9.2
 * @internal #t4744
 */
class ItemsProcFunc
{

	/** @var ExternalTemplate $externalTemplateUtility */
	private $externalTemplateUtility;

	/**
	 * __construct
	 * 
	 * @version 0.9.11
	 * @since 0.9.2
	 * @internal #t4744
	 */
	public function __construct()
	{
		$this->externalTemplateUtility = GeneralUtility::makeInstance( ExternalTemplateUtility::class );
	}

	/**
	 * Itemsproc function to extend the selection of externalTemplate in the plugin
	 *
	 * @param array &$config configuration array
	 * @version 0.9.11
	 * @since 0.9.2
	 * @internal #t4744
	 */
	public function ExternalTemplate( array &$config )
	{
		$pageId = 0;

		$currentColPos = $config[ 'flexParentDatabaseRow' ][ 'colPos' ];
		$pageId = $this->getPageId( $config[ 'flexParentDatabaseRow' ][ 'pid' ] );

		if( $pageId > 0 )
		{
			$externalTemplate = $this->externalTemplateUtility->getAvailableExternalTemplates( $pageId );

			$externalTemplate = $this->reduceExternalTemplates( $externalTemplate, $currentColPos );
			foreach( $externalTemplate as $layout )
			{
				$additionalLayout = [
						htmlspecialchars( $this->getLanguageService()->sL( $layout[ 0 ] ) ),
						$layout[ 1 ]
				];
				array_push( $config[ 'items' ], $additionalLayout );
			}
		}
	}

	/**
	 * Reduce the template layouts by the ones that are not allowed in given colPos
	 *
	 * @param array $externalTemplate
	 * @param int $currentColPos
	 * @return array
	 * @version 0.9.11
	 * @since 0.9.2
	 * @internal #t4744
	 */
	private function reduceExternalTemplates( $externalTemplate, $currentColPos )
	{
		$currentColPos = ( int ) $currentColPos;
		$restrictions = [];
		$allLayouts = [];
		foreach( $externalTemplate as $key => $layout )
		{
			if( is_array( $layout[ 0 ] ) )
			{
				if( isset( $layout[ 0 ][ 'allowedColPos' ] ) && StringUtility::endsWith( $layout[ 1 ], '.' ) )
				{
					$layoutKey = substr( $layout[ 1 ], 0, -1 );
					$restrictions[ $layoutKey ] = GeneralUtility::intExplode( ',', $layout[ 0 ][ 'allowedColPos' ], true );
				}
			}
			else
			{
				$allLayouts[ $key ] = $layout;
			}
		}
		if( !empty( $restrictions ) )
		{
			foreach( $restrictions as $restrictedIdentifier => $restrictedColPosList )
			{
				if( !in_array( $currentColPos, $restrictedColPosList, true ) )
				{
					unset( $allLayouts[ $restrictedIdentifier ] );
				}
			}
		}

		return $allLayouts;
	}

	/**
	 * Get page id, if negative, then it is a "after record"
	 *
	 * @param int $pid
	 * @return int
	 * @version 0.9.11
	 * @since 0.9.2
	 * @internal #t4744
	 */
	private function getPageId( $pid )
	{
		$pid = ( int ) $pid;

		if( $pid > 0 )
		{
			return $pid;
		}

		$row = BackendUtilityCore::getRecord( 'tt_content', abs( $pid ), 'uid,pid' );
		return $row[ 'pid' ];
	}

	/**
	 * Returns LanguageService
	 *
	 * @return \TYPO3\CMS\Lang\LanguageService
	 */
	private function getLanguageService()
	{
		return $GLOBALS[ 'LANG' ];
	}

}