<?php

//declare(strict_types=1);

namespace Netzmacher\Xblog\Hooks\Xblogform;

use Netzmacher\Xblog\Utility\FlashmessageUtility;
use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use TYPO3\CMS\Core\DataHandling\DataHandler;

//use TYPO3\CMS\Extbase\Object\ObjectManager;

/**
 * Class CreateName to autofill field tx_xblog_domain_model_element.name with value 
 * 
 * @version 0.9.4
 * @since 0.9.4
 * @internal #t4450
 */
class CreateName
{

	/**
	 * _changeNameIfNeeded()
	 * 
	 * @param array $incomingFieldArray
	 * @return boolean 
	 * @access private
	 * 
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	private function _changeNameIfNeeded( $incomingFieldArray )
	{
		switch( true )
		{
			case(isset( $incomingFieldArray[ 'field' ] )):
			case(isset( $incomingFieldArray[ 'source' ] )):
			case(isset( $incomingFieldArray[ 'table' ] )):
			case(isset( $incomingFieldArray[ 'type' ] )):
				return true;
			default:
				return false;
		}
	}

	/**
	 * _getLocaltable()
	 * 
	 * @param array $incomingFieldArray
	 * @param array $row
	 * @return string
	 * @access private
	 * 
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	private function _getLocaltable( $incomingFieldArray, $row )
	{
		$xBlogFormUid = $this->_setDefaultValue( $row, 'tx_xblog_domain_model_form', $incomingFieldArray );
		$row = SqlUtility::getRecord( 'tx_xblog_domain_model_form', $xBlogFormUid );
		$row = SqlUtility::getRecord( 'tt_content', $row[ 'tt_content' ] );

//		$header = 'pi_flexform';
//		$text = serialize( $row[ 'pi_flexform' ] );
//		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );

		$sheet = 'ctrl';
		$field = 'settings.flexform.pi13.ctrl.plugin.list';
		$xmlFlexform = $row[ 'pi_flexform' ];
		$xBlogUid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		$row = SqlUtility::getRecord( 'tt_content', $xBlogUid );

		$sheet = 'table';
		$field = 'settings.flexform.pi.table.table';
		$xmlFlexform = $row[ 'pi_flexform' ];
		$localtable = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		return $localtable;
	}

	/**
	 * _setDefaultValue()
	 * 
	 * @param array $row
	 * @param integer $column
	 * @param array $incomingFieldArray
	 * @access private
	 * 
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	private function _setDefaultValue( $row, $column, $incomingFieldArray )
	{
		$value = $row[ $column ];

		if( isset( $incomingFieldArray[ $column ] ) )
		{
			$value = $incomingFieldArray[ $column ];
		}
		return $value;
	}

	/**
	 * _setName()
	 * 
	 * @param array $incomingFieldArray
	 * @param string $table
	 * @param integer $id
	 * @access private
	 * 
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	private function _setName( &$incomingFieldArray, $table, $id )
	{
		if( !$this->_changeNameIfNeeded( $incomingFieldArray, $table, $id ) )
		{
			return;
		}

		$row = SqlUtility::getRecord( $table, $id );
		$type = $this->_setDefaultValue( $row, 'type', $incomingFieldArray );

		switch( $type )
		{
			case('sword'):
				$incomingFieldArray[ 'name' ] = 'sword';
//				$header = 'name';
//				$text = 'sword';
//				FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
				return;
			case('checkbox'):
			case('radio'):
			case('select'):
			case('selectperiod'):
				$incomingFieldArray[ 'name' ] = $this->_setNameBySource( $incomingFieldArray, $row );
//				$header = 'name';
//				$text = $incomingFieldArray[ 'name' ];
//				FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
				return;
			case('button'):
			case('hidden'):
			case('placeholder'):
				return;
			default:
				$prompt = 'BUG: key in switch isn\'t definded. Key"' . $type . '"';
				var_dump( __METHOD__, __LINE__, $prompt );
				die();
		}
	}

	/**
	 * _setNameBySource()
	 * 
	 * @param array $incomingFieldArray
	 * @param array $row
	 * @return string name
	 * @access private
	 * 
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	private function _setNameBySource( $incomingFieldArray, $row )
	{
		$field = $this->_setDefaultValue( $row, 'field', $incomingFieldArray );
		$foreigntable = $this->_setDefaultValue( $row, 'table', $incomingFieldArray );
		$source = $this->_setDefaultValue( $row, 'source', $incomingFieldArray );

		$localtable = $this->_getLocaltable( $incomingFieldArray, $row );

		switch( $source )
		{
			case('localtable'):
				$name = $localtable . '.' . $field;
				return $name;
			case('foreigntable'):
				$name = $localtable . '.' . $foreigntable;
				return $name;
			default:
				$prompt = 'BUG: key in switch isn\'t definded.';
				var_dump( __METHOD__, __LINE__, $prompt );
				die();
		}
	}

	/**
	 * @param array $incomingFieldArray
	 * @param string $table
	 * @param integer $id
	 * @param DataHandler $dataHandler
	 * 
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	public function processDatamap_preProcessFieldArray( &$incomingFieldArray, $table, $id, DataHandler $dataHandler )
	{
		if( $table != 'tx_xblog_domain_model_element' )
		{
			return;
		}
		if( !(( string ) $id === (string) ( int ) $id ))
		{
			// id is something like: NEW5e230d0182eeb793369049
			return;
		}
//		$header = 'id';
//		$text = $id;
//		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
//		$header = 'dataHandler';
//		$text = serialize( $dataHandler );
//		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
		$this->_setName( $incomingFieldArray, $table, $id );
	}

}
