<?php

namespace Netzmacher\Xblog\Plugin;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering paging. Here: bugfix for TYPO 9.5
 *
 * Bug with additionalParams:
 *   - Proper: GeneralUtility::implodeArrayForUrl returns f.e: &tx_xblog_pi1[plugin]=732
 *   - Not proper: HttpUtility::buildQueryString returns f.e: &%5Btx_xblog_pi1%5Bplugin%5D%5D=732
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.8.4
 * @since 0.8.4
 * @internal #i0062

 */
// TYPO3 8.7 code
use TYPO3\CMS\Core\Utility\GeneralUtility;

// TYPO3 9.5 code
//use TYPO3\CMS\Core\Utility\HttpUtility;

class AbstractPlugin extends \TYPO3\CMS\Frontend\Plugin\AbstractPlugin
{

	/**
	 * Link string to the current page.
	 * Returns the $str wrapped in <a>-tags with a link to the CURRENT page, but with $urlParameters set as extra parameters for the page.
	 *
	 * @param string $str The content string to wrap in <a> tags
	 * @param array $urlParameters Array with URL parameters as key/value pairs. They will be "imploded" and added to the list of parameters defined in the plugins TypoScript property "parent.addParams" plus $this->pi_moreParams.
	 * @param bool $cache If $cache is set (0/1), the page is asked to be cached by a &cHash value (unless the current plugin using this class is a USER_INT). Otherwise the no_cache-parameter will be a part of the link.
	 * @param int $altPageId Alternative page ID for the link. (By default this function links to the SAME page!)
	 * @return string The input string wrapped in <a> tags
	 * @see pi_linkTP_keepPIvars(), ContentObjectRenderer::typoLink()
	 * @version 0.8.4
	 * @since 0.8.4
	 * @internal #i0062
	 */
	public function pi_linkTP( $str, $urlParameters = [], $cache = false, $altPageId = 0 )
	{
//var_dump(__METHOD__, __LINE__, $urlParameters);		
		$conf = [];
		// Isn't proper for TYPO3 9.5
		//$conf[ 'useCacheHash' ] = $this->pi_USER_INT_obj ? 0 : $cache;
		// Must be for TYPO3 9.5
		$conf[ 'useCacheHash' ] = 1;
		$conf[ 'no_cache' ] = $this->pi_USER_INT_obj ? 0 : !$cache;
		$conf[ 'parameter' ] = $altPageId ? $altPageId : ($this->pi_tmpPageId ? $this->pi_tmpPageId : $this->frontendController->id);
		// [Proper]: TYPO3 8.7 code. Returns f.e: &tx_xblog_pi1[plugin]=732
		$conf[ 'additionalParams' ] = $this->conf[ 'parent.' ][ 'addParams' ] . GeneralUtility::implodeArrayForUrl( '', $urlParameters, '', true ) . $this->pi_moreParams;
		// [Doesn't run]: TYPO3 9.5 code. Returns f.e: &%5Btx_xblog_pi1%5Bplugin%5D%5D=732
		//$conf[ 'additionalParams' ] = $this->conf[ 'parent.' ][ 'addParams' ] . HttpUtility::buildQueryString( $urlParameters, '&', true ) . $this->pi_moreParams;
		//var_dump( __METHOD__, __LINE__, $conf[ 'additionalParams' ] );
		return $this->cObj->typoLink( $str, $conf );
	}

}
