<?php

namespace Netzmacher\Xblog\TCA\Columns\Config;

use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * AbstractController for the xBlog xtension
 *
 * @package TYPO3
 * @subpackage xblog
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.30.12
 * @since 0.8.24
 */
class ItemsProcFunc
{

	/**
	 * Returns a the list for assistent depending on the current type
	 *
	 * @return array	item array 
	 * @access public
	 * @version 0.8.24
	 * @since 0.8.24
	 */
	public static function Assistent( &$params )
	{
		$items = null;
		$type = $params[ 'row' ][ 'type' ];

		if( $type == 'foreigntable' )
		{
			$items[] = [
					'LLL:EXT:xblog/Resources/Private/Language/locallang_db.xlf:tx_xblog_domain_model_filter.assistent.suggestion',
					'suggestion'
			];
		}
		$items[] = [
				'LLL:EXT:xblog/Resources/Private/Language/locallang_db.xlf:tx_xblog_domain_model_filter.assistent.default',
				'default'
		];
		$items[] = [
				'LLL:EXT:xblog/Resources/Private/Language/locallang_db.xlf:tx_xblog_domain_model_filter.assistent.formula',
				'formula'
		];
		$items[] = [
				'LLL:EXT:xblog/Resources/Private/Language/locallang_db.xlf:tx_xblog_domain_model_filter.assistent.expert',
				'expert'
		];

		$params[ 'items' ] = array_merge( $params[ 'items' ], $items );

		return $params;
	}

	/**
	 * Returns a the list for the operateor depending on the current assistent
	 *
	 * @return array	item array 
	 * @access public
	 * @version 0.8.24
	 * @since 0.8.24
	 */
	public static function Operator( &$params )
	{
		$items = null;
		$assistent = $params[ 'row' ][ 'assistent' ][ '0' ];

		switch( $assistent )
		{
			case('default'):
				$items[] = [ '=', 'eq' ];
				$items[] = [ '!=', 'neq' ];
				$items[] = [ '<', 'lt' ];
				$items[] = [ '<=', 'lte' ];
				$items[] = [ '>', 'gt' ];
				$items[] = [ '>=', 'gte' ];
				$items[] = [ 'IN', 'in' ];
				$items[] = [ 'NOT IN', 'notIn' ];
				$items[] = [ 'IS NULL', 'isNull' ];
				$items[] = [ 'IS NOT NULL', 'isNotNull' ];
				$items[] = [ 'LIKE', 'like' ];
				$items[] = [ 'NOT LIKE', 'notLike' ];
				break;
			case('expert'):
			case('formula'):
				break;
			case('suggestion'):
				$items[] = [ '=', 'eq' ];
				$items[] = [ '!=', 'neq' ];
				$items[] = [ '<', 'lt' ];
				$items[] = [ '<=', 'lte' ];
				$items[] = [ '>', 'gt' ];
				$items[] = [ '>=', 'gte' ];
				$items[] = [ 'IN', 'in' ];
				$items[] = [ 'NOT IN', 'notIn' ];
				break;
			default:
				$debug = var_export( $assistent, true );
				$items[] = [ 'SWITCH ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): assistent = "' . $debug . '"', '' ];
				break;
		}

		//$items[] = [ 'FIND_IN_SET', 'inSet' ];

		if( empty( $items ) )
		{
			return $params;
		}

		$params[ 'items' ] = array_merge( $params[ 'items' ], $items );

		return $params;
	}

	/**
	 * Returns a list of records (uids) depending on the selected table
	 *
	 * @return array	item array 
	 * @access public
	 * @version 0.30.12
	 * @since 0.8.24
	 */
	public static function Suggestions( &$params )
	{
		$items = null;
		$table = $params[ 'row' ][ 'table' ];

		if( empty( $table ) )
		{
			return $params;
		}

		$xBlogMainUid = $params[ 'row' ][ 'tt_content' ];
		$row = SqlUtility::getTtcontentRowByUid( $xBlogMainUid );
		$pages = $row[ 'pages' ];
		$recursive = $row[ 'recursive' ];

		$rows = SqlUtility::getTableUidsTitle( $table, $pages, $recursive );

		foreach( ( array ) $rows as $fields )
		{

			$items[] = [
					// #i0105, 191005, dwildt., 1-/+
					//$fields[ $label ] . ' (' . $fields[ 'uid' ] . ')'
					$fields[ 'title' ]
					, $fields[ 'uid' ]
			];
		}

		if( empty( $items ) )
		{
			return $params;
		}

		switch( true )
		{
			case( empty( TcaUtility::getCtrlSortbyOrDefaultsortby( $table ) ) ):
				// #i0068, 210630, dwildt, 1+
			case( TcaUtility::getCtrlSortbyOrDefaultsortby( $table ) == 'sorting' ):
				array_multisort( array_column( $items, 0 ), SORT_ASC, $items );
				break;
		}

		$params[ 'items' ] = array_merge( $params[ 'items' ], $items );

		return $params;
	}

}
