<?php

namespace Netzmacher\Xblog\TCA\Ctrl;

use Netzmacher\Xblog\Domain\Model\Element;
use Netzmacher\Xblog\Domain\Model\Filter;
use Netzmacher\Xblog\Domain\Model\Tablelayout;
use Netzmacher\Xblog\Utility\Typo3VersionUtility;

if( !defined( 'TYPO3_MODE' ) )
{
	die( 'Access denied.' );
}

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author      Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package     TYPO3
 * @subpackage  xBlog
 * @version     0.10.1
 * @since       0.8.23
 */
class Icons
{

	public static $pathEmpty = 'EXT:xblog/Resources/Public/Icons/xBlogEmpty';
	public static $pathFilterForeigntable = 'EXT:xblog/Resources/Public/Icons/Type/foreigntable';
	public static $pathFilterLocaltable = 'EXT:xblog/Resources/Public/Icons/Type/localtable';
	public static $pathHidden = 'EXT:xblog/Resources/Public/Icons/xBlogHidden';
	public static $pathMain = 'EXT:xblog/Resources/Public/Icons/xBlogMain';
	public static $pathMainCat = 'EXT:xblog/Resources/Public/Icons/xBlogMainCat';
	public static $pathMainDatesheet = 'EXT:xblog/Resources/Public/Icons/xBlogMainDatesheet';
	public static $pathMainForm = 'EXT:xblog/Resources/Public/Icons/xBlogMainForm';
	public static $pathMainImage = 'EXT:xblog/Resources/Public/Icons/xBlogMainImage';
	public static $pathMainMap = 'EXT:xblog/Resources/Public/Icons/xBlogMainMap';
	public static $pathMainMedia = 'EXT:xblog/Resources/Public/Icons/xBlogMainMedia';
	public static $pathMainUncached = 'EXT:xblog/Resources/Public/Icons/xBlogMainUncached';
	public static $pathMini = 'EXT:xblog/Resources/Public/Icons/xBlogMini';
	public static $pathPaging = 'EXT:xblog/Resources/Public/Icons/xBlogPaging';
	public static $pathRelation = 'EXT:xblog/Resources/Public/Icons/xBlogRelation';
	public static $pathRSS = 'EXT:xblog/Resources/Public/Icons/xBlogRSS';
	public static $pathSearch = 'EXT:xblog/Resources/Public/Icons/xBlogSearch';
	public static $pathSlider = 'EXT:xblog/Resources/Public/Icons/xBlogSlider';
	public static $pathTablelayoutForeigntable = 'EXT:xblog/Resources/Public/Icons/tx_xblog_domain_model_tablelayout';
	public static $pathTablelayoutLocaltable = 'EXT:xblog/Resources/Public/Icons/tx_xblog_domain_model_tablelayout';
	public static $pathTeaser = 'EXT:xblog/Resources/Public/Icons/xBlogTeaser/xBlogTeaser-2ColumnsLeft';

	/**
	 * Register Icons
	 *
	 * @return      void
	 * @version     0.8.23
	 * @since       0.8.23
	 */
	static public function RegisterIcons()
	{
		if( Typo3VersionUtility::get() < 7006000 )
		{
			return;
		}

		$iconRegistry = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \TYPO3\CMS\Core\Imaging\IconRegistry::class );

		foreach( self::_getClassWiIcons() as $class => $source )
		{
			$iconRegistry->registerIcon(
							$class
							, \TYPO3\CMS\Core\Imaging\IconProvider\SvgIconProvider::class
							, [ 'source' => $source ]
			);
		}
	}

	/**
	 *
	 * @return      string
	 * @access			private
	 * @version     0.8.23
	 * @since       0.8.23
	 */
	static private function _Ext()
	{
		switch( true )
		{
			case( Typo3VersionUtility::get() < 7006000 ):
				return 'gif';
			case( Typo3VersionUtility::get() >= 7006000 ):
			default:
				return 'svg';
		}
	}

	/**
	 * @param				string $table
	 * @access public
	 * @return      array
	 * @version     0.8.26
	 * @since       0.8.23
	 */
	static public function TypeIconClasses( $table )
	{
		if( Typo3VersionUtility::get() < 7006000 )
		{
			return NULL;
		}
		switch( $table )
		{
			case(Element::TABLE_NAME):
				return array(
						'button' => 'ext-xblogMainForm'
						, 'checkbox' => 'ext-xblogMainForm'
						, 'hidden' => 'ext-xblogHidden'
						, 'placeholder' => 'ext-xblogEmpty'
						, 'radio' => 'ext-xblogMainForm'
						, 'select' => 'ext-xblogMainForm'
						, 'selectperiod' => 'ext-xblogMainForm'
						, 'sword' => 'ext-xblogMainForm'
						, 'text' => 'ext-xblogMainForm'
				);
			case(Filter::TABLE_NAME):
				return array(
						'localtable' => 'ext-xblogFilterLocaltable'
						, 'foreigntable' => 'ext-xblogFilterForeigntable'
				);
			case(Tablelayout::TABLE_NAME):
				return array(
						'localtable' => 'ext-xblogTablelayoutLocaltable'
						, 'foreigntable' => 'ext-xblogTablelayoutForeigntable'
				);
			default:
//				return array(
//						'localtable' => 'ext-xblogFilterLocaltable'
//						, 'foreigntable' => 'ext-xblogFilterForeigntable'
//				);
		}
	}

	/**
	 * Get icon array with classes and paths
	 *
	 * @return      void
	 * @version     0.10.1
	 * @since       0.8.23
	 */
	static private function _getClassWiIcons()
	{
		$ext = self::_Ext();

		return array(
				'ext-xblogEmpty' => self::$pathEmpty . '.' . $ext
				, 'ext-xblogFilterForeigntable' => self::$pathFilterForeigntable . '.' . $ext
				, 'ext-xblogFilterLocaltable' => self::$pathFilterLocaltable . '.' . $ext
				, 'ext-xblogHidden' => self::$pathHidden . '.' . $ext
				, 'ext-xblogMain' => self::$pathMain . '.' . $ext
				, 'ext-xblogMainCat' => self::$pathMainCat . '.' . $ext
				, 'ext-xblogMainDatesheet' => self::$pathMainDatesheet . '.' . $ext
				, 'ext-xblogMainForm' => self::$pathMainForm . '.' . $ext
				, 'ext-xblogMainImage' => self::$pathMainImage . '.' . $ext
				, 'ext-xblogMainMap' => self::$pathMainMap . '.' . $ext
				, 'ext-xblogMainMedia' => self::$pathMainMedia . '.' . $ext
				, 'ext-xblogMainUncached' => self::$pathMainUncached . '.' . $ext
				, 'ext-xblogMini' => self::$pathMini . '.' . $ext
				, 'ext-xblogPaging' => self::$pathPaging . '.' . $ext
				, 'ext-xblogRelation' => self::$pathRelation . '.' . $ext
				, 'ext-xblogRSS' => self::$pathRSS . '.' . $ext
				, 'ext-xblogSearch' => self::$pathSearch . '.' . $ext
				, 'ext-xblogSlider' => self::$pathSlider . '.' . $ext
				, 'ext-xblogTablelayoutForeigntable' => self::$pathTablelayoutForeigntable . '.' . $ext
				, 'ext-xblogTablelayoutLocaltable' => self::$pathTablelayoutLocaltable . '.' . $ext
				, 'ext-xblogTeaser' => self::$pathTeaser . '.' . $ext
		);
	}

}
