<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\TCA\Form\Filter\Element;

use Netzmacher\Xblog\Utility\FlashmessageUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;
use TYPO3\CMS\Backend\Form\Element\SelectSingleElement;

	/**
	 * TableElement
	 *
	 * @version 0.20.18
	 * @since 0.8.23
	 */

class TableElement extends SelectSingleElement
{

	/**
	 * Render single element
	 *
	 * @return array As defined in initializeResultArray() of AbstractNode
	 * @version 0.8.24
	 * @since 0.8.23
	 */
	public function render()
	{
		// For development only - 1/2
//		$empty = false;
//		if( empty( $this->data[ 'inlineParentUid' ] ) )
//		{
//			$this->data[ 'inlineParentUid' ] = 755;
//			$empty = true;
//		}

		$this->_setItems();
		$this->_setItemFormElValue();

		// For development only - 2/2
//		if( $empty )
//		{
//			var_dump( __METHOD__, __CLASS__, $this->data[ 'parameterArray' ] );
//			die();
//		}
		return parent::render();
	}

	/**
	 * Render items for select options
	 *
	 * @return array items
	 * @access	private
	 * @version 0.20.18
	 * @since 0.8.23
	 */
	private function _getItems()
	{
		$items = NULL;
		$parentUid = $this->data[ 'inlineParentUid' ];

		FlashmessageUtility::render( 'inlineParentUid', $this->data[ 'inlineParentUid' ], 'INFO', __METHOD__, __LINE__, false );

		if( empty( $parentUid ) )
		{
			$parentUid = $this->data[ 'databaseRow' ][ 'tt_content' ];
		}

		if( empty( $parentUid ) )
		{
			return null;
		}

		$xBlogSrceTable = SqlUtility::getXBlogSrceTable( $parentUid );

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $xBlogSrceTable ][ 'columns' ] as $table => $properties )
		{
			if( !$this->_getItemsCondition( $properties ) )
			{
				continue;
			}


			$items[] = [
					TcaUtility::getCtrlTitle( $table )
					, $table
					, TcaUtility::getCtrlIconfile( $table )
			];
		}

		// #t5534, 200706, dwildt, 4+
		if( empty( $items ) )
		{
			return;
		}

		array_multisort( array_column( $items, 0 ), SORT_ASC, $items );

//			var_dump( __METHOD__, __LINE__, $parentUid, $xBlogSrceTable, $items );
//			die();
		return $items;
	}

	/**
	 * Condition
	 *
	 * @param array	$properties
	 * @return boolean
	 * @access	private
	 * @todo :todo: missing relation support
	 * @version 0.8.24
	 * @since 0.8.23
	 */
	private function _getItemsCondition( $properties )
	{
		switch( true )
		{
			case(!isset( $properties[ 'config' ][ 'MM' ] ) ):
				return false;
			case(!isset( $properties[ 'config' ][ 'MM_match_fields' ] ) ):
				return false;
			case(!isset( $properties[ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ] ) ):
				return false;
			case(!isset( $properties[ 'config' ][ 'MM_match_fields' ][ 'table_local' ] ) ):
				return false;
			case( isset( $properties[ 'config' ][ 'MM_opposite_field' ] ) ):
				return false;
			default:
				return true;
		}
	}

	/**
	 * Extends the items array for select options
	 *
	 * @return void
	 * @access	private
	 * @version 0.8.24
	 * @since 0.8.23
	 */
	private function _setItems()
	{
		$items = $this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ];
		$items = array_merge( ( array ) $items, ( array ) $this->_getItems() );

//			var_dump( __METHOD__, __LINE__, $items );
//			die();
		$this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ] = $items;
	}

	/**
	 * Workaround for itemFormElValue: Move string to array[0]
	 *
	 * @return void
	 * @access	private
	 * @version 0.8.24
	 * @since 0.8.23
	 */
	private function _setItemFormElValue()
	{
		if( empty( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		if( is_array( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		$selectedValue = $this->data[ 'parameterArray' ][ 'itemFormElValue' ];
		unset( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] );
		$this->data[ 'parameterArray' ][ 'itemFormElValue' ][ 0 ] = $selectedValue;
	}

}
