<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\TCA\Form\Form\Element;

use Netzmacher\Xblog\Utility\FlashmessageUtility;
use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;
use TYPO3\CMS\Backend\Form\Element\SelectSingleElement;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;

class FieldElement extends SelectSingleElement
{

	/**
	 * @return array
	 * @version 0.9.2
	 * @since 0.9.2
	 * @internal #i0116
	 */
	private function _getElementRecord()
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_xblog_domain_model_form' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_element' )
						->where(
										$queryBuilder->expr()->eq( 'uid', $queryBuilder->createNamedParameter( $this->data[ 'vanillaUid' ] ) )
						)
		;

		$result = $queryBuilder->execute();
		$row = $result->fetch();

		return $row;
	}

	/**
	 * @param int	$rowUid
	 * @return array
	 * @version 0.9.2
	 * @since 0.9.2
	 * @internal #i0116
	 */
	private function _getFormRecord( $formUid )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_xblog_domain_model_form' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_form' )
						->where(
										$queryBuilder->expr()->eq( 'uid', $queryBuilder->createNamedParameter( $formUid ) )
						)
		;

		$result = $queryBuilder->execute();
		$row = $result->fetch();

		return $row;
	}

	/**
	 * Render single element
	 *
	 * @return array As defined in initializeResultArray() of AbstractNode
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	public function render()
	{
		$this->_setItems();
		$this->_setItemFormElValue();
		return parent::render();
	}

	/**
	 * Render items for select options
	 *
	 * @return array items
	 * @access	private
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	private function _getItems()
	{

		$items = NULL;

		$this->row = $this->_getElementRecord();
		$formRowUid = $this->data[ 'inlineParentUid' ];

		$formRow = $this->_getFormRecord( $formRowUid );
//		$header = 'formRow';
//		$text = serialize( $formRow );
//		FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
		$xBlogPi13Uid = $formRow[ 'tt_content' ];

		if( empty( $xBlogPi13Uid ) )
		{
			$header = 'xBlog.uid is empty!';
			$text = 'Oops. This in an unexpected behaviour!';
			FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
			$header = 'Tip!';
			$text = 'Don\'t edit inline records directly!';
			FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
			return null;
		}

		$xBlogPi13Row = SqlUtility::getRecord( 'tt_content', $xBlogPi13Uid );

		$sheet = 'ctrl';
		$field = 'settings.flexform.pi13.ctrl.plugin.list';
		$xmlFlexform = $xBlogPi13Row[ 'pi_flexform' ];
		$xBlogMainUid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		//$xBlogMainUid = SqlUtility::getXBlogSrceTable( $xBlogPi13Uid, $sheet, $field );
		if( empty( $xBlogMainUid ) )
		{
			$header = 'Oops!';
			$text = 'Uid of the main xBlog can\'t found!';
			FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
			return null;
		}
		$header = 'Uid';
		$text = '"' . $xBlogMainUid . '"';
		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
		$xBlogSrceTable = SqlUtility::getXBlogSrceTable( $xBlogMainUid );

		if( empty( $xBlogSrceTable ) )
		{
			$header = 'Oops!';
			$text = 'Source table of the main xBlog is empty!';
			FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
			return null;
		}

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $xBlogSrceTable ][ 'columns' ] as $column => $properties )
		{
			if( !$this->_getItemsCondition( $xBlogSrceTable, $column ) )
			{
				continue;
			}

			$items[] = [
					TcaUtility::getColumnLabel( $xBlogSrceTable, $column )
					, $column
			];
		}

		if( !is_array( $items ) )
		{
			$header = $xBlogSrceTable;
			$text = 'Source table hasn\'t any related table!';
			FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
			return;
		}

		array_multisort( array_column( $items, 0 ), SORT_ASC, $items );

		return $items;
	}

	/**
	 * Condition
	 *
	 * @param string $table
	 * @param string $field
	 * @return boolean
	 * @access	private
	 * @version 0.30.7
	 * @since 0.9.2
	 */
	private function _getItemsCondition( $table, $field )
	{
		switch( true )
		{
			case(TcaUtility::getColumnConfigType( $table, $field ) == 'input' ):
				return $this->_getItemsConditionTcaEval( $table, $field );
			default:
				return false;
		}
	}

	/**
	 * Condition
	 *
	 * @param string $table
	 * @param string $field
	 * @return boolean
	 * @access	private
	 * @version 0.30.7
	 * @since 0.9.2
	 */
	private function _getItemsConditionTcaEval( $table, $field )
	{
		$csvEval = TcaUtility::getColumnConfigEval( $table, $field );

		if( empty( $csvEval ) )
		{
			return true;
		}
		$arrEval = explode( ',', trim( $csvEval ) );

		switch( true )
		{
			case(in_array( 'datetime', $arrEval )):
			case(in_array( 'double2', $arrEval )):
			case(in_array( 'int', $arrEval )):
			case(in_array( 'password', $arrEval )):
			case(in_array( 'saltedPassword', $arrEval )):
			case(in_array( 'year', $arrEval )):
				return false;
		}

		return true;
	}

	/**
	 * Extends the items array for select options
	 *
	 * @return void
	 * @access	private
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	private function _setItems()
	{
		$items = $this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ];
//		$header = 'Debugging';
//		$text = serialize( $items );
//		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
		$items = array_merge( ( array ) $items, ( array ) $this->_getItems() );

//		$header = 'Debugging';
//		$text = serialize( $items );
//		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
		$this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ] = $items;
	}

	/**
	 * Workaround for itemFormElValue: Move string to array[0]
	 *
	 * @return void
	 * @access	private
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	private function _setItemFormElValue()
	{
		if( empty( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		if( is_array( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		$selectedValue = $this->data[ 'parameterArray' ][ 'itemFormElValue' ];
		unset( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] );
		$this->data[ 'parameterArray' ][ 'itemFormElValue' ][ 0 ] = $selectedValue;
	}

}
