<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\TCA\Form\Form\Element;

use Netzmacher\Xblog\Utility\FlashmessageUtility;
use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;
use TYPO3\CMS\Backend\Form\Element\SelectSingleElement;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;

class TableElement extends SelectSingleElement
{

	/**
	 * @param int	$rowUid
	 * @return array
	 * @version 0.8.26
	 * @since 0.8.26
	 * @internal #i0116
	 */
	private function _getFormRecord( $formUid )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_xblog_domain_model_form' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_form' )
						->where(
										$queryBuilder->expr()->eq( 'uid', $queryBuilder->createNamedParameter( $formUid ) )
						)
		;

		$result = new \stdClass();
		$result = $queryBuilder->execute();
		$row = $result->fetch();

		return $row;
	}

	/**
	 * Render single element
	 *
	 * @return array As defined in initializeResultArray() of AbstractNode
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	public function render()
	{
		$this->_setItems();
		$this->_setItemFormElValue();
		return parent::render();
	}

	/**
	 * Render items for select options
	 *
	 * @return array items
	 * @access	private
	 * @version 0.9.3
	 * @since 0.8.26
	 */
	private function _getItems()
	{

		$items = NULL;
		$formRowUid = $this->data[ 'inlineParentUid' ];

		$formRow = $this->_getFormRecord( $formRowUid );
		$xBlogPi13Uid = $formRow[ 'tt_content' ];

		if( empty( $xBlogPi13Uid ) )
		{
			$header = 'xBlog.uid is empty!';
			$text = 'Oops. This in an unexpected behaviour!';
			FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
			$header = 'Tip!';
			$text = 'Don\'t edit inline records directly!';
			FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
			return null;
		}
		
		$xBlogPi13Row = SqlUtility::getRecord('tt_content', $xBlogPi13Uid);

		$sheet = 'ctrl';
		$field = 'settings.flexform.pi13.ctrl.plugin.list';
		$xmlFlexform = $xBlogPi13Row[ 'pi_flexform' ];
		$xBlogMainUid = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );
		//$xBlogMainUid = SqlUtility::getXBlogSrceTable( $xBlogPi13Uid, $sheet, $field );
		if( empty( $xBlogMainUid ) )
		{
			$header = 'Oops!';
			$text = 'Uid of the main xBlog can\'t found!';
			FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
			return null;
		}
//			$header = 'Uid';
//			$text = '"' . $xBlogMainUid . '"';
//			FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );

		$xBlogSrceTable = SqlUtility::getXBlogSrceTable( $xBlogMainUid );

		if( empty( $xBlogSrceTable ) )
		{
			$header = 'Oops!';
			$text = 'Source table of the main xBlog is empty!';
			FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
			return null;
		}

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $xBlogSrceTable ][ 'columns' ] as $table => $properties )
		{
			if( !$this->_isRelatedTable( $properties, $xBlogSrceTable, $table ) )
			{
				continue;
			}

			// #t4446, 200117, dwildt, ~
			$items[] = [
					TcaUtility::getCtrlTitle( $table )
//					, $properties[ 'config' ][ 'foreign_table' ]
					, $table
			];
		}

		if( !is_array( $items ) )
		{
			$header = $xBlogSrceTable;
			$text = 'Source table hasn\'t any related table!';
			FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );
			return;
		}

		array_multisort( array_column( $items, 0 ), SORT_ASC, $items );

		return $items;
	}

	/**
	 * Condition
	 *
	 * @param array	$properties
	 * @param string $xBlogSrceTable label of the current table
	 * @param string $field current field
	 * @return boolean
	 * @access	private
	 * @todo :todo: missing relation support
	 * @version 0.9.3
	 * @since 0.8.26
	 */
	private function _isRelatedTable( $properties, $xBlogSrceTable, $field )
	{
		switch( true )
		{
			case( $this->_isTransOrigPointerField( $xBlogSrceTable, $field ) ):
				return false;
			case( $this->_isRelatedTableWoAnySupport( $properties ) ):
				return false;
			case( $this->_isRelatedTableWiMM( $properties ) ):
				return true;
			case( $this->_isRelatedTableWoMM( $properties ) ):
				return true;
			default:
				return false;
		}
	}

	/**
	 * @param array	$properties
	 * @return boolean
	 * @access	private
	 * @todo :todo: missing relation support
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	private function _isRelatedTableWiMM( $properties )
	{
		switch( true )
		{
			case(isset( $properties[ 'config' ][ 'MM' ] ) ):
				return true;
			default:
				return false;
		}
	}

	/**
	 * @param array	$properties
	 * @return boolean
	 * @access	private
	 * @todo :todo: missing relation support
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	private function _isRelatedTableWoAnySupport( $properties )
	{
		switch( true )
		{
			case(!isset( $properties[ 'config' ][ 'foreign_table' ] ) ):
				return false;
			case($properties[ 'config' ][ 'foreign_table' ] == 'sys_category' ):
			case($properties[ 'config' ][ 'foreign_table' ] == 'sys_file_reference' ):
				return true;
			default:
				return false;
		}
	}

	/**
	 * @param string $xBlogSrceTable label of the current table
	 * @param string $field label of the current field
	 * @return boolean
	 * @access	private
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	private function _isTransOrigPointerField( $xBlogSrceTable, $field )
	{
		if( $field == $GLOBALS[ 'TCA' ][ $xBlogSrceTable ][ 'ctrl' ][ 'transOrigPointerField' ] )
		{
			return true;
		}
		return false;
	}

	/**
	 * @param array	$properties
	 * @return boolean
	 * @access	private
	 * @todo :todo: missing relation support
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	private function _isRelatedTableWoMM( $properties )
	{
		switch( true )
		{
			case(isset( $properties[ 'config' ][ 'foreign_table' ] ) ):
				return true;
			default:
				return false;
		}
	}

	/**
	 * Extends the items array for select options
	 *
	 * @return void
	 * @access	private
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	private function _setItems()
	{
		$items = $this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ];
		$items = array_merge( ( array ) $items, ( array ) $this->_getItems() );

//			var_dump( __METHOD__, __LINE__, $items );
//			die();
		$this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ] = $items;
	}

	/**
	 * Workaround for itemFormElValue: Move string to array[0]
	 *
	 * @return void
	 * @access	private
	 * @version 0.8.26
	 * @since 0.8.26
	 */
	private function _setItemFormElValue()
	{
		if( empty( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		if( is_array( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		$selectedValue = $this->data[ 'parameterArray' ][ 'itemFormElValue' ];
		unset( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] );
		$this->data[ 'parameterArray' ][ 'itemFormElValue' ][ 0 ] = $selectedValue;
	}

}
