<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\TCA\Form\Tablelayout\Element;

use Netzmacher\Xblog\Utility\FlashmessageUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;
use TYPO3\CMS\Backend\Form\Element\SelectSingleElement;

class ColumnvalueElement extends SelectSingleElement
{

	/**
	 * Render single element
	 *
	 * @return array As defined in initializeResultArray() of AbstractNode
	 * @version 0.30.19
	 * @since 0.30.19
	 */
	public function render()
	{
		$this->_setItems();
		$this->_setItemTablelayoutColumnvalue();

		return parent::render();
	}

	/**
	 * Render items for select options
	 *
	 * @return array items
	 * @access	private
	 * @version 0.30.19
	 * @since 0.30.19
	 */
	private function _getTableFromXblog()
	{
		$parentUid = $this->data[ 'inlineParentUid' ];
		if( empty( $parentUid ) )
		{
			$parentUid = $this->data[ 'databaseRow' ][ 'tt_content' ];
		}
		if( !empty( $parentUid ) )
		{
			$xBlogSrceTable = SqlUtility::getXBlogSrceTable( $parentUid );
			return $xBlogSrceTable;
		}

		$header = 'Ooops, an error occured!';
		$text = 'Record hasn\'t any parent uid!';
		FlashmessageUtility::render( $header, $text, 'ERROR', __METHOD__, __LINE__ );
	}

	/**
	 * Render items for select options
	 *
	 * @return array items
	 * @access	private
	 * @version 0.30.19
	 * @since 0.30.19
	 */
	private function _addItems()
	{
		$items = NULL;

		$table = $this->_getTableFromXblog();
		$field = $this->data[ 'databaseRow' ][ 'field' ];

		$foreignTable = TcaUtility::getColumnConfigForeigntable( $table, $field );

		if( !$foreignTable )
		{
			$items[] = [
					'No column available'
					, ''
			];
			return $items;
		}

//		$header = 'ColumnvalueElement';
//		$text = $foreignTable;
//		FlashmessageUtility::render( $header, $text, 'INFO', __METHOD__, __LINE__ );

		foreach( ( array ) $GLOBALS[ 'TCA' ][ $foreignTable ][ 'columns' ] as $column => $properties )
		{
			if( !$this->_addItemsCondition( $properties ) )
			{
				continue;
			}
			if( !isset( $properties[ 'label' ] ) )
			{
				continue;
			}
			$label = $GLOBALS[ 'LANG' ]->sL( $properties[ 'label' ] );
			if( empty( $label ) )
			{
				$label = $column;
			}

			$items[] = [
					$label
					, $column
			];
		}

		if( empty( $items ) )
		{
			return;
		}

		array_multisort( array_column( $items, 0 ), SORT_ASC, $items );

		return $items;
	}

	/**
	 * Condition
	 *
	 * @param array	$properties
	 * @return boolean
	 * @access	private
	 * @todo :todo: missing relation support
	 * @version 0.30.19
	 * @since 0.30.19
	 */
	private function _addItemsCondition( $properties )
	{
		switch( true )
		{
			case(isset( $properties[ 'config' ][ 'foreign_table' ] ) ):
				return false;
			case(isset( $properties[ 'config' ][ 'MM' ] ) ):
				return false;
			case($properties[ 'config' ][ 'special' ] == 'languages' ):
				return false;
			default:
				return true;
		}
	}

	/**
	 * Extends the items array for select options
	 *
	 * @return void
	 * @access	private
	 * @version 0.30.19
	 * @since 0.30.19
	 */
	private function _setItems()
	{
		$items = $this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ];
		$items = array_merge( ( array ) $items, ( array ) $this->_addItems() );

		$this->data[ 'parameterArray' ][ 'fieldConf' ][ 'config' ][ 'items' ] = $items;
	}

	/**
	 * Workaround for itemFormElValue: Move string to array[0]
	 *
	 * @return void
	 * @access	private
	 * @version 0.30.19
	 * @since 0.30.19
	 */
	private function _setItemTablelayoutColumnvalue()
	{
		if( empty( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		if( is_array( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] ) )
		{
			return;
		}

		$selectedValue = $this->data[ 'parameterArray' ][ 'itemFormElValue' ];
		unset( $this->data[ 'parameterArray' ][ 'itemFormElValue' ] );
		$this->data[ 'parameterArray' ][ 'itemFormElValue' ][ 0 ] = $selectedValue;
	}

}
