<?php

namespace Netzmacher\Xblog\Userfunc\AndWhere;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version		0.30.13
 * @since			0.20.55
 */
class DatetimeAndwhereUserfunc extends AbstractUserfunc {

	public $_conf;
	private $_table;
	private $_field;
	private $_condition;
	private $_showUid;
	private $_debugUid;

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string andWhere statement
	 * @version 0.30.13
	 * @since 0.20.55
	 */
	public function AndWhere($content, $conf) {
		unset($content);

		$andWhere = NULL;

		$this->_conf = $conf;
		$this->_debugUid = $this->_cObj('debugUid');
		$this->_listType = $this->_cObj('list_type');

		$this->_andWhereRegister();
		$this->_requirements();

		switch (true) {
			case( $this->_showUid != 'calUid' ):
				return;
			case(!$this->_condition || empty($this->_field) ):
				return;
			default:
				// follow the workflow
				break;
		}

		switch (true) {
			case($this->_condition == 'default'):
				return;
			case($this->_condition == 'uptodate'):
				$andWhere = $this->_andWhereUptodate();
				break;
			case($this->_condition == 'deprecated'):
				$andWhere = '<= UNIX_TIMESTAMP()';
				break;
			default:
				$this->_dieCondition($this->_condition);
				break;
		}

		$andWhere = ' AND ' . $this->_table . '.' . $this->_field . ' ' . $andWhere . ' ';

		return $andWhere;
	}

	/**
	 * @return		void
	 * @version		0.30.13
	 * @since			0.30.13
	 * @internal	#v1333
	 */
	private function _andWhereRegister() {
		switch ($this->_listType) {
			case('xblog_pi1'):
			case('xblog_pi2'):
			case('xblog_pi3'):
				$this->_condition = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_cal_andwhere'];
				$this->_conditionUptodate = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_cal_andwhereUptodate'];
				$this->_field = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_datetimefield'];
				$this->_showUid = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_showUid'];
				$this->_table = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi_table_table'];
				break;
			case('xblog_pi11'):
				$this->_condition = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_cal_andwhere'];
				$this->_conditionUptodate = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_cal_andwhereUptodate'];
				$this->_field = $GLOBALS['TSFE']->register['pi11__pi_flexform__settings_flexform_pi1_profile_datetimefield'];
				$this->_showUid = $GLOBALS['TSFE']->register['pi11__pi_flexform__settings_flexform_pi1_profile_showUid'];
				$this->_table = $GLOBALS['TSFE']->register['pi11__pi_flexform__settings_flexform_pi_table_table'];
				break;
			case('xblog_pi12'):
				$this->_condition = $GLOBALS['TSFE']->register['pi12__pi_flexform__settings_flexform_pi1_profile_cal_andwhere'];
				$this->_conditionUptodate = $GLOBALS['TSFE']->register['pi1__pi_flexform__settings_flexform_pi1_profile_cal_andwhereUptodate'];
				$this->_field = $GLOBALS['TSFE']->register['pi12__pi_flexform__settings_flexform_pi1_profile_datetimefield'];
				$this->_showUid = $GLOBALS['TSFE']->register['pi12__pi_flexform__settings_flexform_pi1_profile_showUid'];
				$this->_table = $GLOBALS['TSFE']->register['pi12__pi_flexform__settings_flexform_pi_table_table'];
				break;
			default:
				$this->_dieListType();
		}
	}

	/**
	 * @return string andWhere statement
	 * @version 0.25.2
	 * @since 0.20.55
	 */
	private function _andWhereUptodate() {

		switch ($this->_conditionUptodate) {
			case('hour'):
				$andWhere = '>= UNIX_TIMESTAMP(CONCAT( YEAR(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', MONTH(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', DAY(FROM_UNIXTIME(UNIX_TIMESTAMP())), \' \', HOUR(FROM_UNIXTIME(UNIX_TIMESTAMP())), \':00:00\' )) ';
				break;
			case('day'):
				$andWhere = '>= UNIX_TIMESTAMP(CONCAT( YEAR(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', MONTH(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', DAY(FROM_UNIXTIME(UNIX_TIMESTAMP())), \' 00:00:00\' )) ';
				break;
			case('week'):
				$andWhere = '>= UNIX_TIMESTAMP(CONCAT( YEAR(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', MONTH(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', DAY(FROM_UNIXTIME(UNIX_TIMESTAMP())) - WEEKDAY(FROM_UNIXTIME(UNIX_TIMESTAMP())), \' 00:00:00\' )) ';
				break;
			case('month'):
				$andWhere = '>= UNIX_TIMESTAMP(CONCAT( YEAR(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-\', MONTH(FROM_UNIXTIME(UNIX_TIMESTAMP())), \'-01 00:00:00\' )) ';
				break;
			case('year'):
				$andWhere = '>= UNIX_TIMESTAMP(CONCAT( YEAR(FROM_UNIXTIME(UNIX_TIMESTAMP())),		\'-01-01 00:00:00\' )) ';
				break;
			case('now'):
			default:
				$andWhere = '>= UNIX_TIMESTAMP() ';
				break;
		}

		return $andWhere;
		return $andWhere . ' @497t1oug';
	}

	/**
	 * _dieCondition()
	 * 
	 * @return void
	 * @version 0.25.2
	 * @since 0.20.55
	 */
	private function _dieCondition() {
		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  Condition is undefined'
						. '</h2>'
						. '<p>'
						. '  table = "' . $this->_condition . '"<br />'
						. '</p>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $this->_debugUid . '<br />'
						. '</p>'
		;
		die($prompt);
	}

	/**
	 * _dieCondition()
	 * 
	 * @return	void
	 * @version 0.30.13
	 * @since		0.30.13
	 */
	private function _dieListType() {
		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  Switch case is undefined'
						. '</h2>'
						. '<p>'
						. '  list_type: ' . $this->_listType . ' isn\'t respected<br />'
						. '</p>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $this->_debugUid . '<br />'
						. '</p>'
		;
		die($prompt);
	}

	/**
	 * _dieTable()
	 * 
	 * @return void
	 * @version 0.25.2
	 * @since 0.20.55
	 */
	private function _dieTable() {
		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  There is an empty field'
						. '</h2>'
						. '<p>'
						. '  table = "' . $this->_table . '"<br />'
						. '</p>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $this->_debugUid . '<br />'
						. '</p>'
		;
		die($prompt);
	}

	/**
	 * _requirements()
	 * 
	 * @return void
	 * @version 0.25.2
	 * @since 0.20.55
	 */
	private function _requirements() {
		switch (true) {
			case(empty($this->_table)):
				$this->_dieTable();
			default:
				return;
		}
	}

}
