<?php

namespace Netzmacher\Xblog\Userfunc\AndWhere;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.20.5
 * @since 0.8.16
 * @internal #t4182
 */
class FilterAndwhereUserfunc extends AbstractUserfunc
{

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string andWhere statement
	 * @version 0.8.21
	 * @since 0.8.16
	 * @internal #t4182
	 */
	public function AndWhere( $content, $conf )
	{
		unset( $content );

		$andWhere = NULL;
		$andWherePart = NULL;
		$andWhereParts = NULL;

		$this->_conf = $conf;
		$table = $this->_cObj( 'table' );
		$this->_uid = $this->_cObj( 'uid' );
		$this->_unique = '_filter';

		$rows = SqlUtility::getTxXblogFilter( $this->_uid );
		if( empty( $rows ) )
		{
			return;
		}


		foreach( ( array ) $rows as $fields )
		{
			$andWherePart = $this->_andWhere( $table, $fields );
			if( empty( $andWherePart ) )
			{
				continue;
			}
			$andWhereParts[] = $andWherePart;
		}
		$andWhereParts = array_unique( ( array ) $andWhereParts );

		switch( count( $rows ) )
		{
			case(1):
				$andWhere = $andWhereParts;
			default:
				$andWhere = implode( NULL, $andWhereParts );
				break;
		}
		// 191009, dwildt: enable next line, if you like to get a SQL-error-prompt
		//   with the whole query in the frontendend
		//$andWhere = $andWhere . '-DEBUGTHISQUERY';
		//var_dump( __METHOD__, __LINE__, $andWhere );
		//var_dump( __METHOD__, __LINE__, count( $rows ), $rows, $andWhere );
		//die();

		return $andWhere;
	}

	/**
	 * If $MM_match_table_local is empty, there will be a prompt. Process will die!
	 *
	 * @param string $MM_match_table_local
	 * @param string $localTable
	 * @param string $fieldTable
	 * @param string $method
	 * @param string $line
	 * 
	 * @version 0.8.22
	 * @since 0.8.22
	 */
	protected function _promptMatchTableLocalEmptyDie( $MM_match_table_local, $localTable, $fieldTable, $method, $line )
	{
		if( !empty( $MM_match_table_local ) )
		{
			return;
		}
		$prompt = 'ERROR @ ' . $method . ' (#' . $line . ')<br />' . PHP_EOL
						. '<br />' . PHP_EOL
						. 'THIS IS A TODO FOR THE DEVELOPER!<br />' . PHP_EOL
						. '<br />' . PHP_EOL
						. 'Process isn\'t proper without a MM match table local :(' . PHP_EOL
						. '<br />' . PHP_EOL
						. 'MM_match_table_local: ' . $MM_match_table_local . '<br />' . PHP_EOL
						. 'table: ' . $localTable . '<br />' . PHP_EOL
						. 'column: ' . $fieldTable;
		die( $prompt );
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.16
	 * @internal #t4182, #t4183
	 */
	private function _andWhere( $localTable, $fields )
	{
		$fieldAndor = $fields[ 'andor' ];
		if( $fieldAndor == ')' )
		{
			return $fieldAndor;
		}

		$type = $fields[ 'type' ];
		switch( $type )
		{
			case('localtable'):
				return $this->_localtable( $localTable, $fields );
			case(''):
			case('foreigntable'):
				return $this->_foreigntable( $localTable, $fields );
			default:
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): type is "' . $type . '"';
				die( $prompt );
		}
	}

	/**
	 * Get the operator AND, OR or AND (, ) AND (, ) AND, OR (, ) OR (, ) OR
	 *
	 * @param string $fieldAndor 
	 * 
	 * @return string $andor
	 * @version 0.8.21
	 * @since 0.8.21
	 * @internal #t4183
	 */
	private function _andWhereAndOr( $fieldAndor )
	{
		switch( $fieldAndor )
		{
			case(''):
			case('AND'):
				$andor = ' AND ';
				break;
			case('AND ('):
				$andor = ' AND (';
				break;
			case(') AND ('):
				$andor = ') AND (';
				break;
			case(') AND'):
				$andor = ') AND ';
				break;
			case('OR'):
				$andor = ' OR ';
				break;
			case('OR ('):
				$andor = ' OR (';
				break;
			case(') OR ('):
				$andor = ') OR (';
				break;
			case(') OR'):
				$andor = ') OR ';
				break;
			case(')'):
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): case is "' . $fieldAndor . "'" . PHP_EOL
								. 'This isn\'t proper here. Take a look into the code below. Look for "' . $fieldAndor . "'."
				;
				die( $prompt );
			default:
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): case is ' . $fieldAndor;
				die( $prompt );
		}
		return $andor;
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.20.5
	 * @since 0.8.16
	 * @internal #t4182, #t4183
	 * @todo :todo: security
	 */
	private function _andWhereMM( $localTable, $fields )
	{
		$assistent = $fields[ 'assistent' ];

		$fieldAndor = $fields[ 'andor' ];
		// :todo: security
		$fieldExpertsql = $this->_removeQuote( $fields[ 'expertsql' ] );
		$fieldField = $fields[ 'field' ];
		$fieldFormula = $fields[ 'formula' ];
		$fieldOperator = $fields[ 'operator' ];
		// :todo: security
		$fieldPattern = $this->_removeQuote( $fields[ 'pattern' ] );
		$fieldSuggestion = $fields[ 'suggestion' ];
		$fieldSuggestions = $fields[ 'suggestions' ];
		$fieldTable = $fields[ 'table' ];
		// :todo: security
		$fieldValue = $this->_removeQuote( $fields[ 'value' ] );

		if( !isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM' ] ) )
		{
			return;
		}
		//$MM = $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM' ];
		$MMalias = 'MM_' . $fieldTable . $this->_unique;

		// 201018, dwildt, -/+
//		if( !isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'foreign_table' ] ) )
//		{
//			$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): There isn\'t any foreign table :(';
//			die( $prompt );
//		}
//		$foreign_table = $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'foreign_table' ];
		$foreign_table = TcaUtility::getColumnConfigForeigntable( $localTable, $fieldTable );
		if( empty( $foreign_table ) )
		{
			$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): There isn\'t any foreign table :(';
			die( $prompt );
		}

		$MM_match_table_local = NULL;
// :todo: filter match table foreign:
// 191006: Isn't realy needed?
//		$MM_match_table_foreign = NULL;
		if( isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM_match_fields' ] ) )
		{
			if( isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM_match_fields' ][ 'table_local' ] ) )
			{
				$MM_match_table_local = $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM_match_fields' ][ 'table_local' ];
			}
// :todo:  filter match table foreign:
// 191006: Isn't realy needed?
//			if( isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ] ) )
//			{
//				$MM_match_table_foreign = $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $fieldTable ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ];
//			}
		}

		$this->_promptMatchTableLocalEmptyDie( $MM_match_table_local, $localTable, $fieldTable, __METHOD__, __LINE__ );

		$operator = $this->_andWhereOperator( $fieldOperator );

		switch( $operator )
		{
			case('IS NULL'):
			case('IS NOT NULL'):
				$value = '';
				break;
			case('LIKE'):
			case('NOT LIKE'):
				$value = '"' . $fieldPattern . '"';
				break;
			case('IN'):
			case('NOT IN'):
				if( $assistent == 'suggestion' )
				{
					// #t5313, 200530, dwildt, 4+
					if( $fieldSuggestions == '' )
					{
						return;
					}
					$value = ' (' . $fieldSuggestions . ')';
					break;
				}
				// #t5313, 200530, dwildt, 4+
				if( $fieldValue == '' )
				{
					return;
				}
				$value = ' (' . $fieldValue . ')';
				break;
			default:
				if( $assistent == 'suggestion' )
				{
					// #t5449, 200619, dwildt, -2/+1
//					// #t5313, 200530, dwildt, 4+
//					if( $fieldSuggestions == '' )
					if( $fieldSuggestion == '' )
					{
						return;
					}
					$value = $fieldSuggestion;
					break;
				}
				// #t5313, 200530, dwildt, 4+
				if( $fieldValue == '' )
				{
					return;
				}
				$value = '"' . $fieldValue . '"';
				break;
		}

		switch( $assistent )
		{
			case('expert'):
				$tableField = $foreign_table . '.';
				break;
			case('default'):
				$tableField = $foreign_table . '.' . $fieldField;
				break;
			case('formula'):
				$tableField = $foreign_table . '.' . $fieldField;
				break;
			case(''):
			case('suggestion'):
				// :todo: 191007. Check a case with uid_local!
				$tableField = $MMalias . '.uid_foreign';
				break;
			default:
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): assistent is "' . $assistent . '"';
				die( $prompt );
		}

		switch( true )
		{
			case($operator == 'IS NULL'):
			case($operator == 'IS NOT NULL'):
				$operatorValue = ' ' . $operator;
				break;
			case($operator == 'LIKE'):
			case($operator == 'NOT LIKE'):
				$operatorValue = ' ' . $operator . ' ' . $value;
				break;
			case($assistent == 'formula'):
				$operatorValue = ' ' . $fieldFormula;
				break;
			case($assistent == ''):
			case($assistent == 'suggestion'):
				$operatorValue = ' ' . $operator . ' ' . $value;
				break;
			default:
				$operatorValue = ' ' . $operator . ' ' . $value;
				break;
		}

		$andor = $this->_andWhereAndOr( $fieldAndor );
		switch( $assistent )
		{
			case('expert'):
				$statement = $andor . $tableField . $fieldExpertsql;
				break;
			case('default'):
				$statement = $andor . $tableField . $operatorValue;
				break;
			case('formula'):
				$statement = $andor . $tableField . $operatorValue;
				break;
			case(''):
			case('suggestion'):
				// :todo: 191007. Check a case with uid_foreign!
				$statement = $andor . $tableField . $operatorValue;
				break;
			default:
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): assistent is "' . $assistent . '"';
				die( $prompt );
		}

//		var_dump( __METHOD__, __LINE__, $statement );
//		die();
		return $statement;
	}

	/**
	 * Get the comparision operator
	 *
	 * @param string $fieldOperator 
	 * 
	 * @return string $operator
	 * @version 0.8.22
	 * @since 0.8.21
	 */
	private function _andWhereOperator( $fieldOperator )
	{
		switch( $fieldOperator )
		{
			case('eq'):
				$operator = '=';
				break;
			case('neq'):
				$operator = '!=';
				break;
			case('lt'):
				$operator = '<';
				break;
			case('lte'):
				$operator = '<=';
				break;
			case('gt'):
				$operator = '>';
				break;
			case('gte'):
				$operator = '>=';
				break;
			case('isNull'):
				$operator = 'IS NULL';
				break;
			case('isNotNull'):
				$operator = 'IS NOT NULL';
				break;
			case('like'):
				$operator = 'LIKE';
				break;
			case('notLike'):
				$operator = 'NOT LIKE';
				break;
			case('in'):
				$operator = 'IN';
				break;
			case('notIn'):
				$operator = 'NOT IN';
				break;
			default:
// :todo: #t4184
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): case is ' . $fieldOperator;
				die( $prompt );
		}

		//var_dump( __METHOD__, __LINE__, $operator );
		return $operator;
	}

	/**
	 * Get the andWhere statement for the foreigntable
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4182, #t4183
	 */
	private function _foreigntable( $localTable, $fields )
	{
		$assistent = $fields[ 'assistent' ];

		switch( $assistent )
		{
			case('expert'):
				return $this->_foreigntableExpert( $localTable, $fields );
			case('default'):
				return $this->_foreigntableDefault( $localTable, $fields );
			case('formula'):
				return $this->_foreigntableFormula( $localTable, $fields );
			case(''):
			case('suggestion'):
				return $this->_foreigntableSuggestion( $localTable, $fields );
			default:
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): assistent is "' . $assistent . '"';
				die( $prompt );
		}
	}

	/**
	 * Get the andWhere statement for the foreigntable with default mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _foreigntableDefault( $localTable, $fields )
	{
		$statement = $this->_andWhereMM( $localTable, $fields );
//		var_dump( __METHOD__, __LINE__, $statement, $fields );
//		die();
		return $statement;
	}

	/**
	 * Get the andWhere statement for the foreigntable with expert mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _foreigntableExpert( $localTable, $fields )
	{
		$statement = $this->_andWhereMM( $localTable, $fields );
//		var_dump( __METHOD__, __LINE__, $statement, $fields );
//		die();
		return $statement;
	}

	/**
	 * Get the andWhere statement for the foreigntable with formula mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _foreigntableFormula( $localTable, $fields )
	{
		$statement = $this->_andWhereMM( $localTable, $fields );
//		var_dump( __METHOD__, __LINE__, $statement, $fields );
//		die();
		return $statement;
	}

	/**
	 * Get the andWhere statement for the foreigntable with suggestion mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _foreigntableSuggestion( $localTable, $fields )
	{
		$statement = $this->_andWhereMM( $localTable, $fields );
		return $statement;
	}

	/**
	 * Get the andWhere statement for the localtable
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _localtable( $localTable, $fields )
	{
		$assistent = $fields[ 'assistent' ];

		switch( $assistent )
		{
			case('expert'):
				return $this->_localtableExpert( $localTable, $fields );
			case('default'):
				return $this->_localtableDefault( $localTable, $fields );
			case('formula'):
				return $this->_localtableFormula( $localTable, $fields );
			case('suggestion'):
				return $this->_localtableSuggestion( $localTable, $fields );
			default:
				$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . '): assistent is "' . $assistent . '"';
				die( $prompt );
		}
	}

	/**
	 * Get the andWhere statement for the localtable with default mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.9.17
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _localtableDefault( $localTable, $fields )
	{
		$fieldAndor = $fields[ 'andor' ];
		$fieldField = $fields[ 'fieldlocaltable' ];
		$fieldOperator = $fields[ 'operator' ];
		$fieldPattern = $fields[ 'pattern' ];
		$fieldValue = $fields[ 'value' ];

		$andor = $this->_andWhereAndOr( $fieldAndor );
		$operator = $this->_andWhereOperator( $fieldOperator );

		switch( $operator )
		{
			case('LIKE'):
			case('NOT LIKE'):
				$operatorValue = $operator . ' "' . $fieldPattern . '"';
				$statement = $andor . $localTable . '.' . $fieldField . ' ' . $operatorValue;
				break;
			case($operator == 'IS NULL'):
			case($operator == 'IS NOT NULL'):
				// #i0133, 200424, dwildt, ~
				$operatorValue = $operator;
				$statement = $andor . $localTable . '.' . $fieldField . ' ' . $operatorValue;
				break;
			case($operator == 'IN'):
			case($operator == 'NOT IN'):
				$operatorValue = $operator . ' (' . $localTable . '.' . $fieldField . ')';
				$statement = $andor . $fieldValue . ' ' . $operatorValue;
				break;
			default:
				// #i0133, 200424, dwildt, ~
				$operatorValue = $operator . ' ' . $fieldValue;
				$statement = $andor . $localTable . '.' . $fieldField . ' ' . $operatorValue;
				break;
		}

		return $statement;
	}

	/**
	 * Get the andWhere statement for the localtable with expert mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _localtableExpert( $localTable, $fields )
	{
		$fieldAndor = $fields[ 'andor' ];
		$fieldExpertsql = $fields[ 'expertsql' ];

		$andor = $this->_andWhereAndOr( $fieldAndor );

		// :todo: security
		$statement = $andor . $localTable . '.' . $fieldExpertsql;

//		var_dump( __METHOD__, __LINE__, $statement, $fields );
//		die();
		return $statement;
	}

	/**
	 * Get the andWhere statement for the localtable with formula mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _localtableFormula( $localTable, $fields )
	{
		$fieldAndor = $fields[ 'andor' ];
		$fieldField = $fields[ 'fieldlocaltable' ];
		$fieldFormula = $fields[ 'formula' ];

		$andor = $this->_andWhereAndOr( $fieldAndor );

		$statement = $andor . $localTable . '.' . $fieldField . ' ' . $fieldFormula;

//		var_dump( __METHOD__, __LINE__, $statement, $fields );
//		die();
		return $statement;
	}

	/**
	 * Get the andWhere statement for the localtable with suggestion mode
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string andWhere statement
	 * @version 0.8.23
	 * @since 0.8.23
	 * @internal #t4186
	 */
	private function _localtableSuggestion( $localTable, $fields )
	{
		unset( $localTable );
		unset( $fields );

		// Suggestion isn't supported for the localtable!
		return;
	}

	/**
	 * Remove '"' 
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string value withou doublequotes
	 * @version 0.8.24
	 * @since 0.8.24
	 */
	private function _removeDoublequote( $value )
	{
		// 191009, dwildt: Doesnt' make sense. I.e. expression is 'Like "%sword%"'
		//$value = str_replace( '"', null, $value );

		return $value;
	}

	/**
	 * Remove "'" 
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return string value withou doublequotes
	 * @version 0.8.24
	 * @since 0.8.24
	 */
	private function _removeQuote( $value )
	{
		// 191009, dwildt: Doesn't make sense. I.e. expression is 'Like "%sword%"'
		$value = str_replace( "'", null, $value );

		return $value;
	}

}
