<?php

namespace Netzmacher\Xblog\Userfunc\AndWhere;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;
use Netzmacher\Xblog\Userfunc\AndWhere\SwordAndwhereUserfunc;
use Netzmacher\Xblog\Utility\SessionForFormsUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\SqlInjectionUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.10
 * @since 0.9.5
 */
class FormAndwhereUserfunc extends AbstractUserfunc
{

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * @return string andWhere statement
	 * 
	 * @version 0.30.10
	 * @since 0.9.5
	 */
	public function AndWhere( $content, $conf )
	{
		unset( $content );

		$andWhere = NULL;
		$andWherePart = NULL;
		$andWhereParts = NULL;

		$this->_conf = $conf;
		$this->_uid = $this->_cObj( 'uid' );
		$pid = $this->_cObj( 'pid' );
		$pluginUid = $this->_cObj( 'pluginUid' );
		$xBlogPi1Table = $this->_cObj( 'table' );

		if( !empty($pluginUid) && ($pluginUid != $this->_uid) )
		{
			return;
		}

		// RETURN, if there isn't an xBlogForm plugin
		$xBlogFormUid = SqlUtility::getTxXblogFormUidByXblogMain( $this->_uid, $pid );
		if( empty( $xBlogFormUid ) )
		{
			return;
		}

		// RETURN, if there isn't any session data
		$sessionData = SessionForFormsUtility::getValues( $xBlogFormUid );
		if( empty( $sessionData ) )
		{
			return;
		}

		// RETURN, if xBlogForm hasn't any row
		$rows = SqlUtility::getTxXblogFormRows( $xBlogFormUid );
		if( empty( $rows ) )
		{
			return;
		}

		foreach( $rows as $row )
		{
			switch( $row[ 'type' ] )
			{
				case('button'):
					break;
				case('checkbox'):
					$andWherePart = $this->_andWhereTableField( $row[ 'name' ], $sessionData, $xBlogPi1Table );
					break;
				case('placeholder'):
					break;
				case('radio'):
					$andWherePart = $this->_andWhereTableField( $row[ 'name' ], $sessionData, $xBlogPi1Table );
					break;
				case('select'):
					$andWherePart = $this->_andWhereTableField( $row[ 'name' ], $sessionData, $xBlogPi1Table );
					break;
				case('sword'):
					$andWherePart = $this->_andWhereSword( $sessionData, $xBlogPi1Table );
					break;
				default:
					$prompt = 'Undefined value "' . $row[ 'type' ] . '" @' . __METHOD__ . '#' . __LINE__;
					die( $prompt );
			}
			if( empty( $andWherePart ) )
			{
				continue;
			}
			$andWhereParts[] = $andWherePart;
		}
		$andWhereParts = array_unique( ( array ) $andWhereParts );

		switch( count( $rows ) )
		{
			case(1):
				$andWhere = $andWhereParts;
			default:
				$andWhere = implode( NULL, $andWhereParts );
				break;
		}

		return $andWhere;
	}

	/**
	 * _andWhereSword(): AND WHERE statement for a sword
	 *
	 * @param array		$sessionData
	 * @param string	$table
	 * @return string andWhere statement
	 * 
	 * @version 0.30.7
	 * @since		0.30.7
	 */
	private function _andWhereSword( $sessionData, $table )
	{
		if( !in_array( 'sword', array_keys( $sessionData ) ) )
		{
			return;
		}
		$andWherePart = SwordAndwhereUserfunc::WhereSword( $table, $sessionData[ 'sword' ] );
		return $andWherePart;
	}

	/**
	 * _andWhereTableField(): AND WHERE statement for a field
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * @return string andWhere statement
	 * 
	 * @version 0.30.7
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _andWhereTableField( $tableField, $sessionData, $xBlogPi1Table )
	{
		if( !in_array( $tableField, array_keys( $sessionData ) ) )
		{
			return;
		}

		list($table, $field) = explode( '.', $tableField );
		$foreigntable = TcaUtility::getColumnConfigForeigntable( $table, $field );

		switch( true )
		{
			case(empty( $foreigntable )):
				$andWhere = $this->_localtable( $table, $field, $sessionData );
				break;
			case(!TcaUtility::getColumnConfigMM( $table, $field, $foreigntable )):
				$andWhere = $this->_foreigntableWoMM( $table, $field, $foreigntable, $sessionData );
				break;
			case(TcaUtility::getColumnConfigMM( $table, $field, $foreigntable )):
			default:
				$andWhere = $this->_foreigntableWiMM( $table, $field, $foreigntable, $sessionData );
				break;
		}

		return $andWhere;
	}

	/**
	 * Get the andWhere statement for the foreigntable
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * @return string andWhere statement
	 * 
	 * @version 0.9.5
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _foreigntableWiMM( $table, $field, $foreigntable, $sessionData )
	{
		$mmTable = TcaUtility::getColumnConfigMM( $table, $field );

		$mmMatchLocal = TcaUtility::getColumnConfigMMmatchfieldsLocal( $table, $field );
		$mmMatchForeign = TcaUtility::getColumnConfigMMmatchfieldsForeign( $table, $field );

		switch( true )
		{
			case(!empty( $mmMatchLocal . $mmMatchForeign )):
				$statement = $this->_foreigntableWiMMWiMatchFields( $table, $field, $sessionData );
				break;
			case( empty( $mmMatchLocal . $mmMatchForeign )):
			default:
				$statement = $this->_foreigntableWiMMWoMatchFields( );
				break;
		}

		return $statement;
	}

	/**
	 * Get the andWhere statement for the foreigntable
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * @return string andWhere statement
	 * 
	 * @version 0.9.6
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _foreigntableWiMMWiMatchFields( $table, $field, $sessionData )
	{
		$mmTable = TcaUtility::getColumnConfigMM( $table, $field );
		// #t4450.2, 200127, dwildt, ~
		$mmAlias = $mmTable . 'Pi13' . $field;

		$mmMatchLocal = TcaUtility::getColumnConfigMMmatchfieldsLocal( $table, $field );
		$mmMatchForeign = TcaUtility::getColumnConfigMMmatchfieldsForeign( $table, $field );

		$tableField = $table . '.' . $field;
		$value = $sessionData[ $tableField ];

		if( is_array( $value ) )
		{
			return;
		}

		if( $value == '' )
		{
			return;
		}
		$statement = ' AND ' . $mmAlias . '.uid_foreign = ' . ( int ) $value;
		$statement = $statement
						. ' AND ' . $mmAlias . '.table_local = "' . $mmMatchLocal . '"'
						. ' AND ' . $mmAlias . '.table_foreign = "' . $mmMatchForeign . '"'
		;

		return $statement;
	}

	/**
	 * Get the andWhere statement for the foreigntable
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * @return string andWhere statement
	 * 
	 * @version 0.9.5
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _foreigntableWiMMWoMatchFields( )
	{
		$prompt = ''
						. '<div style="background:red;color:white;font-weight:bold;padding:1rem;text-align:center;">'
						. '  Ooops! You are using a MM-relation without match-fields. This isn\'t currently supported.<br />'
						. '  method: ' . __METHOD__ . '@' . __LINE__
						. '</div>'
		;
		echo $prompt;
		//die( $prompt );
	}

	/**
	 * Get the andWhere statement for the foreigntable
	 *
	 * @param string $localTable label of the localTable
	 * @param array $fields fields of the current SQL row
	 * @return string andWhere statement
	 * 
	 * @version 0.9.5
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _foreigntableWoMM( $table, $field, $foreigntable, $sessionData )
	{
		$foreignAlias = $foreigntable . 'Pi13';
		$tableField = $table . '.' . $field;
		$csvValues = self::_valuesToCsv( $sessionData[ $tableField ] );
		if( empty( $csvValues ) )
		{
			return;
		}

		$statement = ' AND ' . $foreignAlias . '.uid IN (' . $csvValues . ')';
		//var_dump( __METHOD__, __LINE__, $statement );

		return $statement;
	}

	/**
	 * Get the andWhere statement for the localtable
	 *
	 * @param string	$table
	 * @param string	$field
	 * @param array		$sessionData
	 * @return string andWhere statement
	 * 
	 * @version 0.30.7
	 * @since 0.30.7
	 */
	private function _localtable( $table, $field, $sessionData )
	{
		$type = TcaUtility::getColumnConfigType( $table, $field );

		switch( $type )
		{
			case('check'):
			case('radio'):
			case('select'):
				$statement = $this->_localtableTcaTypeCheckRadioSelect( $table, $field, $sessionData );
				break;
			case('input'):
			case('text'):
				$statement = $this->_localtableTcaTypeInputText( $table, $field, $sessionData );
				break;
			case('group'):
			case('inline'):
				$statement = $this->_localtableTcaTypeGroupInline( $table, $field, $sessionData );
				break;
			default:
				$this->_localtableTcaTypeUndefined( $table, $field, $sessionData );
				break;
		}

		return $statement;
	}

	/**
	 * Get the andWhere statement for the localtable
	 *
	 * @param string	$table
	 * @param string	$field
	 * @param array		$sessionData
	 * @return string andWhere statement
	 * 
	 * @version 0.30.7
	 * @since 0.30.7
	 */
	private function _localtableTcaTypeCheckRadioSelect( $table, $field, $sessionData )
	{
		$tableField = $table . '.' . $field;
		$values = $sessionData[ $tableField ];
		var_dump( __METHOD__, __LINE__, $tableField, $type, array_keys( $sessionData ), $values );
	}

	/**
	 * Get the andWhere statement for the localtable
	 *
	 * @param string	$table
	 * @param string	$field
	 * @param array		$sessionData
	 * @return string andWhere statement
	 * 
	 * @version 0.30.7
	 * @since 0.30.7
	 */
	private function _localtableTcaTypeInputText( $table, $field, $sessionData )
	{
		$tableField = $table . '.' . $field;
		$values = $sessionData[ $tableField ];

		if( !is_array( $values ) )
		{
			$value = SqlInjectionUtility::String( $value );
			$statement = ' AND ' . $table . '.' . $field . ' LIKE "' . $value . '"';
			return $statement;
		}

		foreach( ( array ) $values as $value )
		{
			$value = SqlInjectionUtility::String( $value );
			$orParts[] = '(' . $table . '.' . $field . ' LIKE "' . $value . '")';
		}
		$statement = ' AND (' . implode( ' OR ', $orParts ) . ')';
		return $statement;
	}

	/**
	 * Get the andWhere statement for the foreigntable
	 *
	 * @param mixed $values 
	 * @param string $varType 'int' || 'string'
	 * @return string andWhere statement
	 * 
	 * @version 0.9.5
	 * @since 0.9.5
	 */
	private function _valuesToCsv( $values )
	{
		// $values isn't an array
		if( !is_array( $values ) )
		{
			$value = SqlInjectionUtility::String( $values );
			$intValues[] = $value;
			$csvValues = implode( ',', $intValues );
			return $csvValues;
		}

		// $values is an array
		foreach( $values as $value )
		{
			$value = SqlInjectionUtility::String( $value );
			$intValues[] = ( int ) $value;
		}

		$csvValues = implode( ',', $intValues );
		return $csvValues;
	}

}
