<?php

namespace Netzmacher\Xblog\Userfunc\AndWhere;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;
use Netzmacher\Xblog\Utility\SqlUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.7
 * @since 0.25.1
 */
class SingleviewAndwhereUserfunc extends AbstractUserfunc
{

	/**
	 * @var array	typoscript configuration
	 */
	public $_conf;

	/**
	 * @var string
	 */
	private $_debugUid;

	/**
	 * @var string [empty] | first | selected
	 */
	private $_mode;

	/**
	 * @var string like showUid, calUid, nesUid, ...
	 */
	private $_showUid;

	/**
	 * @var string current table
	 */
	private $_table;

	/**
	 * @var integer		uid of the selected record given by the URL
	 */
	private $_uidByUrl;

	/**
	 * @var integer		uid of the selected record given by the flexform
	 */
	private $_uidByFlexform;

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string andWhere statement
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	public function AndWhere( $content, $conf )
	{
		unset( $content );

		$andWhere = NULL;

		$this->_conf = $conf;

		$this->_table = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_table' ];
		$this->_mode = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_record_mode' ];
		$this->_showUid = $GLOBALS[ 'TSFE' ]->register[ 'settings_flexform_pi1_profile_showUid' ];
		$this->_uidByFlexform = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_record_uid' ];
		$this->_uidByUrl = $GLOBALS[ 'TSFE' ]->register[ 'xBlogPi1Uid' ];
		$this->_debugUid = $this->_cObj( 'debugUid' );

		$this->_requirements();

		switch( $this->_mode )
		{
			case('first'):
				$andWhere = $this->_andWhereUidFirst();
				break;
			case('selected'):
				$andWhere = ' AND ' . $this->_table . '.uid = ' . $this->_uidByFlexform;
				break;
			default:
				$andWhere = $this->_andWhereByUrl();
				break;
		}

		return $andWhere;
	}

	/**
	 * _andWhereUidFirst()
	 *
	 * @return string andWhere statement
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	private function _andWhereUidFirst()
	{
		$table = $this->_table;
		$recursive = $GLOBALS[ 'TSFE' ]->register[ 'pi1__recursive' ];
		$pages = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pages' ];
		$max = 1;

		$rows = SqlUtility::getRecordsByTable( $table, $pages, $recursive, $max );
		$uid = $rows[ 0 ][ 'uid' ];

		$andWhere = ' AND ' . $this->_table . '.uid = ' . $uid;
//		var_dump( __METHOD__, __LINE__, $andWhere, $uid, $rows, $table, $recursive, $pages, $GLOBALS[ 'TSFE' ]->register );
//		die();
		return $andWhere;
	}

	/**
	 * _andWhereByUrl()
	 *
	 * @return string andWhere statement
	 * @version 0.30.7
	 * @since 0.25.1
	 */
	private function _andWhereByUrl()
	{
		if( !$this->_uidByUrl )
		{
			$params = GeneralUtility::_GP( 'tx_xblog_pi1' );
//			if( empty( $params ) )
//			{
//				$params = GeneralUtility::_GP( 'tx_xblog_pi15' );
//			}
			$this->_uidByUrl = ( int ) $params[ $this->_showUid ];
		}

		$andWhere = ' AND ' . $this->_table . '.uid = ' . $this->_uidByUrl;
//		var_dump( __METHOD__, __LINE__, $_GET, $_POST, $params, $this->_uidByUrl );
//		die();
		return $andWhere;
	}

	/**
	 * _requirements()
	 * 
	 * @return void
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	private function _requirements()
	{
		if( empty( $this->_mode ) )
		{
			return;
		}

		if( $this->_mode == 'first' )
		{
			return;
		}

		if( $this->_mode != 'selected' )
		{
			$this->_requirementsDie();
		}

		if( !empty( $this->_uidByFlexform ) )
		{
			return;
		}

		$this->_requirementsDie();
	}

	/**
	 * _requirementsDie()
	 * 
	 * @return void
	 * @version 0.25.1
	 * @since 0.25.1
	 */
	private function _requirementsDie()
	{

		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  There isn\'t a proper configuration. Maybe a field is empty'
						. '</h2>'
						. '<ul>'
						. '  <li>'
						. '    table = "' . $this->_table . '"'
						. '  </li>'
						. '  <li>'
						. '    mode = "' . $this->_mode . '" <- must be one of first or selected'
						. '  </li>'
						. '  <li>'
						. '    uidByFlexform = "' . $this->_uidByFlexform . '" <- can be empty, if mode is "first"'
						. '  </li>'
						. '</ul>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $this->_debugUid . '<br />'
						. '</p>'
		;
		die( $prompt );
	}

}
