<?php

namespace Netzmacher\Xblog\Userfunc\AndWhere;

use Netzmacher\Xblog\Userfunc\SysCategoryAbstractUserfunc;
use Netzmacher\Xblog\Utility\SqlUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.21.16
 * @since 0.20.33
 * @internal #t4182
 */
class SysCategoryAndwhereUserfunc extends SysCategoryAbstractUserfunc
{

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string andWhere statement
	 * @version 0.21.16
	 * @since 0.20.33
	 */
	public function AndWhere( $content, $conf )
	{
		unset( $content );

		$andWhere = NULL;

		$this->_conf = $conf;
		$table = $this->_cObj( 'table' );
		$enabled = $this->_cObj( 'enabled' );
		$field = $this->_cObj( 'field' );
		$uid = $this->_cObj( 'uid' );
		$andOr = $this->_cObj( 'andOr' );
		$debugUid = $this->_cObj( 'debugUid' );
		$pid = $this->_cObj( 'pid' );

//		var_dump( __METHOD__, __LINE__, $conf, $table, $uid, $enabled, $field, $andOr, $pid, $debugUid );
//		die();

		if( empty( $field ) || empty( $enabled ) )
		{
			return null;
		}

		$rows = SqlUtility::getXblogSysCategories( $uid );
		if( empty( $rows ) )
		{
			return;
		}

		$andWhere = $this->_andWhere( $table, $field, $rows, $andOr );

		unset( $debugUid );
		unset( $pid );

//		var_dump( __METHOD__, __LINE__, $andWhere, $debugUid );
//		die();

		return $andWhere;
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $rows values from the flexform
	 * @param string $andOr 
	 * 
	 * @return string andWhere statement
	 * @version 0.20.33
	 * @since 0.20.33
	 */
	private function _andWhere( $localTable, $field, $rows, $andOr )
	{
		foreach( ( array ) $rows as $row )
		{
			$sysCategoryUids[] = $row[ 'uid_local' ];
		}

		// Must correspond with SysCategoryJoinUserfunc::_andWhere
		switch( $andOr )
		{
			case('and'):
				$andWhereStatement = self::_andWhereAnd( $localTable, $field, $sysCategoryUids );
				break;
			case('notAnd'):
				$andWhereStatement = self::_andWhereNotAnd( $localTable, $field, $sysCategoryUids );
				break;
			case('notOr'):
				$andWhereStatement = self::_andWhereNotOr( $localTable, $field, $sysCategoryUids );
				break;
			case('or'):
				$andWhereStatement = self::_andWhereOr( $sysCategoryUids );
				break;
			default:
				$debugUid = $this->_cObj( 'debugUid' );
				$prompt = 'FATAL ERROR: Value for switch isn\'t defined.<br />'
								. 'Value is "' . $andOr . '"<br />'
								. '<br />'
								. '  Error occurs @<br />'
								. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
								. '  TypoScript: ' . $debugUid . '<br />'
				;
				die( $prompt );
		}

		return $andWhereStatement;
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $sysCategoryUids list of uids of current sys-categories
	 * 
	 * @return string andWhere statement
	 * @version 0.20.33
	 * @since 0.20.33
	 */
	private function _andWhereAnd( $localTable, $field, $sysCategoryUids )
	{
		// Get all records, which are conected with all sys_categories
		$uids = SqlUtility::getTableUidsSysCategoryAnd( $localTable, $field, $sysCategoryUids );
		$strUids = implode( ',', ( array ) $uids );

		$andWhere = ' AND ' . $localTable . '.uid IN (' . $strUids . ')';

//		var_dump( __METHOD__, __LINE__, $uids, $strUids, $andWhere );
//		die();

		return $andWhere;
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $sysCategoryUids list of uids of current sys-categories
	 * 
	 * @return string andWhere statement
	 * @version 0.20.33
	 * @since 0.20.33
	 */
	private function _andWhereNotAnd( $localTable, $field, $sysCategoryUids )
	{
		// Get all records, which are conected with all sys_categories
		$uids = SqlUtility::getTableUidsSysCategoryAnd( $localTable, $field, $sysCategoryUids );
		$strUids = implode( ',', ( array ) $uids );

		$andWhere = ' AND ' . $localTable . '.uid NOT IN (' . $strUids . ')';

//		var_dump( __METHOD__, __LINE__, $uids, $strUids, $andWhere );
//		die();

		return $andWhere;
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $sysCategoryUids list of uids of current sys-categories
	 * 
	 * @return string andWhere statement
	 * @version 0.20.33
	 * @since 0.20.33
	 */
	private function _andWhereNotOr( $localTable, $field, $sysCategoryUids )
	{
		// Get all records, which are conected with all sys_categories
		$uids = SqlUtility::getTableUidsSysCategoryOr( $localTable, $field, $sysCategoryUids );
		$strUids = implode( ',', ( array ) $uids );

		$andWhere = ' AND ' . $localTable . '.uid NOT IN (' . $strUids . ')';

//		var_dump( __METHOD__, __LINE__, $uids, $strUids, $andWhere );
//		die();

		return $andWhere;
	}

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $localTable label of the localTable
	 * @param array $sysCategoryUids list of uids of current sys-categories
	 * 
	 * @return string andWhere statement
	 * @version 0.20.33
	 * @since 0.20.33
	 */
	private function _andWhereOr( $sysCategoryUids )
	{
		foreach( $sysCategoryUids as $sysCategoryUid )
		{
			$andWherePart[] = 'sys_category_record_mm.uid_local = ' . ( int ) $sysCategoryUid;
		}

		if( empty( $andWherePart ) )
		{
			return;
		}

		switch( true )
		{
			case(count( $andWherePart ) == 1):
				$andWhereStatement = $andWherePart[ 0 ];
				break;
			case(count( $andWherePart ) > 1):
			default:
				$andWhereStatement = '(' . implode( ') OR (', $andWherePart ) . ')';
				break;
		}

		$andWhere = ' AND ' . $andWhereStatement;

//		var_dump( __METHOD__, __LINE__, $sysCategoryUids, $andWhere );
//		die();

		return $andWhere;
	}

}
