<?php

namespace Netzmacher\Xblog\Userfunc\AndWhere;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.20.62
 * @since 0.20.62
 * @internal #i0051
 */
class TopitemAndwhereUserfunc extends AbstractUserfunc
{

	/**
	 * Get the andWhere statement like AND ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string andWhere statement
	 * @version 0.20.62
	 * @since 0.20.62
	 */
	public function AndWhere( $content, $conf )
	{
		unset( $content );

		$andWhere = NULL;

		$this->_conf = $conf;

		$this->_requirements();

		$table = $this->_cObj( 'table' );
		$field = $this->_cObj( 'field' );
		$condition = $this->_cObj( 'condition' );
		

//		var_dump( __METHOD__, __LINE__, $conf, $table, $field, $condition );
//		die();

		if( !$condition || empty( $field ) )
		{
			return;
		}

		$andWhere = ' AND ' . $table . '.' . $field . ' = 1 ';

//		var_dump( __METHOD__, __LINE__, $andWhere );
//		die();

		return $andWhere;
	}

	/**
	 * _requirements()
	 * 
	 * @return void
	 * @version 0.20.62
	 * @since 0.20.62
	 */
	private function _requirements()
	{
		$table = $this->_cObj( 'table' );
		$debugUid = $this->_cObj( 'debugUid' );

		switch( true )
		{
			case(empty( $table )):
				$this->_requirementsDie( $debugUid, $table );
				die( __METHOD__ . '#' . __LINE__ );
			default:
				return;
		}
	}

	/**
	 * _requirementsDie()
	 * 
	 * @return void
	 * @version 0.20.62
	 * @since 0.20.62
	 */
	private function _requirementsDie( $debugUid, $table )
	{
		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  There is an empty field'
						. '</h2>'
						. '<p>'
						. '  table = "' . $table . '"<br />'
						. '</p>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $debugUid . '<br />'
						. '</p>'
		;
		die( $prompt );
	}

}
