<?php

namespace Netzmacher\Xblog\Userfunc\Join;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.19
 * @since		0.30.19
 */
class AbstractjoinUserfunc extends AbstractUserfunc
{

	/**
	 * JOIN statement for a foreign table with an MM relation
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $unique	unique string identifier
	 * 
	 * @return string JOIN statement
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	protected function JoinWiMM( $table, $field, $unique = null )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM' ] ) )
		{
			return;
		}
		$MM = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM' ];
		$MMalias = 'MM_' . $field . $unique;
		$table_unique = $table . $unique;

		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ] ) )
		{
			$_join = $MM . ' AS ' . $MMalias . ' ON ' . $MMalias . '.uid_local = ' . $table_unique . '.uid';
			return $_join;
		}

		$table_local = $this->_joinWiMMAndTablelocal( $table, $field, $MMalias );
		$table_foreign = $this->_joinWiMMAndTableforeign( $table, $field, $MMalias, $unique );

		$_join = ''
						. $MM . ' AS ' . $MMalias
						. ' ON ' . $MMalias . '.uid_local = ' . $table . '.uid' . $table_local . $table_foreign;
		return $_join;
	}

	/**
	 * JOIN statement for a foreign table without an MM relation
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $unique	unique string identifier
	 * 
	 * @return	string JOIN statement
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	protected function JoinWoMM( $localTable, $field, $unique = null )
	{
		$foreignTable = TcaUtility::getColumnConfigForeigntable( $localTable, $field );
		switch( true )
		{
			case(TcaUtility::isMM( $localTable, $field )):
			case(empty( $foreignTable )):
			case($foreignTable == 'sys_file_reference'):
				return;
			default:
				// follow the workflow
				break;
		}

		$foreignAlias = $foreignTable . $unique;
		$join = $foreignTable . ' AS ' . $foreignAlias . ' ON ' . $foreignAlias . '.uid IN (' . $localTable . '.' . $field . ')';

		return $join;
	}

	/**
	 * AND statement for table_foreign of an MM relation
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $MMalias
	 * 
	 * @return string AND statement
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _joinWiMMAndTableforeign( $table, $field, $MMalias, $unique )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ] ) )
		{
			return;
		}

		$tcaTableforeign = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ];
		$tcaTableforeignUnique = $tcaTableforeign . $unique;
		$table_foreign = ''
						. ' AND ' . $MMalias . '.table_foreign = "' . $tcaTableforeign . '"'
						. ' INNER JOIN ' . $tcaTableforeign . ' AS ' . $tcaTableforeignUnique
						. ' ON ' . $MMalias . '.uid_foreign = ' . $tcaTableforeignUnique . '.uid'
		;
		return $table_foreign;
	}

	/**
	 * AND statement for table_local of an MM relation
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $MMalias
	 * 
	 * @return string AND statement
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _joinWiMMAndTablelocal( $table, $field, $MMalias )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ][ 'table_local' ] ) )
		{
			return;
		}

		$tcaTablelocal = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ][ 'table_local' ];
		$table_local = ' AND ' . $MMalias . '.table_local = "' . $tcaTablelocal . '"';
		return $table_local;
	}

}
