<?php

namespace Netzmacher\Xblog\Userfunc\Join;

use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.19
 * @since		0.8.16
 */
class FilterjoinUserfunc extends AbstractjoinUserfunc
{

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string JOIN statement
	 * @version 0.8.21
	 * @since 0.8.16
	 * @internal #t4182
	 */
	public function Join( $content, $conf )
	{
		unset( $content );

		$join = NULL;
		$joinParts = NULL;

		$this->_conf = $conf;
		$table = $this->_cObj( 'table' );
		$uid = $this->_cObj( 'uid' );
		$unique = '_filter';

		$rows = SqlUtility::getTxXblogFilter( $uid );
		if( empty( $rows ) )
		{
			return;
		}

		$joinParts = $this->_join( $table, $rows, $unique );
		if( empty( $joinParts ) )
		{
			return;
		}

		$join = implode( ' INNER JOIN ', $joinParts );

		if( $GLOBALS[ 'TSFE' ]->register[ 'join' ] )
		{
			$join = ' INNER JOIN ' . $join;
		}

		$GLOBALS[ 'TSFE' ]->register[ 'join' ] = true;
		return $join;
	}

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string	$table
	 * @param array		$rows
	 * @param string	$unique
	 * 
	 * @return string INNER JOIN statement
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	private function _join( $table, $rows, $unique )
	{
		$joinParts = array_merge(
						( array ) $this->_joinWiMM( $table, $rows, $unique )
						, ( array ) $this->_joinWoMM( $table, $rows, $unique )
		);
		$joinParts = array_unique( ( array ) $joinParts );
		return $joinParts;
	}

	/**
	 * Check conditions for a join
	 *
	 * @param array $fields fields of the current SQL row
	 * 
	 * @return boolean
	 * @version 0.8.22
	 * @since 0.8.22
	 */
	private function _joinCondition( $fields )
	{
		$fieldOperator = $fields[ 'operator' ];
		// #i0131, 200424, dwildt, 1+
		$fieldType = $fields[ 'type' ];
		$foreignUid = $fields[ 'value' ];

		// #i0131, 200424, dwildt, 4+
		if( $fieldType != 'foreigntable' )
		{
			return false;
		}

		switch( true )
		{
			case(empty( $fieldType = 'isNull' )):
				return false;
			case($fieldOperator = 'isNull'):
			case($fieldOperator = 'isNotNull'):
				return true;
			case(empty( $foreignUid )):
				return false;
			default:
				return true;
		}
	}

	/**
	 * Get the join statement for a relation with an MM table
	 *
	 * @param string	$table
	 * @param array		$rows
	 * @param string	$unique
	 * 
	 * @return string INNER JOIN statement
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	private function _joinWiMM( $table, $rows, $unique )
	{
		foreach( ( array ) $rows as $fields )
		{
			if( !$this->_joinCondition( $fields ) )
			{
				continue;
			}
			$field = $fields[ 'table' ];
			if( !TcaUtility::isMM( $table, $field ) )
			{
				continue;
			}
			$joinPart = $this->_joinWiMMRow( $table, $field, $unique );
			if( empty( $joinPart ) )
			{
				continue;
			}
			$joinParts[] = $joinPart;
		}
		return $joinParts;
	}

	/**
	 * Get the join statement for the foreign table with an MM table
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $unique
	 * 
	 * @return string JOIN statement
	 * @version 0.30.19
	 * @since 0.8.16
	 */
	private function _joinWiMMRow( $table, $field, $unique )
	{
		$_join = $this->JoinWiMM( $table, $field, $unique );
		return $_join;
	}

	/**
	 * Get the join statement for a relation without an MM table
	 *
	 * @param string	$table
	 * @param array		$rows
	 * @param string	$unique
	 * 
	 * @return string INNER JOIN statement
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	private function _joinWoMM( $table, $rows, $unique )
	{
		foreach( ( array ) $rows as $fields )
		{
			if( !$this->_joinCondition( $fields ) )
			{
				continue;
			}
			$field = $fields[ 'table' ];
			$joinPart = $this->_joinWoMMRow( $table, $field, $unique );
			if( empty( $joinPart ) )
			{
				continue;
			}
			$joinParts[] = $joinPart;
		}
		return $joinParts;
	}

	/**
	 * Get the join statement for the foreign table without an MM table
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $unique
	 * 
	 * @return string JOIN statement
	 * @version 0.30.19
	 * @since 0.8.22
	 */
	private function _joinWoMMRow( $table, $field, $unique )
	{
		$_join = $this->JoinWoMM( $table, $field, $unique );
		return $_join;
	}

}
