<?php

namespace Netzmacher\Xblog\Userfunc\Join;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;
use Netzmacher\Xblog\Utility\SessionForFormsUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.19
 * @since 0.9.5
 */
class FormjoinUserfunc extends AbstractUserfunc
{

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string JOIN statement
	 * @version 0.30.10
	 * @since 0.9.4
	 * @internal #t4450
	 */
	public function Join( $content, $conf )
	{
		unset( $content );
		unset( $GLOBALS[ 'TSFE' ]->register[ 'join' ] );

		$join = NULL;
		$joinPart = NULL;
		$joinParts = NULL;

		$this->_conf = $conf;
		$this->_uid = $this->_cObj( 'uid' );
		$pid = $this->_cObj( 'pid' );
		$pluginUid = $this->_cObj( 'pluginUid' );

		if( !empty( $pluginUid ) && ($pluginUid != $this->_uid) )
		{
			return;
		}

		//$xBlogPi1Uid = $this->_cObj( 'xBlogPi1Uid' );
//		var_dump( __METHOD__, __LINE__, $this->_uid, $xBlogPi1Uid);
		// RETURN, if there isn't an xBlogForm plugin
		$xBlogFormUid = SqlUtility::getTxXblogFormUidByXblogMain( $this->_uid, $pid );
		//var_dump( __METHOD__, __LINE__, $this->_uid, $xBlogFormUid);
		if( empty( $xBlogFormUid ) )
		{
			return;
		}

		// RETURN, if there isn't any session data
		$sessionData = SessionForFormsUtility::getValues( $xBlogFormUid );
		if( empty( $sessionData ) )
		{
			return;
		}

		//var_dump( __METHOD__, __LINE__, $sessionData );
		// RETURN, if xBlogForm hasn't any row
		$rows = SqlUtility::getTxXblogFormRows( $xBlogFormUid );
		if( empty( $rows ) )
		{
			return;
		}

		foreach( $rows as $row )
		{
			if( !in_array( $row[ 'name' ], array_keys( $sessionData ) ) )
			{
				continue;
			}
			// #t6183, 210626, dwildt, 4+
			if( empty( $sessionData[ $row[ 'name' ] ] ) )
			{
				continue;
			}
			$joinPart = $this->_getJoinPartByTablefield( $row[ 'name' ] );
			if( empty( $joinPart ) )
			{
				continue;
			}
			$joinParts[] = $joinPart;
		}
		$joinParts = array_unique( ( array ) $joinParts );

		if( empty( $joinParts ) )
		{
			return;
		}

		switch( count( $joinParts ) )
		{
			case(1):
				$join = $joinParts;
			default:
				$join = implode( ' INNER JOIN ', $joinParts );
				break;
		}

//		var_dump( __METHOD__, __LINE__, $join );
//		var_dump( __METHOD__, __LINE__, count( $rows ), $rows, $join );
//		die();

		$GLOBALS[ 'TSFE' ]->register[ 'join' ] = true;
		return $join;
	}

	/**
	 * 
	 *
	 * @param string $tableField
	 * @return string 
	 * 
	 * @version 0.30.19
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _getJoinPartByTablefield( $tableField )
	{
		$join = null;

		list($table, $field) = explode( '.', $tableField );
		$foreigntable = TcaUtility::getColumnConfigForeigntable( $table, $field );
//		var_dump( __METHOD__, __LINE__, $tableField, $foreigntable );

		if( empty( $foreigntable ) )
		{
			return;
		}

		switch( true )
		{
			case(!TcaUtility::getColumnConfigMM( $table, $field, $foreigntable )):
				$join = $this->_joinWoMM( $table, $field, $foreigntable );
				break;
			case(TcaUtility::getColumnConfigMM( $table, $field, $foreigntable )):
			default:
				$join = $this->_joinWiMM( $table, $field );
				break;
		}

		//var_dump( __METHOD__, __LINE__, $table, $foreigntable, $join );
		return $join;
	}

	/**
	 * Get the join statement for the foreign table
	 *
	 * @param string $localtable
	 * @param string $foreigntable
	 * @return string Part for a JOIN statement
	 * 
	 * @version 0.9.5
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _joinWoMM( $localtable, $field, $foreigntable )
	{
		$foreignAlias = $foreigntable . 'Pi13';

		$join = $foreigntable . ' AS ' . $foreignAlias . ' ON ' . $localtable . '.' . $field . ' = ' . $foreignAlias . '.uid';

//		var_dump( __METHOD__, __LINE__, $join );
//		die();
		return $join;
	}

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string $localtable label of the localTable
	 * @param array $field fields of the current SQL row
	 * 
	 * @return string JOIN statement
	 * @version 0.30.19
	 * @since 0.9.5
	 * @internal #t4450
	 */
	private function _joinWiMM( $localtable, $field )
	{
		$mmTable = TcaUtility::getColumnConfigMM( $localtable, $field );
		// #t4450.2, 200127, dwildt, ~
		$mmAlias = $mmTable . 'Pi13' . $field;

		if( !TcaUtility::getColumnConfigMMmatchfields( $localtable, $field ) )
		{
			$join = $mmTable . ' AS ' . $mmAlias . ' ON ' . $mmAlias . '.uid_local = ' . $localtable . '.uid';
			return $join;
		}

		$andTableLocal = NULL;
		if( TcaUtility::getColumnConfigMMmatchfieldsLocal( $localtable, $field ) )
		{
			$andTableLocal = ' AND ' . $mmAlias . '.table_local = "' . TcaUtility::getColumnConfigMMmatchfieldsLocal( $localtable, $field ) . '"';
		}
		$andTableForeign = NULL;
		if( TcaUtility::getColumnConfigMMmatchfieldsForeign( $localtable, $field ) )
		{
			$andTableForeign = ' AND ' . $mmAlias . '.table_foreign = "' . TcaUtility::getColumnConfigMMmatchfieldsForeign( $localtable, $field ) . '"';
		}

		$join = $mmTable . ' AS ' . $mmAlias . ' ON ' . $mmAlias . '.uid_local = ' . $localtable . '.uid' . $andTableLocal . $andTableForeign;

//		var_dump( __METHOD__, __LINE__, $join );
//		die();
		return $join;
	}

}
