<?php

namespace Netzmacher\Xblog\Userfunc\Join;

use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version		0.30.19
 * @since			0.30.17
 */
class OrderjoinUserfunc extends AbstractjoinUserfunc
{

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string JOIN statement
	 * @version 0.30.19
	 * @since 0.30.17
	 */
	public function Join( $content, $conf )
	{
		unset( $content );

		$join = NULL;
		$joinParts = NULL;

		$this->_conf = $conf;
		$debugUid = $this->_cObj( 'debugUid' );
		$list_type = $this->_cObj( 'list_type' );
		$this->_uid = $this->_cObj( 'uid' );
		$unique = '_tablelayout';

		if( $list_type != 'xblog_pi1' )
		{
			return;
		}

		$externalTemplate = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate' ];
		if( $externalTemplate != 'tablelayout' )
		{
			return;
		}

		$rows = $this->_getRowsFromTxXblogDomainModelTablelayout();
		if( empty( $rows ) )
		{
			return;
		}

		$table = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_table' ];
		$joinParts = $this->_join( $table, $rows, $unique );
		if( empty( $joinParts ) )
		{
			return;
		}

		$join = implode( ' INNER JOIN ', $joinParts );

		if( $GLOBALS[ 'TSFE' ]->register[ 'join' ] )
		{
			$join = ' INNER JOIN ' . $join;
		}

		$GLOBALS[ 'TSFE' ]->register[ 'join' ] = true;
		return $join;
	}

	/**
	 * Get the join statement for the foreign table
	 *
	 * @return array	$rows Configuration rows of table columns 
	 * @version 0.30.17
	 * @since 0.30.17
	 */
	private function _getRowsFromTxXblogDomainModelTablelayout()
	{
		$table = 'tx_xblog_domain_model_tablelayout';
		$pages = null;
		$depth = 0;
		$where = [];
		$where[ 'AND' ][] = [
				'eq' =>
				[
						'tt_content' => $this->_uid
				]
		];

		$rows = SqlUtility::Query( $table, $pages, $depth, $where );
		return $rows;
	}

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string	$table
	 * @param array		$rows
	 * @param string	$unique
	 * 
	 * @return string INNER JOIN statement
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	private function _join( $table, $rows, $unique )
	{
		$joinParts = array_merge(
						( array ) $this->_joinWiMM( $table, $rows, $unique )
						, ( array ) $this->_joinWoMM( $table, $rows, $unique )
		);
		$joinParts = array_unique( ( array ) $joinParts );
		return $joinParts;
	}

	/**
	 * Get the join statement for a relation with an MM table
	 *
	 * @param string	$table
	 * @param array		$rows
	 * @param string	$unique
	 * 
	 * @return string INNER JOIN statement
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	private function _joinWiMM( $table, $rows, $unique )
	{
		foreach( ( array ) $rows as $row )
		{
			$field = $row[ 'field' ];
			if( !TcaUtility::isMM( $table, $field ) )
			{
				continue;
			}
			$joinPart = $this->_joinWiMMRow( $table, $field, $unique );
			if( empty( $joinPart ) )
			{
				continue;
			}
			$joinParts[] = $joinPart;
		}
		return $joinParts;
	}

	/**
	 * Get the join statement for the foreign table with an MM table
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $unique
	 * 
	 * @return string JOIN statement
	 * @version	0.30.19
	 * @since		0.30.17
	 */
	private function _joinWiMMRow( $table, $field, $unique )
	{
		$_join = $this->JoinWiMM( $table, $field, $unique );
		return $_join;
	}

	/**
	 * Get the join statement for a relation without an MM table
	 *
	 * @param string	$table
	 * @param array		$rows
	 * @param string	$unique
	 * 
	 * @return string INNER JOIN statement
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	private function _joinWoMM( $table, $rows, $unique )
	{
		foreach( ( array ) $rows as $row )
		{
			$field = $row[ 'field' ];
			$joinPart = $this->_joinWoMMRow( $table, $field, $unique );
			if( empty( $joinPart ) )
			{
				continue;
			}
			$joinParts[] = $joinPart;
		}
		return $joinParts;
	}

	/**
	 * Get the join statement for the foreign table without an MM table
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $unique
	 * 
	 * @return string INNER JOIN statement
	 * @version 0.30.19
	 * @since 0.30.17
	 */
	private function _joinWoMMRow( $table, $field, $unique )
	{
		$_join = $this->JoinWoMM( $table, $field, $unique );
		return $_join;
	}

}
