<?php

namespace Netzmacher\Xblog\Userfunc\Join;

use Netzmacher\Xblog\Userfunc\SysCategoryAbstractUserfunc;
use Netzmacher\Xblog\Utility\SqlUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
	 * @version 0.21.16
 * @since 0.20.33
 * @internal #t4182
 */
class SysCategoryJoinUserfunc extends SysCategoryAbstractUserfunc
{

	/**
	 * Get the join statement like tx_org_mm_all ON tx_org_mm_all.uid_local = tx_org_news.uid
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string JOIN statement
	 * @version 0.21.16
	 * @since 0.20.33
	 * @internal #t4182
	 */
	public function Join( $content, $conf )
	{
		unset( $content );

		$join = NULL;

		$this->_conf = $conf;
		$table = $this->_cObj( 'table' );
		$enabled = $this->_cObj( 'enabled' );
		$field = $this->_cObj( 'field' );
		$uid = $this->_cObj( 'uid' );
		$andOr = $this->_cObj( 'andOr' );

//		var_dump( __METHOD__, __LINE__, $conf, $table, $uid, $enabled, $field, $andOr, $debugUid );
//		die();

		if( empty( $field ) || empty( $enabled ) )
		{
			return null;
		}

		$rows = SqlUtility::getXblogSysCategories( $uid );
		if( empty( $rows ) )
		{
			return;
		}

		$join = $this->_join( $table, $field, $andOr );

//		var_dump( __METHOD__, __LINE__, $join );
//		die();

		return $join;
	}

	/**
	 * Get the join statement
	 *
	 * @return string JOIN statement
	 * @version 0.20.33
	 * @since 0.20.33
	 */
	private function _join( $localTable, $field, $andOr )
	{
		// Must correspond with SysCategoryAndwhereUserfunc::_andWhere
		switch( $andOr )
		{
			case('and'):
				$join = null;
				break;
			case('notAnd'):
				$join = null;
				break;
			case('notOr'):
				$join = null;
				break;
			case('or'):
				$join = ''
								. 'sys_category_record_mm '
								. 'ON ' . $localTable . '.uid = sys_category_record_mm.uid_foreign  '
								. 'AND sys_category_record_mm.tablenames = "' . $localTable . '" '
								. 'AND sys_category_record_mm.fieldname = "' . $field . '" '
				;
				break;
			default:
				$prompt = 'FATAL ERROR: Value for switch isn\'t defined.<br />'
								. 'Value is "' . $andOr . '"<br />'
								. '<br />'
								. '@' . __METHOD__ . '#' . __LINE__
				;
				die( $prompt );
		}

//		var_dump( __METHOD__, __LINE__, $join );
//		die();
		return $join;
	}

}
