<?php

namespace Netzmacher\Xblog\Userfunc;

use Netzmacher\Xblog\Userfunc\AbstractUserfunc;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version		0.30.15
 * @since			0.30.15
 */
class OrderbyUserfunc extends AbstractUserfunc
{

	public $_conf;
	private $_debugUid;
	private $_listType;
	private $_orderDirection01;
	private $_orderDirection02;
	private $_orderDirection03;
	private $_orderField01;
	private $_orderField02;
	private $_orderField03;
	private $_table;
	private $_uid;

	/**
	 * Get the ORDER BY statement like ...
	 *
	 * @param string $content (empty)
	 * @param array $conf typoscript configuration of current object
	 * 
	 * @return string ORDER BY statement
	 * @version 0.30.15
	 * @since 0.30.15
	 */
	public function OrderBy( $content, $conf )
	{
		unset( $content );

		$orderBy = NULL;

		$this->_conf = $conf;
		$this->_debugUid = $this->_cObj( 'debugUid' );
		$this->_listType = $this->_cObj( 'list_type' );
		$this->_uid = $this->_cObj( 'uid' );

		$this->_setFields();
		$this->_requirements();
		$orderBy = $this->_orderBy();

		return $orderBy;
	}

	/**
	 * _dieCondition()
	 * 
	 * @return	void
	 * @version 0.30.13
	 * @since		0.30.13
	 */
	private function _dieListType()
	{
		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  Switch case is undefined'
						. '</h2>'
						. '<p>'
						. '  list_type: ' . $this->_listType . ' isn\'t respected<br />'
						. '</p>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $this->_debugUid . '<br />'
						. '</p>'
		;
		die( $prompt );
	}

	/**
	 * _dieTable()
	 * 
	 * @return void
	 * @version 0.25.2
	 * @since 0.20.55
	 */
	private function _dieTable()
	{
		$prompt = ''
						. '<h1>'
						. '  FATAL ERROR'
						. '</h1>'
						. '<h2>'
						. '  There is an empty field'
						. '</h2>'
						. '<p>'
						. '  table = "' . $this->_table . '"<br />'
						. '</p>'
						. '<p>'
						. '  Error occurs @<br />'
						. '  PHP: ' . __METHOD__ . '#' . __LINE__ . '<br />'
						. '  TypoScript: ' . $this->_debugUid . '<br />'
						. '</p>'
		;
		die( $prompt );
	}

	/**
	 * Get the join statement for the foreign table
	 *
	 * @param integer $uid Uid of the current 
	 * 
	 * @return string JOIN statement
	 * @version 0.30.17
	 * @since 0.30.17
	 */
	private function _getRowsFromTxXblogDomainModelTablelayout()
	{
		$table = 'tx_xblog_domain_model_tablelayout';
		$pages = null;
		$depth = 0;
		$where = [];
		$where[ 'AND' ][] = [
				'eq' =>
				[
						'tt_content' => $this->_uid
				]
		];
		$orderBy = TcaUtility::getCtrlSortbyOrDefaultsortby( $table );

		$rows = SqlUtility::Query( $table, $pages, $depth, $where, $orderBy );
		return $rows;
	}

	/**
	 * _orderBy()
	 * 
	 * @return	void
	 * @version 0.30.17
	 * @since		0.30.13
	 */
	private function _orderBy()
	{
		$externalTemplate = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_tmpl_externalTemplate' ];
		switch( true )
		{
			case($externalTemplate == 'tablelayout' && $this->_listType == 'xblog_pi1'):
				$orderBy = $this->_orderByTablelayout();
				break;
			default:
				$orderBy = $this->_orderByList();
				break;
		}
		return $orderBy;
	}

	/**
	 * _orderByList()
	 * 
	 * @return	void
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	private function _orderByList()
	{
		$orderBy = $this->_orderByList01();
		$orderBy = $this->_orderByList02( $orderBy );
		$orderBy = $this->_orderByList03( $orderBy );

		//var_dump( __METHOD__, __LINE__, $orderBy );

		return $orderBy;
	}

	/**
	 * _orderByList01()
	 * 
	 * @return	string	$orderBy
	 * @version 0.30.13
	 * @since		0.30.13
	 */
	private function _orderByList01()
	{
		if( empty( $this->_orderField01 ) )
		{
			return;
		}

		$orderBy = $this->_table . '.' . $this->_orderField01 . ' ' . $this->_orderDirection01;
		return $orderBy;
	}

	/**
	 * _orderByList02()
	 * 
	 * @param		string	$orderBy
	 * @return	string	$orderBy
	 * @version 0.30.13
	 * @since		0.30.13
	 */
	private function _orderByList02( $orderBy )
	{
		switch( true )
		{
			case(empty( $orderBy )):
			case(empty( $this->_orderField02 )):
				return $orderBy;
			default:
				// follow the workflow
				break;
		}

		$orderBy = $orderBy . ', ' . $this->_table . '.' . $this->_orderField02 . ' ' . $this->_orderDirection02;
		return $orderBy;
	}

	/**
	 * _orderByList03()
	 * 
	 * @param		string	$orderBy
	 * @return	string	$orderBy
	 * @version 0.30.13
	 * @since		0.30.13
	 */
	private function _orderByList03( $orderBy )
	{
		switch( true )
		{
			case(empty( $orderBy )):
			case(empty( $this->_orderField03 )):
				return $orderBy;
			default:
				// follow the workflow
				break;
		}

		$orderBy = $orderBy . ', ' . $this->_table . '.' . $this->_orderField03 . ' ' . $this->_orderDirection03;
		return $orderBy;
	}

	/**
	 * _orderByTablelayout()
	 * 
	 * @return	void
	 * @version 0.30.23
	 * @since		0.30.17
	 */
	private function _orderByTablelayout()
	{
		$rows = $this->_getRowsFromTxXblogDomainModelTablelayout();
		foreach( ( array ) $rows as $row )
		{
			$tableForeign = $this->_getTableforeign( $row );
			if( empty( $tableForeign ) )
			{
				$tableForeigns[] = trim( $this->_table . '.' . $row[ 'field' ] . ' ' . $row[ 'order' ] );
				continue;
			}
			$tableField = $tableForeign . '_tablelayout.' . TcaUtility::getCtrlLabel( $tableForeign );
			$tableForeigns[] = trim( $tableField . ' ' . $row[ 'order' ] );
		}

		if( empty( $tableForeigns ) )
		{
			return;
		}

		$orderBy = implode( ', ', $tableForeigns );
		//var_dump( __METHOD__, __LINE__, $orderBy );

		return $orderBy;
	}

	/**
	 * ORDER statement: get the foreign_table
	 *
	 * @param array $row
	 * @return string $table_foreign
	 * @version 0.30.23
	 * @since 0.30.17
	 */
	private function _getTableforeign( $row )
	{
		$field = $row[ 'field' ];
		$localTable = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_table' ];

		switch( true )
		{
			case(isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $field ][ 'config' ][ 'foreign_table' ] )):
				$table_foreign = $this->_getTableforeignCSV( $localTable, $field );
				break;
			default:
				$table_foreign = $this->_getTableforeignMM( $localTable, $field );
		}

		return $table_foreign;
	}

	/**
	 * ORDER statement: get the foreign_table in case of a CSV relation
	 *
	 * @param string $localTable
	 * @param string $field
	 * @return string $foreign_table
	 * @version 0.30.23
	 * @since		0.30.23
	 */
	private function _getTableforeignCSV( $localTable, $field )
	{
		$foreign_table = $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $field ][ 'config' ][ 'foreign_table' ];
		if( $foreign_table == 'sys_file_reference' )
		{
			return;
		}
		return $foreign_table;
	}

	/**
	 * ORDER statement: get the foreign_table in case of a MM-relation
	 *
	 * @param string $localTable
	 * @param string $field
	 * @return string $table_foreign
	 * @version 0.30.23
	 * @since		0.30.17
	 */
	private function _getTableforeignMM( $localTable, $field )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ] ) )
		{
			return;
		}

		if( !isset( $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ] ) )
		{
			return;
		}

		$table_foreign = $GLOBALS[ 'TCA' ][ $localTable ][ 'columns' ][ $field ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ];

		return $table_foreign;
	}

	/**
	 * _requirements()
	 * 
	 * @return void
	 * @version 0.25.2
	 * @since 0.20.55
	 */
	private function _requirements()
	{
		switch( true )
		{
			case(empty( $this->_table )):
				$this->_dieTable();
			default:
				return;
		}
	}

	/**
	 * _setFields()
	 * 
	 * @return	void
	 * @version 0.30.13
	 * @since		0.30.13
	 */
	private function _setFields()
	{
		switch( $this->_listType )
		{
			case('xblog_pi1'):
				$this->_orderField01 = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_selectOrderField' ];
				$this->_orderDirection01 = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_selectOrderDirection' ];
				$this->_orderField02 = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_selectOrderField02' ];
				$this->_orderDirection02 = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_selectOrderDirection02' ];
				$this->_orderField03 = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_selectOrderField03' ];
				$this->_orderDirection03 = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_selectOrderDirection03' ];
				$this->_table = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi_table_table' ];
				break;
			case('xblog_pi2'): // Mini
				$this->_orderField01 = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_selectOrderField' ];
				$this->_orderDirection01 = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_selectOrderDirection' ];
				$this->_orderField02 = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_selectOrderField02' ];
				$this->_orderDirection02 = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_selectOrderDirection02' ];
				$this->_orderField03 = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_selectOrderField03' ];
				$this->_orderDirection03 = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_selectOrderDirection03' ];
				$this->_table = $GLOBALS[ 'TSFE' ]->register[ 'pi2__pi_flexform__settings_flexform_pi_table_table' ];
				break;
			case('xblog_pi11'): // Slider
				$this->_orderField01 = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_selectOrderField' ];
				$this->_orderDirection01 = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_selectOrderDirection' ];
				$this->_orderField02 = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_selectOrderField02' ];
				$this->_orderDirection02 = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_selectOrderDirection02' ];
				$this->_orderField03 = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_selectOrderField03' ];
				$this->_orderDirection03 = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_selectOrderDirection03' ];
				$this->_table = $GLOBALS[ 'TSFE' ]->register[ 'pi11__pi_flexform__settings_flexform_pi_table_table' ];
				break;
			case('xblog_pi12'): // RSS
				$this->_orderField01 = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_selectOrderField' ];
				$this->_orderDirection01 = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_selectOrderDirection' ];
				$this->_orderField02 = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_selectOrderField02' ];
				$this->_orderDirection02 = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_selectOrderDirection02' ];
				$this->_orderField03 = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_selectOrderField03' ];
				$this->_orderDirection03 = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_selectOrderDirection03' ];
				$this->_table = $GLOBALS[ 'TSFE' ]->register[ 'pi12__pi_flexform__settings_flexform_pi_table_table' ];
				break;
			case('xblog_pi14'): // Teaser
				$this->_orderField01 = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_selectOrderField' ];
				$this->_orderDirection01 = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_selectOrderDirection' ];
				$this->_orderField02 = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_selectOrderField02' ];
				$this->_orderDirection02 = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_selectOrderDirection02' ];
				$this->_orderField03 = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_selectOrderField03' ];
				$this->_orderDirection03 = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_selectOrderDirection03' ];
				$this->_table = $GLOBALS[ 'TSFE' ]->register[ 'pi14__pi_flexform__settings_flexform_pi_table_table' ];
				break;
			default:
				$this->_dieListType();
		}
	}

}
