<?php

namespace Netzmacher\Xblog\Userfunc;

use Netzmacher\Xblog\Utility\SessionFeUserUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.7
 * @since 0.6.2
 */
class PointerUserfunc extends AbstractUserfunc
{

	/**
	 * @var array current configuration array from TypoScript of the calling userFunc 
	 */
	public $_conf;

	/**
	 * @var integer limit for select statement
	 */
	private $_flexformLimit;

	/**
	 * Begin():
	 *
	 * @param  string          Empty string (no content to process)
	 * @param  array           TypoScript configuration
	 * @return integer         pointer, the begin value of a select statement
	 * @access public
	 * @version 0.20.17
	 * @since 0.4.0
	 */
	public function Begin( $content, $conf )
	{
//		var_dump( __METHOD__, __LINE__, $this->cObj->data );
//		die();
		unset( $content );
		$this->_conf = $conf;

		$this->_getFlexformValues();

		$begin = $this->_getBegin();
		if( $begin < 1 )
		{
			return $begin;
		}

//		var_dump( __METHOD__, __LINE__, $begin, $this->_flexformLimit );
		$begin = $begin * $this->_flexformLimit;
		return $begin;
	}

	/**
	 * _getBegin():
	 *
	 * @return integer         pointer, the begin value of a select statement
	 * @access private
	 * @version 0.30.7
	 * @since 0.4.0
	 */
	private function _getBegin()
	{
		$params = GeneralUtility::_GP( 'tx_xblog_pi1' );
//		if( empty( $params ) )
//		{
//			$params = GeneralUtility::_GP( 'tx_xblog_pi15' );
//		}

		if( empty( $params ) )
		{
//			var_dump( __METHOD__, __LINE__ );
			return null;
		}

		if( !isset( $params[ 'pointer' ] ) )
		{
			$pointer = $this->_getPointerFromSession();
		}
		else
		{
			$pointer = ( int ) $params[ 'pointer' ];
		}

		return $pointer;
	}

	/**
	 * _getFlexformValues():
	 *
	 * @return void
	 * @access private
	 * @version 0.8.12
	 * @since 0.4.0
	 */
	private function _getFlexformValues()
	{
		$this->_flexformLimit = $this->_getFlexformValue( 'table', 'settings.flexform.pi.table.max' );
	}

	/**
	 * _getPointerFromSession( ) :
	 *
	 * @return integer	$pointer
	 * @internal Must correspond with AbstractController::getPointerFromSession
	 * @access private
	 * @version 0.20.17
	 * @since 0.20.17
	 */
	private function _getPointerFromSession()
	{
		$uid = $this->contentObject->data[ 'uid' ];
		$arrSessionData = SessionFeUserUtility::Get( 'tx_xblog' );

		$pointer = $arrSessionData[ $uid ][ 'pointer' ];
//		var_dump(__METHOD__, __LINE__, $arrSessionData, $arrSessionData[ $uid ], $pointer);
		return $pointer;
	}

}
