<?php

namespace Netzmacher\Xblog\Userfunc;

use Netzmacher\Xblog\Utility\SessionFeUserUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\Typo3VersionUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.20.17
 * @since 0.3.0
 */
class SessionUserfunc extends AbstractUserfunc
{

	/**
	 * @var array
	 */
	public $_conf;

	/**
	 * @var string
	 */
	private $_promptParam = ''
					. '<h2>'
					. '	userFunc = Netzmacher\Xblog\Userfunc\SessionUserfunc->SetXblogVar'
					. '</h2>'
					. '<p>'
					. '	Params are:'
					. '</p>'
					. '<ul>'
					. '	<li>'
					. '		uid'
					. '	</li>'
					. '	<li>'
					. '		param'
					. '	</li>'
					. '	<li>'
					. '		value'
					. '	</li>'
					. '	<li>'
					. '		devUid (optional)'
					. '	</li>'
					. '	<li>'
					. '		dontClearCache (optional)'
					. '</li>'
					. '</ul>'

	;

	/**
	 * SetXblogVar():
	 *
	 * @param  string          Empty string (no content to process)
	 * @param  array           TypoScript configuration
	 * @return void
	 * @version 0.3.0
	 * @since 0.3.0
	 */
	public function SetXblogVar( $content, $conf )
	{
		unset( $content );
		$this->_conf = $conf;

		$this->_promptDevUid();

		if( !$this->_requirements() )
		{
			return;
		}

		$this->_clearCache();
		$this->_storeSessionData();

		return;
	}

// #t5442.3, #t5442.3, 200704, dwildt, -
//	/**
//	 * SetXblogVarExclusive():
//	 *
//	 * @param  string          Empty string (no content to process)
//	 * @param  array           TypoScript configuration
//	 * @return void
//	 * @version 0.3.0
//	 * @since 0.3.0
//	 */
//	public function SetXblogVarExclusive( $content, $conf )
//	{
//		if( !$this->_pluginExclusive() )
//		{
//			return;
//		}
//		return $this->SetXblogVar( $content, $conf );
//	}

	/**
	 * _clearCache( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.10.1
	 * @since 0.3.0
	 */
	private function _clearCache()
	{
		$dontClearCache = $this->_cObj( 'dontClearCache' );
		switch( true )
		{
			case( $dontClearCache == 1 ):
			case( $dontClearCache == true ):
				return;
			case(empty( $dontClearCache )):
			case( $dontClearCache == 0 ):
			case( $dontClearCache == false ):
			default:
			// follow the workflow;
		}

		switch( true )
		{
			case(Typo3VersionUtility::Get() < 10000000):
				#t5192, 200512, dwildt, ~
				SqlUtility::Truncate( 'cf_cache_pages' );
				SqlUtility::Truncate( 'cf_cache_pagesection' );
				break;
			case(Typo3VersionUtility::Get() >= 10000000):
			default:
				#t5230, 200516, dwildt, 2+
				SqlUtility::Truncate( 'cache_pages' );
				SqlUtility::Truncate( 'cache_pagesection' );
				break;
		}
	}

	/**
	 * _requirements( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.6.0
	 * @since 0.6.0
	 */
	private function _requirements()
	{
		switch( true )
		{
			case(!$this->_extManagerEnabledSessionManagement() ):
			case( $this->_unproperParams() ):
				return false;
			default:
				return true;
		}
	}

	/**
	 * _storeSessionData( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.20.17
	 * @since 0.5.2
	 */
	private function _storeSessionData()
	{
		$uid = $this->_cObj( 'uid' );
		$param = $this->_cObj( 'param' );
		$value = $this->_cObj( 'value' );

		$arrSessionData = SessionFeUserUtility::Get( 'tx_xblog' );

		if( $value === '' )
		{
			if( isset( $arrSessionData[ $uid ][ $param ] ) )
			{
				unset( $arrSessionData[ $uid ][ $param ] );
			}
			if( empty( $arrSessionData[ $uid ] ) )
			{
				unset( $arrSessionData[ $uid ] );
			}
			SessionFeUserUtility::Set( 'tx_xblog', $arrSessionData );
			//var_dump( __METHOD__, __LINE__, $value );
			return;
		}

		$arrSessionData[ $uid ][ $param ] = $value;
		SessionFeUserUtility::Set( 'tx_xblog', $arrSessionData );
		return;
	}

	/**
	 * _unproperParams( ) :
	 *
	 * @return void
	 * @access private
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _unproperParams()
	{
		$this->_unproperParam( 'uid', $this->_promptParam );
		$this->_unproperParam( 'param', $this->_promptParam );
	}

}
