<?php

namespace Netzmacher\Xblog\Utility;

/**
 * This file is part of the "xBlog" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code. 
 */
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * ExternalTemplate utility class
 * 
 * @version 0.9.11
 * @since 0.9.2
 * @internal #t4744
 */
class ExternalTemplateUtility implements SingletonInterface
{

	/**
	 * Get available template layouts for a certain page
	 *
	 * @param int $pageUid
	 * @return array
	 * @version 0.9.11
	 * @since 0.9.2
	 * @internal #t4744
	 */
	public function getAvailableExternalTemplates( $pageUid )
	{
		$templateLayouts = [];

		// Add TsConfig values
		foreach( $this->getExternalTemplatesFromTsConfig( $pageUid ) as $templateKey => $title )
		{
			if( GeneralUtility::isFirstPartOfStr( $title, '--div--' ) )
			{
				$optGroupParts = GeneralUtility::trimExplode( ',', $title, true, 2 );
				$title = $optGroupParts[ 1 ];
				$templateKey = $optGroupParts[ 0 ];
			}
			$templateLayouts[] = [ $title, $templateKey ];
		}

		return $templateLayouts;
	}

	/**
	 * Get template layouts defined in TsConfig
	 *
	 * @param $pageUid
	 * @return array
	 * @version 0.9.11
	 * @since 0.9.2
	 * @internal #t4744
	 */
	protected function getExternalTemplatesFromTsConfig( $pageUid )
	{
		$templateLayouts = [];
		$pagesTsConfig = BackendUtility::getPagesTSconfig( $pageUid );
		if( isset( $pagesTsConfig[ 'tx_xblog.' ][ 'templateLayouts.' ] ) && is_array( $pagesTsConfig[ 'tx_xblog.' ][ 'templateLayouts.' ] ) )
		{
			$templateLayouts = $pagesTsConfig[ 'tx_xblog.' ][ 'templateLayouts.' ];
		}
		return $templateLayouts;
	}

}
