<?php

namespace Netzmacher\Xblog\Utility;

use Netzmacher\Xblog\Utility\SqlUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.22.0
 * @since 0.22.0
 */
class RegisterUtility
{

	/**
	 * _getArrFlexformData():
	 *
	 * @param  array   $row: row of a tt_content record. Record should be record of an xBlog.
	 * @return array		
	 * @access private
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	private static function _getArrFlexformData( $row )
	{
		$xmlFlexform = $row[ 'pi_flexform' ];
		if( empty( $xmlFlexform ) )
		{
			return;
		}

		$arrFlexform = GeneralUtility::xml2array( $xmlFlexform );

		return $arrFlexform[ 'data' ];
	}

	/**
	 * _setRegisterData():
	 *
	 * @param  string $plugin:	name of the plugin. I.e. pi1
	 * @param  array $row:			current tt_content row. Should be a row of an xBlog.
	 * @return void
	 * @access private
	 * @since 0.23.0
	 * @version 0.20.10
	 */
	private static function _setRegisterData( $plugin, $row )
	{
		self::_setRegisterDataField( $plugin, $row, 'list_type' );
		self::_setRegisterDataField( $plugin, $row, 'pages' );
		self::_setRegisterDataField( $plugin, $row, 'pid' );
		self::_setRegisterDataField( $plugin, $row, 'recursive' );
		self::_setRegisterDataField( $plugin, $row, 'uid' );
	}

	/**
	 * _setRegisterDataField():
	 *
	 * @param  string $plugin:	name of the plugin. I.e. pi1
	 * @param  array $row:			current tt_content row. Should be a row of an xBlog.
	 * @return void
	 * @access private
	 * @since 0.23.0
	 * @version 0.23.0
	 */
	private static function _setRegisterDataField( $plugin, $row, $field )
	{
		$key = $plugin . '__' . $field;
		$GLOBALS[ 'TSFE' ]->register[ $key ] = $row[ $field ];
	}

	/**
	 * _getArrFlexformData():
	 *
	 * @param  string $plugin:		name of the plugin. I.e. pi1
	 * @param  string $register:	name of the register
	 * @param  string $value:			value of the register
	 * @return void
	 * @access private
	 * @version 0.30.0
	 * @since 0.22.0
	 */
	private static function _setRegisterFlexformUnderlined( $plugin, $register, $value )
	{
		$key = $plugin . '__pi_flexform__' . $register;
		$key = str_replace( '.', '_', $key );
		//echo $key . ': ' . $value . PHP_EOL;
		$GLOBALS[ 'TSFE' ]->register[ $key ] = $value;
	}

	/**
	 * _setRegisterFlexformWiDots():
	 *
	 * @param  string $register:	name of the register
	 * @param  string $value:			value of the register
	 * @return void
	 * @access private
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	private static function _setRegisterFlexformWiDots( $register, $value )
	{
		$GLOBALS[ 'TSFE' ]->register[ $register ] = $value;
	}

	/**
	 * setRegister():
	 *
	 * @param  integer	$uid: Uid of the current plugin
	 * @param  string		$plugin: name of the plugin
	 * @param  array		TypoScript configuration
	 * @return void
	 * @access public
	 * @version 0.22.0
	 * @since 0.22.0
	 */
	public static function setRegister( $uid, $plugin )
	{
		$row = SqlUtility::getTtcontentRowByUid( $uid );

		if( empty( $row ) )
		{
			$prompt = 'FATAL ERROR: $row is empty. '
							. 'Error occured while using destination xBlog plugin with the uid #' . $uid . '. '
							. 'Likely the xBlog plugin is hidden or removed. '
			;
			throw new \RuntimeException( $prompt, 1615735647 );
		}

		$arrFlexformData = self::_getArrFlexformData( $row );

		if( empty( $arrFlexformData ) )
		{
			$prompt = '<div style="background:red;color:white;font-weight:bold;padding:1rem;text-align:center;">'
							. '  FATAL ERROR: Form is empty.<br />'
							. '  SOLUTION: Please save the xBlog form once. Or enable the hidden form.<br />'
							. '  CAUSE: Likely the xBlog plugin was never saved, target xBlog is hidden or removed.<br />'
							. '  <br />'
							. '  Error occured while using destination xBlog plugin with the uid #' . $uid . '.<br />'
							. '  method: ' . __METHOD__ . '@' . __LINE__
							. '</div>'
			;
			echo $prompt;
			throw new \RuntimeException( $prompt, 1615735783 );
		}

		foreach( $arrFlexformData as $elements )
		{
			foreach( $elements[ 'lDEF' ] as $element => $setting )
			{
				self::_setRegisterFlexformUnderlined( $plugin, $element, $setting[ 'vDEF' ] );
				self::_setRegisterFlexformWiDots( $element, $setting[ 'vDEF' ] );
			}
		}

		self::_setRegisterData( $plugin, $row );

//		if( $plugin == 'pi11' )
//		{
//			var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register );
//			die();
//		}
	}

}
