<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\Utility;

use Netzmacher\Xblog\Utility\Typo3VersionUtility;

/**
 * Class SessionFeUserUtility
 * 
 * @version 0.20.17
 * @internal #t5183
 */
class SessionFeUserUtility
{

	/**
	 * Read session data
	 *
	 * @param string $sessionId
	 * @return array Returns the session data
	 */
	public static function Get( string $sessionId )
	{
		$space = self::_getSessionSpace();
		$arrSessionData = $GLOBALS[ 'TSFE' ]->fe_user->getKey( $space, $sessionId );

		return $arrSessionData;
	}

	/**
	 * Set session data
	 *
	 * @param string $sessionId
	 * @param array $arrSessionData
	 * @return void
	 */
	public static function Set( string $sessionId, $arrSessionData )
	{
		$space = self::_getSessionSpace();
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( $space, $sessionId, $arrSessionData );
		self::_storeSessionData();
	}

	/**
	 * Store session data
	 *
	 * @return void
	 * @version 0.20.17
	 */
	private static function _storeSessionData()
	{
		switch( true )
		{
			case(Typo3VersionUtility::Get() < 10000000):
				$GLOBALS[ "TSFE" ]->storeSessionData();
				break;
			case(Typo3VersionUtility::Get() >= 10000000):
			default:
				$GLOBALS[ "TSFE" ]->fe_user->storeSessionData();
				break;
		}
	}

	/**
	 * _getSessionSpace( ): Get the name of the session data space.
	 *                      The name is user, if a frontend user is logged in.
	 *                      The name is ses, if any frontend user isn't logged in.
	 *
	 * @return	string		user || ses
	 */
	private static function _getSessionSpace()
	{
		switch( self::_loginUser() )
		{
			case( true ):
				return 'user';
			default:
				return 'ses';
		}
	}

	/**
	 * _loginUser
	 *
	 * @return boolean
	 * @internal #t5184
	 */
	private static function _loginUser()
	{
		switch( true )
		{
			case(Typo3VersionUtility::Get() < 10000000):
				return $GLOBALS[ 'TSFE' ]->loginUser;
			case(Typo3VersionUtility::Get() >= 10000000):
			default:
				return $GLOBALS[ 'TSFE' ]->fe_user->loginUser;
		}
	}

}
