<?php

//declare(strict_types = 1);

namespace Netzmacher\Xblog\Utility;

use Netzmacher\Xblog\Utility\SessionFeUserUtility;
use Netzmacher\Xblog\Utility\SqlUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class ConfigurationUtility
 * 
 * @version 0.20.17
 * 
 */
class SessionForFormsUtility extends AbstractUtility
{

	/**
	 * Set session data for the current form
	 *
	 * @param integer uid		uid of the current form
	 * @return array sessionData
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	private static function _getValues( $uid )
	{
		$arrSessionData = SessionFeUserUtility::Get( 'tx_xblog' );

		return $arrSessionData[ $uid ];
	}

	/**
	 * Set session data for the current form
	 *
	 * @param array params		GET/POST parameters
	 * @param integer uid		uid of the current form
	 * @param string key			
	 * @return void
	 * @version 0.20.17
	 * @since 0.9.3
	 */
	private static function _setValue( $params, $uid, $key )
	{
		if( empty( $key ) )
		{
			return;
		}
		$arrSessionData = SessionFeUserUtility::Get( 'tx_xblog' );

		$value = $params[ $key ];
		if( $value === '' )
		{
			if( isset( $arrSessionData[ $uid ][ $key ] ) )
			{
				unset( $arrSessionData[ $uid ][ $key ] );
			}
			if( empty( $arrSessionData[ $uid ] ) )
			{
				unset( $arrSessionData[ $uid ] );
			}
			SessionFeUserUtility::Set( 'tx_xblog', $arrSessionData );
			return;
		}

		$arrSessionData[ $uid ][ $key ] = $value;
		SessionFeUserUtility::Set( 'tx_xblog', $arrSessionData );
		return;
	}

	/**
	 * Set session data for the current form
	 *
	 * @param integer uid		uid of the current form
	 * @param array rows		the elments of the current form
	 * @return void
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	private static function _setValues( $uid, $rows )
	{
		if( empty( $rows ) )
		{
			return;
		}

		$paramsPi13 = GeneralUtility::_GP( 'tx_xblog_pi13' );
		if( $paramsPi13[ 'uid' ] != $uid )
		{
			return;
		}

		foreach( $rows as $row )
		{
			switch( $row[ 'type' ] )
			{
				case('checkbox'):
				case('radio'):
				case('select'):
				case('selectperiod'):
				case('text'):
					self::_setValue( $paramsPi13, $uid, $row[ 'name' ] );
					break;
				case('sword'):
					self::_setValue( $paramsPi13, $uid, 'sword' );
					break;
				case('button'):
				case('placeholder'):
					break;
				default:
					$prompt = 'BUG: key in switch isn\'t definded.';
					var_dump( __METHOD__, __LINE__, $prompt );
					die();
			}
		}
	}

	/**
	 * Get session data from the current form
	 *
	 * @param integer uid of the current form plugin
	 * @return array sessionData
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	public static function getValues( $uid )
	{
		return self::_getValues( $uid );
	}

	/**
	 * Set session data for the current form
	 *
	 * @param integer uid of the current form plugin
	 * @param integr $xBlogMainUid
	 * @return void
	 * @version 0.9.10
	 * @since 0.9.3
	 */
	public static function setValues( $uid )
	{
		$rows = SqlUtility::getTxXblogFormRows( $uid );
		self::_setValues( $uid, $rows );
	}

}
