<?php

namespace Netzmacher\Xblog\Utility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.30.19
 * @since 0.7.2
 */
class SqlInjectionUtility
{

	/**
	 * String():
	 *
	 * @param	string		$value            : value
	 * @param	boolean		$strip_tags       : process strip_tags
	 * @param	boolean		$htmlspecialchars : process $htmlspecialchars
	 * @param	boolean		$quoteStr         : process $quoteStr
	 * @return	string  $value
	 * @version 0.30.19
	 * @since   0.7.2
	 * @access public
	 */
	public static function String( $value, $strip_tags = true, $htmlspecialchars = true, $quoteStr = true )
	{
		if( $strip_tags )
		{
			$value = strip_tags( $value );
		}

		if( $htmlspecialchars )
		{
			$value = htmlspecialchars( $value );
		}

		if( $quoteStr )
		{
			// #t5192, 200514, dwildt, 1-,+ 
			//$value = $GLOBALS[ 'TYPO3_DB' ]->quoteStr( $value, $table );
			$value = self::_mysql_escape_mimic( $value );
		}

		return $value;
	}

	/*
	 * _mysql_escape_mimic():
	 * 
	 * @param	string		$value            : value
	 * @return	string	
	 * @version 0.10.1
	 * @since   0.9.10
	 * https://www.php.net/manual/de/function.mysql-real-escape-string.php
	 * 
	 */

	private static function _mysql_escape_mimic( $value )
	{
		if( is_array( $value ) )
		{
			return array_map( __METHOD__, $value );
		}

		if( empty( $value ) )
		{
			return $value;
		}

		//var_dump( __METHOD__, __LINE__, $value );
		$value = str_replace( array( '\\', "\0", "\n", "\r", "'", '"', "\x1a" ), array( '\\\\', '\\0', '\\n', '\\r', "\\'", '\\"', '\\Z' ), $value );
		//var_dump( __METHOD__, __LINE__, $value );

		return $value;
	}

}
