<?php

namespace Netzmacher\Xblog\Utility;

use Netzmacher\Xblog\Utility\FlashmessageUtility;
use Netzmacher\Xblog\Utility\FlexformValueUtility;
use Netzmacher\Xblog\Utility\TcaUtility;
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\QueryBuilder;
use TYPO3\CMS\Core\Database\Query\QueryHelper;
use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
use TYPO3\CMS\Core\Database\QueryGenerator;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * 
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  xblog
 *
 * @version 0.31.0
 * @since 0.8.16
 */
class SqlUtility extends AbstractUtility
{

	/**
	 * Query():
	 *
	 * @param string	$table		table
	 * @param string	$pages		list of pages
	 * @param integer	$depth		levels
	 * @param string	$where		SQL WHERE statemant
	 * @param string	$orderBy	SQL ORDERBY statemant
	 * @param string	$begin		1st part of LIMIT
	 * @param string	$max			2nd part of LIMIT
	 * @return array	row from database
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public static function Query( $table, $pages, $depth = 0, $where = '', $orderBy = '', $begin = 0, $max = 0 )
	{
		$pidList = self::_getPidList( $pages, $depth );
//		$pidList = self::_getPidList( $cObj, $pages, $depth );
//		var_dump( __METHOD__, __LINE__, $table, $pages, $depth, $pidList );
//		die();

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( $table );

		$queryBuilder
						->select( '*' )
						->from( $table )
						->where(
										$queryBuilder->expr()->in( 'pid', $pidList )
						)
		;

		if( !empty( $where ) )
		{
			$whereExpressions = self::_whereExpressionsBuilder( $queryBuilder, $where );
			$queryBuilder->where( ...$whereExpressions );
		}

		$queryBuilder = self::_orderBy( $queryBuilder, $orderBy );

		if( $begin )
		{
			$queryBuilder->setFirstResult( $begin );
		}
		if( $max )
		{
			$queryBuilder->setMaxResults( $max );
		}
		$rows = $queryBuilder
						->execute()
						->fetchAll()
		;

//		debug( $queryBuilder->getSQL() );
//		debug( $queryBuilder->getParameters() );
		return $rows;
	}

	/**
	 * QueryBuilder()
	 * 
	 * @param string $table
	 * @return QueryBuilder
	 * 
	 * @version	0.10.1
	 * @since	0.10.1
	 */
	protected static function QueryBuilder( $table ): QueryBuilder
	{
		return self::_connection( $table )->createQueryBuilder( $table );
	}

	/**
	 * Truncate()
	 * 
	 * @param string $table
	 * @return QueryBuilder
	 * 
	 * @version	0.10.1
	 * @since	0.10.1
	 */
	public static function Truncate( $table )
	{
		self::_connection( $table )->truncate( $table );
	}

	/**
	 * _connection
	 * 
	 * @param string $table
	 * @return Connection
	 * 
	 * @version	0.10.1
	 * @since	0.10.1
	 */
	private static function _connection( $table ): Connection
	{
		return GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( $table );
	}

	/**
	 * _convertTableUidsSysCategoryRowsToArray: Convert rows to an array
	 *
	 * @param array	$rows current rows from sys_category_record_mm
	 * @return array	$rows $arr['uid_local'][] => 'uid_foreign'
	 * @access public
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	private static function _convertTableUidsSysCategoryRowsToArray( $rows )
	{
		$arrUidForeignUidLocal = [];

		foreach( array_keys( $rows ) as $uidLocal )
		{
			foreach( $rows[ $uidLocal ] as $record )
			{
				$arrUidForeignUidLocal[ $uidLocal ][] = $record[ 'uid_foreign' ];
			}
		}

		return $arrUidForeignUidLocal;
	}

	/**
	 * _getPageChildren():
	 *
	 * @param mixed		$pidList	pages uids as csv-string or pages as array
	 * @return array	row from database
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	private static function _getPageChildren( $pidList )
	{
		$pidList = self::_getPidlistAsCsv( $pidList );

		$arrUids = [];
		$pidListChildren = '';

		$queryBuilder = self::QueryBuilder( 'pages' );
		$queryBuilder
						->select( 'uid' )
						->from( 'pages' )
						->where(
										$queryBuilder->expr()->in( 'pid', $pidList )
						)
		;
		$rows = $queryBuilder
						->execute()
						->fetchAll()
		;

		if( empty( $rows ) )
		{
			return;
		}

		foreach( $rows as $row )
		{
			$arrUids[] = $row[ 'uid' ];
		}

		$pidListChildren = implode( ',', $arrUids );
//		var_dump( __METHOD__, __LINE__, $pidList, $pidListChildren );
		return $pidListChildren;
	}

	/**
	 * _getPidList
	 * 
	 * @param string		List of page ids
	 * @param integer		Recursive levels
	 * @return string
	 * 
	 * @version	0.30.5
	 * @since	0.10.1
	 */
	private static function _getPidList( $pages, $recursive )
	{
		$children = $pages;

		for( $level = 0; $level < $recursive; $level++ )
		{
			$children = self::_getPageChildren( $children );
			$pages = self::_getPidlistAsCsv( $pages );
			if( empty( $children ) )
			{
				return $pages;
			}
			$pages = $pages . ',' . $children;
			if( $level > 100 )
			{
				$prompt = 'level > 100 @ ' . __METHOD__ . '#' . __LINE__;
				die( $prompt );
			}
		}
		$pages = self::_getPidlistAsCsv( $pages );

		return $pages;
	}

	/**
	 * _getPidlistAsCsv():
	 *
	 * @param mixed		$pidList	pages uids as csv-string or pages as array
	 * @return array	row from database
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	private static function _getPidlistAsCsv( $pidList )
	{
		$pidListAsCsv = '';
		$uids = [];

		if( !is_array( $pidList ) )
		{
			return $pidList;
		}

		foreach( $pidList as $page )
		{
			$uids[] = $page[ 'uid' ];
		}

		$pidListAsCsv = implode( ',', $uids );
		return $pidListAsCsv;
	}

	/**
	 * _getTableUidsSysCategory:
	 *
	 * @paam string	$table local table
	 * @param integer	$uid uid in sys_category
	 * @param integer	uid
	 * @access public
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	private static function _getTableUidsSysCategory( $table, $field, $uid )
	{
		if( empty( $uid ) )
		{
			$prompt = 'FATAL ERROR: uid is empty';
			var_dump( __METHOD__, __LINE__, $prompt );
			die();
		}

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'sys_category_record_mm' );
		$queryBuilder
						->getRestrictions()
		;

		$rows = $queryBuilder
						->select( 'uid_foreign' )
						->from( 'sys_category_record_mm' )
						->where(
										$queryBuilder->expr()->eq( 'uid_local', $queryBuilder->createNamedParameter( $uid, \PDO::PARAM_INT ) )
						)
						->andWhere(
										$queryBuilder->expr()->eq( 'tablenames', $queryBuilder->createNamedParameter( $table ) )
										, $queryBuilder->expr()->eq( 'fieldname', $queryBuilder->createNamedParameter( $field ) )
						)
						->execute()
						->fetchAll();

//		var_dump( __METHOD__, __LINE__, $queryBuilder->getSQL(), $queryBuilder->getParameters(), $rows );
//		die();

		return $rows;
	}

	/**
	 * _getTtcontentByListType():
	 *
	 * @param string $listType 
	 * 
	 * @return array	row from database
	 * @access private
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	private static function _getTtcontentByListType( $listType )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tt_content' );

		$queryBuilder
						->getRestrictions()
						->removeAll()
						->add( GeneralUtility::makeInstance( DeletedRestriction::class ) )
		;

		$queryBuilder
						->select( '*' )
						->from( 'tt_content' )
						->where( $queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( $listType ) ) );

		$rows = $queryBuilder
						->execute()
						->fetchAll();

		return $rows;
	}

	/**
	 * getTxXblogForm:
	 *
	 * @return array	rows from database
	 * @access private
	 * @version 0.10.1
	 * @since 0.9.4
	 * @internal #t4450
	 */
	private static function _getTxXblogFormPlugins( $pid )
	{
		if( empty( $pid ) )
		{
			return;
		}

		$queryBuilder = self::QueryBuilder( 'tt_content' );

		$queryBuilder
						->getRestrictions()
						->removeAll()
						->add( GeneralUtility::makeInstance( DeletedRestriction::class ) )
		;

		$queryBuilder
						->select( '*' )
						->from( 'tt_content' )
						->where(
										$queryBuilder->expr()->eq( 'pid', $queryBuilder->createNamedParameter( $pid, \PDO::PARAM_INT ) )
										, $queryBuilder->expr()->eq( 'list_type', '"xblog_pi13"' )
		);

		$rows = $queryBuilder
						->execute()
						->fetchAll();

		return $rows;
	}

	/**
	 * _gradeUpTitle():
	 *
	 * @param array		rows
	 * @return array	rows
	 * @access private
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	private static function _gradeUpTitle( $rows, $table, $label, $labelAlt )
	{
		//var_dump( __METHOD__, __LINE__, $rows );

		$labelAltForce = TcaUtility::getCtrlLabelAltForce( $table );

		$rows = self::_gradeUpTitleTimestamp( $rows, $table, $label );
		$rows = self::_gradeUpTitleAltTimestamp( $rows, $table, $labelAlt );

		if( !$labelAltForce )
		{
			foreach( $rows as $index => $row )
			{
				$rows[ $index ][ 'title' ] = $row[ $label ];
			}
			return $rows;
		}

		$csvLabelAlt = TcaUtility::getCtrlLabelAlt( $table );
		$arrLabelAlt = explode( ',', $csvLabelAlt );
		foreach( $rows as $index => $row )
		{
			$arrAppendix = [];
			foreach( $arrLabelAlt as $labelAlt )
			{
				$labelAlt = trim( $labelAlt );
				//var_dump( __METHOD__, __LINE__, $labelAlt, $row[ $labelAlt ] );
				$arrAppendix[] = $row[ $labelAlt ];
			}
			$appendix = implode( ', ', $arrAppendix );

			$rows[ $index ][ 'title' ] = $row[ $label ] . ': ' . $appendix;
		}


//		var_dump( __METHOD__, __LINE__, $lable, $csvLabelAlt, $arrLabelAlt, $rows );
//		die();

		return $rows;
	}

	/**
	 * _gradeUpTitleTimestamp():
	 *
	 * @param array		rows
	 * @param string	table
	 * @param	string	label
	 * @return array	rows
	 * @access private
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	private static function _gradeUpTitleTimestamp( $rows, $table, $label )
	{
		$titleIsTimestamp = TcaUtility::isTimestamp( $table, $label );

		if( !$titleIsTimestamp )
		{
			return $rows;
		}

		foreach( $rows as $index => $row )
		{
			$dateFormat = TcaUtility::getColumnDateFormat( $table, $label );
			$rows[ $index ][ $label ] = date( $dateFormat, $row[ $label ] );
		}
		return $rows;
	}

	/**
	 * _gradeUpTitleAltTimestamp():
	 *
	 * @param array		rows
	 * @param string	table
	 * @param	string	labelAlt
	 * @return array	rows
	 * @access private
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	private static function _gradeUpTitleAltTimestamp( $rows, $table, $labelAlt )
	{
		$csvLabelAlt = TcaUtility::getCtrlLabelAlt( $table );
		$arrLabelAlt = explode( ',', $csvLabelAlt );
		foreach( $arrLabelAlt as $labelAlt )
		{
			$labelAlt = trim( $labelAlt );
			$labelAltIsTimestamp = TcaUtility::isTimestamp( $table, $labelAlt );
			if( !$labelAltIsTimestamp )
			{
				continue;
			}
			foreach( $rows as $index => $row )
			{
				$dateFormat = TcaUtility::getColumnDateFormat( $table, $labelAlt );
				$rows[ $index ][ $labelAlt ] = date( $dateFormat, $row[ $labelAlt ] );
			}
		}

		return $rows;
	}

	/**
	 * _intersect: Get uids from local table only, if local table matches all current sys_categories
	 *
	 * @param array	$uids uids in sys_category
	 * @return array $arrUidsLocal uids of the matched local table
	 * @access private
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	private static function _intersect( $arrUidForeignUidLocal )
	{
		$amountOfForeignUids = count( $arrUidForeignUidLocal );

		switch( $amountOfForeignUids )
		{
			case($amountOfForeignUids == 1):
				$arrUidsLocal = $arrUidForeignUidLocal[ key( $arrUidForeignUidLocal ) ];
				break;
			case($amountOfForeignUids > 1):
				$arrUidsLocal = call_user_func_array( 'array_intersect', $arrUidForeignUidLocal );
				break;
			case($amountOfForeignUids == 0):
			default:
				$prompt = 'FATAL ERROR: operator is not expected: "' . $amountOfForeignUids . '"';
				var_dump( __METHOD__, __LINE__, $prompt );
				die();
		}

		// if there isn't any result, set uid to 0, so that any record will displayed
		if( empty( $arrUidsLocal ) )
		{
			$arrUidsLocal[] = 0;
		}

//		var_dump( __METHOD__, __LINE__, $arrUidForeignUidLocal, $arrUidsLocal );
//		die();

		return $arrUidsLocal;
	}

	/**
	 * _merge: Get uids from local table only, if local table matches all current sys_categories
	 *
	 * @param array	$uids uids in sys_category
	 * @return array $arrUidsLocal uids of the matched local table
	 * @access private
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	private static function _merge( $arrUidForeignUidLocal )
	{
		$amountOfForeignUids = count( $arrUidForeignUidLocal );

		switch( $amountOfForeignUids )
		{
			case($amountOfForeignUids == 1):
				$arrUidsLocal = $arrUidForeignUidLocal[ key( $arrUidForeignUidLocal ) ];
				break;
			case($amountOfForeignUids > 1):
				$arrUidsLocal = call_user_func_array( 'array_merge', $arrUidForeignUidLocal );
				break;
			case($amountOfForeignUids == 0):
			default:
				$prompt = 'FATAL ERROR: operator is not expected: "' . $amountOfForeignUids . '"';
				var_dump( __METHOD__, __LINE__, $prompt );
				die();
		}

		// if there isn't any result, set uid to 0, so that any record will displayed
		if( empty( $arrUidsLocal ) )
		{
			$arrUidsLocal[] = 0;
		}

//		var_dump( __METHOD__, __LINE__, $arrUidForeignUidLocal, $arrUidsLocal );
//		die();

		return $arrUidsLocal;
	}

	/**
	 * _orderBy():
	 *
	 * @param object	$queryBuilder
	 * @param array		$uglyOrderBy
	 * @return object $queryBuilder
	 * @access private
	 * @version 0.10.1
	 * @since 0.8.16
	 */
	private static function _orderBy( $queryBuilder, $uglyOrderBy )
	{
		foreach( QueryHelper::parseOrderBy( ( string ) $uglyOrderBy ) as $orderPair )
		{
			list($fieldName, $order) = $orderPair;
			$queryBuilder->addOrderBy( $fieldName, $order );
		}
		return $queryBuilder;
	}

	/**
	 * _whereExpressionsBuilder( ) :
	 *
	 * @param object $queryBuilder
	 * @param array $where  AND || OR
	 * @return array
	 * @access private
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	private static function _whereExpressionsBuilder( $queryBuilder, $where )
	{
		if( empty( $where ) )
		{
			return;
		}
		$expressions = null;
		foreach( $where[ 'AND' ] as $expression )
		{
			$expressions[] = self::_whereExpressionsBuilderExpression( $queryBuilder, $expression );
		}
		unset( $where[ 'AND' ] );

		$keys = array_keys( $where );
		if( $keys == null )
		{
			return $expressions;
		}

		$prompt = 'ERROR: where-array contains unexpected elements: ' . $keys;
		var_dump( __METHOD__, __LINE__, $prompt );
		die();
	}

	/**
	 * _whereExpressionsBuilderExpression( ) :
	 *
	 * @param object $queryBuilder
	 * @param array $expression  eq || like || ...
	 * @return array
	 * @access private
	 * @version 0.30.19
	 * @since 0.10.1
	 */
	private static function _whereExpressionsBuilderExpression( $queryBuilder, $expression )
	{
		$operator = key( $expression );

		$parts = $expression[ $operator ];
		$field = key( $parts );
		$value = $parts[ $field ];

		switch( $operator )
		{
			case('eq'):
				return $queryBuilder->expr()
												->eq(
																$field
																, $queryBuilder->createNamedParameter( $value )
												)
				;
			case('gt'):
				return $queryBuilder->expr()
												->gt(
																$field
																, $queryBuilder->createNamedParameter( $value )
												)
				;
			case('inset'):
				return $queryBuilder->expr()
												->inset(
																$field
																, $queryBuilder->createNamedParameter( $value )
												)
				;
			case('like'):
				return $queryBuilder->expr()
												->like(
																$field
																, $queryBuilder->createNamedParameter( '%' . $queryBuilder->escapeLikeWildcards( $value ) . '%' )
												)
				;
			default:
				$prompt = 'ERROR: operator is not expected: ' . $operator;
				var_dump( __METHOD__, __LINE__, $prompt );
				die();
		}
	}

	/**
	 * getXblogSysCategories:
	 *
	 * @param integer	uid
	 * @return array	rows from database
	 * @access public
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	public static function getXblogSysCategories( $uid )
	{
		if( empty( $uid ) )
		{
			return;
		}

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'sys_category_record_mm' );
		$queryBuilder
						->getRestrictions()
		;

		$rows = $queryBuilder
						->select( '*' )
						->from( 'sys_category_record_mm' )
						->where(
										$queryBuilder->expr()->eq( 'uid_foreign', $queryBuilder->createNamedParameter( $uid, \PDO::PARAM_INT ) )
						)
						->andWhere(
										$queryBuilder->expr()->eq( 'tablenames', $queryBuilder->createNamedParameter( 'tt_content' ) )
										, $queryBuilder->expr()->eq( 'fieldname', $queryBuilder->createNamedParameter( 'xBlogMain' ) )
						)
						->execute()
						->fetchAll();

		return $rows;
	}

	/**
	 * getPagesWiPluginCaddyPi1():
	 *
	 * @return array	row from database
	 * @access public
	 * @version 0.31.0
	 * @since 0.30.0
	 */
	public static function getPagesWiPluginCaddyPi1()
	{
		return self::_getTtcontentByListType( 'caddy_pi1' );
	}

	/**
	 * getPagesWiPluginCaddyPi1():
	 *
	 * @return array	row from database
	 * @access public
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	public static function getPagesWiPluginPublish()
	{
		return self::_getTtcontentByListType( 'publish_pi1' );
	}

	/**
	 * getPagesFromList():
	 *
	 * @param string	$pidlist
	 * @return array	rows with page records
	 * @access public
	 * @version 0.24.0
	 * @since 0.24.0
	 */
	public static function getPagesFromList( $pidlist )
	{
		if( empty( $pidlist ) )
		{
			return;
		}

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'pages' );
		$queryBuilder
						->getRestrictions()
						->removeAll()
						->add( GeneralUtility::makeInstance( DeletedRestriction::class ) )
		;

		$rows = $queryBuilder
						->select( '*' )
						->from( 'pages' )
						->where(
										$queryBuilder->expr()->in( 'uid', $pidlist )
						)
						->execute()
						->fetchAll()
		;

		return $rows;
	}

	/**
	 * getTableUidsSysCategoryAnd:
	 *
	 * @paam string	$table local table
	 * @param array	$uids uids in sys_category
	 * @return array $arrUidsForeignTable
	 * @access public
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	public static function getTableUidsSysCategoryAnd( $table, $field, $uids )
	{
		$arrUidForeignUidLocal = [];
		$rows = [];

		if( empty( $uids ) )
		{
			$prompt = 'FATAL ERROR: uids is empty';
			var_dump( __METHOD__, __LINE__, $prompt );
			die();
		}

		foreach( $uids as $uid )
		{
			$rows[ $uid ] = self::_getTableUidsSysCategory( $table, $field, $uid );
		}

		// Convert rows to an array
		$arrUidForeignUidLocal = self::_convertTableUidsSysCategoryRowsToArray( $rows );
		// Get uids from local table only, if local table matches all current sys_categories
		$arrUidsForeignTable = self::_intersect( $arrUidForeignUidLocal );

//		var_dump( __METHOD__, __LINE__, $arrUidForeignUidLocal, $arrUidsForeignTable );
//		die();

		return $arrUidsForeignTable;
	}

	/**
	 * getTableUidsSysCategoryOr:
	 *
	 * @paam string	$table local table
	 * @param array	$uids uids in sys_category
	 * @return array $arrUidsForeignTable
	 * @access public
	 * @version 0.20.33
	 * @since 0.20.33
	 * @internal #v0107
	 */
	public static function getTableUidsSysCategoryOr( $table, $field, $uids )
	{
		$arrUidForeignUidLocal = [];
		$rows = [];

		if( empty( $uids ) )
		{
			$prompt = 'FATAL ERROR: uids is empty';
			var_dump( __METHOD__, __LINE__, $prompt );
			die();
		}

		foreach( $uids as $uid )
		{
			$rows[ $uid ] = self::_getTableUidsSysCategory( $table, $field, $uid );
		}

		// Convert rows to an array
		$arrUidForeignUidLocal = self::_convertTableUidsSysCategoryRowsToArray( $rows );
		// Get uids from local table only, if local table matches all current sys_categories
		$arrUidsForeignTable = self::_merge( $arrUidForeignUidLocal );

//		var_dump( __METHOD__, __LINE__, $arrUidsForeignTable, $arrUidsForeignTable );
//		die();

		return $arrUidsForeignTable;
	}

	/**
	 * getRecordsByTable():
	 *
	 * @param string	table
	 * @param string	pages
	 * @param integer	recursive
	 * @param integer	max records
	 * @return array	rows from database
	 * @access public
	 * @version 0.10.1
	 * @since 0.8.16
	 */
	public static function getRecordsByTable( $table, $pages = '', $recursive = 0, $max = 0 )
	{
		if( empty( $table ) )
		{
			return;
		}

		$orderBy = TcaUtility::getCtrlSortbyOrDefaultsortby( $table );

		$queryBuilder = self::QueryBuilder( $table );

		//$queryBuilder->select( 'tt_content.bodytext AS t1.text' );
		$queryBuilder
						->select( '*' )
						->from( $table )
		;

		if( $pages )
		{
			$pidList = self::_getPidList( $pages, $recursive );
			$queryBuilder->where( $queryBuilder->expr()->in( 'pid', $pidList ) );
		}

		if( $max )
		{
			$queryBuilder->setMaxResults( $max );
		}

		$queryBuilder = self::_orderBy( $queryBuilder, $orderBy );


		$rows = $queryBuilder
						->execute()
						->fetchAll();

		return $rows;
	}

	/**
	 * getRecordsFromTableMM():
	 *
	 * @param string	$mmTable
	 * @param integer	$uid
	 * @param string	$table_foreign
	 * @param string	$table_local
	 * @return array	rows from database
	 * @access public
	 * @version 0.30.17
	 * @since		0.8.16
	 */
	public static function getRecordsFromTableMM( $mmTable, $uid, $table_foreign, $table_local )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( $mmTable );
		$queryBuilder
						->select( '*' )
						->from( $mmTable )
						->where(
										$queryBuilder->expr()->eq( 'uid_local', $queryBuilder->createNamedParameter( $uid ) )
						)
		;

		if( $table_foreign )
		{
			$queryBuilder->andWhere(
							$queryBuilder->expr()->eq( 'table_foreign', $queryBuilder->createNamedParameter( $table_foreign ) )
			);
		}
		if( $table_local )
		{
			$queryBuilder->andWhere(
							$queryBuilder->expr()->eq( 'table_local', $queryBuilder->createNamedParameter( $table_local ) )
			);
		}

//		debug( $queryBuilder->getSQL() );
//		debug( $queryBuilder->getParameters() );

		$rows = $queryBuilder->execute()->fetchAll();
		return $rows;
	}

	/**
	 * getTableUidsTitle():
	 *
	 * @param string	table
	 * @param string	pages
	 * @param integer	recursive
	 * @return array	rows from database
	 * @access public
	 * @version 0.10.1
	 * @since 0.8.16
	 */
	public static function getTableUidsTitle( $table, $pages = '', $recursive = 0 )
	{
		if( empty( $table ) )
		{
			return;
		}

		$label = TcaUtility::getCtrlLabel( $table );
		$labelAltForce = TcaUtility::getCtrlLabelAltForce( $table );

		$orderBy = TcaUtility::getCtrlSortbyOrDefaultsortby( $table );

		$queryBuilder = self::QueryBuilder( $table );

		//$queryBuilder->select( 'tt_content.bodytext AS t1.text' );
		$queryBuilder
						->select( 'uid' )
						->addSelect( $label )
		;
		if( $labelAltForce )
		{
			$csvLabelAlt = TcaUtility::getCtrlLabelAlt( $table );
			$arrLabelAlt = explode( ',', $csvLabelAlt );
			foreach( $arrLabelAlt as $labelAlt )
			{
				$labelAlt = trim( $labelAlt );
				$queryBuilder->addSelect( $labelAlt );
			}
		}
		$queryBuilder->from( $table );

		if( $pages )
		{
			$pidList = self::_getPidList( $pages, $recursive );
			$queryBuilder->where( $queryBuilder->expr()->in( 'pid', $pidList ) );
		}

		$queryBuilder = self::_orderBy( $queryBuilder, $orderBy );


		$rows = $queryBuilder
						->execute()
						->fetchAll();

		$rows = self::_gradeUpTitle( $rows, $table, $label, $labelAlt );

		return $rows;
	}

	/**
	 * getTreeList():
	 *
	 * @param string $pages pids for the start in the treelist (csv)
	 * @param int $depth depth in the treelist
	 * 
	 * @return array	list of pids
	 * 
	 * @access public
	 * @version 0.9.8
	 * @since 0.8.10
	 * 
	 * @internal #i0092
	 */
	public static function getTreeList( $pages = NULL, $depth = 1, $begin = 0, $permClause = '' )
	{
		$aPidList = [];

		if( empty( $pages ) )
		{
			return;
		}

		$queryGenerator = GeneralUtility::makeInstance( QueryGenerator::class );

		$pids = GeneralUtility::intExplode( ',', $pages );
		foreach( ( array ) $pids as $pid )
		{
			$sPidlist = $queryGenerator->getTreeList( $pid, $depth, $begin, $permClause );
			$aPidlist = array_merge( $aPidList, GeneralUtility::intExplode( ',', $sPidlist ) );
		}

		return $aPidlist;
	}

	/**
	 * getTtcontentRowByUid():
	 *
	 * @param integer	uid
	 * @return array	row from database
	 * @access public
	 * @version 0.8.16
	 * @since 0.8.16
	 */
	public static function getTtcontentRowByUid( $uid )
	{
		if( $uid === NULL )
		{
			return;
		}

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tt_content' );
		$queryBuilder
						->getRestrictions()
						->removeAll()
						->add( GeneralUtility::makeInstance( DeletedRestriction::class ) )
		;

		$row = $queryBuilder
						->select( '*' )
						->from( 'tt_content' )
						->where(
										$queryBuilder->expr()->eq( 'uid', $uid )
						)
						->execute()
						->fetch()
		;

		return $row;
	}

	/**
	 * getTtcontentXBlogs():
	 *
	 * @return array	row from database
	 * @access public
	 * @version 0.30.0
	 * @since 0.8.10
	 */
	public static function getTtcontentXBlogs()
	{
		return self::_getTtcontentByListType( 'xblog_pi1' );
	}

	/**
	 * getTxXblogFilter:
	 *
	 * @param integer	uid
	 * @return array	rows from database
	 * @access public
	 * @version 0.8.21
	 * @since 0.8.16
	 */
	public static function getTxXblogFilter( $uid )
	{
		if( empty( $uid ) )
		{
			return;
		}

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tx_xblog_domain_model_filter' );
		$queryBuilder
						->getRestrictions()
//						->removeAll()
//						->add( GeneralUtility::makeInstance( DeletedRestriction::class ) )
		;

		$rows = $queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_filter' )
						->where(
										$queryBuilder->expr()->eq( 'tt_content', $uid )
						)
						// #t4183, 191005, dwildt, 1+
						->orderBy( 'sorting' )
						->execute()
						->fetchAll();

		return $rows;
	}

	/**
	 * getTxXblogFormRowsByXblogMain:
	 *
	 * @param integer	uid of the xBlogMain plugin
	 * @param integer	pid current page id
	 * @return integer uid of the xBlogForm plugin
	 * @access public
	 * @version 0.9.4
	 * @since 0.9.4
	 * @internal #t4450
	 */
	public static function getTxXblogFormUidByXblogMain( $uid, $pid )
	{
//		var_dump( __METHOD__, __LINE__, $pid );

		$rows = self::_getTxXblogFormPlugins( $pid );

		if( empty( $rows ) )
		{
			return;
		}

		$xBlogFormUid = null;
		foreach( $rows as $row )
		{
//			var_dump( __METHOD__, __LINE__, $row['pi_flexform'] );
			$xmlFlexform = $row[ 'pi_flexform' ];
			$sheet = 'ctrl';
			$field = 'settings.flexform.pi13.ctrl.plugin.list';
			$xBlogSrceTable = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );
			if( $xBlogSrceTable == $uid )
			{
				$xBlogFormUid = $row[ 'uid' ];
				continue;
			}
			//var_dump( __METHOD__, __LINE__, $xBlogSrceTable, $uid );
		}

		return $xBlogFormUid;
	}

	/**
	 * getRecord:
	 *
	 * @param string table
	 * @param integer uid
	 * @return array	rows from database
	 * @access public
	 * @version 0.9.4
	 * @since 0.9.4
	 */
	public static function getRecord( $table, $uid )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( $table );
		$queryBuilder->getRestrictions();

		$queryBuilder
						->select( '*' )
						->from( $table )
						->where(
										$queryBuilder->expr()->eq( 'uid', $uid )
		);

		$result = new \stdClass();
		$result = $queryBuilder->execute();
		$row = $result->fetch();

		return $row;
	}

	/**
	 * getTxXblogFormElements:
	 *
	 * @param array uids
	 * @return array	rows from database
	 * @access public
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	public static function getTxXblogFormElements( $uids )
	{
		$orderBy = TcaUtility::getCtrlSortbyOrDefaultsortby( 'tx_xblog_domain_model_element' );

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tx_xblog_domain_model_element' );
		$queryBuilder
						->getRestrictions()
		;

		$queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_element' )
						->where(
										$queryBuilder->expr()->in( 'tx_xblog_domain_model_form', $uids )
		);

		$queryBuilder = self::_orderBy( $queryBuilder, $orderBy );

//		var_dump( __METHOD__, __LINE__, $queryBuilder->getParameters(), $queryBuilder->getSQL() );
//		die();

		$rows = $queryBuilder
						->execute()
						->fetchAll();

		if( empty( $rows ) )
		{
			return;
		}

		return $rows;
	}

	/**
	 * getTxXblogFormRows:
	 *
	 * @param integer	uid of the XblogForm
	 * @return array	rows from database
	 * @access public
	 * @version 0.9.3
	 * @since 0.9.3
	 */
	public static function getTxXblogFormRows( $uid )
	{
		$formUids = null;

		if( empty( $uid ) )
		{
			return;
		}

		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tx_xblog_domain_model_form' );
		$queryBuilder
						->getRestrictions()
		;

		$rows = $queryBuilder
						->select( '*' )
						->from( 'tx_xblog_domain_model_form' )
						->where(
										$queryBuilder->expr()->eq( 'tt_content', $uid )
						)
						// #t4183, 191005, dwildt, 1+
						->orderBy( 'sorting' )
						->execute()
						->fetchAll();

		if( empty( $rows ) )
		{
			return;
		}

		foreach( $rows as $row )
		{
			$formUids[] = $row[ 'uid' ];
		}

		$fields = self::getTxXblogFormElements( $formUids );


		return $fields;
	}

	/**
	 * getXBlogSrceTable():
	 *
	 * @param integer	tt_content uid
	 * @param string	sheet
	 * @param string	field
	 * @return string table from xBlog source
	 * @access public
	 * @version 0.8.26
	 * @since 0.8.16
	 */
	public static function getXBlogSrceTable( $xBlogSrceUid, $sheet = 'table', $field = 'settings.flexform.pi.table.table' )
	{
		$row = self::getTtcontentRowByUid( $xBlogSrceUid );
		if( empty( $row ) )
		{
			$header = 'Oops!';
			$prompt = 'There isn\'t any xBlogMain plugin with the uid "' . $xBlogSrceUid . '"';
			FlashmessageUtility::render( $header, $prompt, 'ERROR', __METHOD__, __LINE__ );
			return;
		}
		$xmlFlexform = $row[ 'pi_flexform' ];
		$xBlogSrceTable = FlexformValueUtility::getFlexformValue( $xmlFlexform, $sheet, $field, __METHOD__, __LINE__ );

		if( !empty( $xBlogSrceTable ) )
		{
			return $xBlogSrceTable;
		}

		$header = 'Oops!';
		$prompt = 'Flexform value could not detected! Sheet: ' . $sheet . ', field: ' . $field;
		FlashmessageUtility::render( $header, $prompt, 'ERROR', __METHOD__, __LINE__ );
	}

	/**
	 * getXblogs:
	 * 
	 * @param	int			$pid
	 * @param	int			$uid					:	Uid of the current plugin
	 * @param	string	$table
	 * @param	boolean	$onlyUncached
	 *
	 * @return array	rows from database
	 * @access public
	 * @version 0.30.7
	 * @since 0.20.55
	 */
	public static function getXblogs( $pid, $uid, $table = null, $onlyUncached = false )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tt_content' );
		$queryBuilder
						->getRestrictions()
		;

		$queryBuilder
						->select( '*' )
						->from( 'tt_content' )
		;

		$queryBuilder->where(
						$queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( 'xblog_pi1' ) )
		);

//		switch( $onlyUncached )
//		{
//			case(true):
//				$queryBuilder->where(
//								$queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( 'xblog_pi15' ) )
//				);
//				break;
//			case(false):
//			default:
//				$queryBuilder->orWhere(
//								$queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( 'xblog_pi1' ) )
//								, $queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( 'xblog_pi15' ) )
//				);
//				break;
//		}

		$queryBuilder->andWhere(
						$queryBuilder->expr()->neq( 'uid', $queryBuilder->createNamedParameter( $uid, \PDO::PARAM_INT ) )
		);

		if( $pid )
		{
			$queryBuilder->andWhere(
							$queryBuilder->expr()->eq( 'pid', $queryBuilder->createNamedParameter( $pid, \PDO::PARAM_INT ) )
			);
		}

		if( $table )
		{
			$queryBuilder->andWhere(
							$queryBuilder->expr()->comparison(
											$queryBuilder->quoteIdentifier( 'pi_flexform' )
											, 'REGEXP'
											, '\'<field index="settings\\.flexform\\.pi\\.table\\.table">[[:space:]]+<value index="vDEF">' . $table . '</value>[[:space:]]+</field>\''
							)
			);
		}

		$rows = $queryBuilder
						->execute()
						->fetchAll();

//		var_dump( __METHOD__, __LINE__, $queryBuilder->getSQL(), $queryBuilder->getParameters() );
//		die();
		return $rows;
	}

	/**
	 * getXblogs:
	 * 
	 * @param	int	$pid
	 * @param	int	$uid	:	Uid of the current plugin
	 *
	 * @return array	rows from database
	 * @access public
	 * @version 0.30.10
	 * @since 0.30.10
	 */
	public static function getXblogsForm( $pid, $uid )
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getQueryBuilderForTable( 'tt_content' );
		$queryBuilder
						->getRestrictions()
		;

		$queryBuilder
						->select( '*' )
						->from( 'tt_content' )
		;

		$queryBuilder->where(
						$queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( 'xblog_pi13' ) )
						, $queryBuilder->expr()->neq( 'uid', $queryBuilder->createNamedParameter( $uid, \PDO::PARAM_INT ) )
		);

		if( $pid )
		{
			$queryBuilder->andWhere(
							$queryBuilder->expr()->eq( 'pid', $queryBuilder->createNamedParameter( $pid, \PDO::PARAM_INT ) )
			);
		}

		$rows = $queryBuilder
						->execute()
						->fetchAll();

		return $rows;
	}

}
