<?php

namespace Netzmacher\Xblog\Utility;

use Netzmacher\Xblog\Utility\LanguageUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 *
 * @author  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package TYPO3
 * @subpackage  xBlog
 * @version     0.30.7
 * @since       0.8.16
 */
class TcaUtility
{

	/**
	 * @var string
	 */
	private static $_LLLpath = 'LLL:EXT:xblog/Resources/Private/Language/locallang_db.xlf:';

	/**
	 * _dateFormat():
	 *
	 * @param string	table
	 * @param string	field
	 * @return string	format	format for php date method
	 * @access private
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public static function _dateFormat( $table, $field )
	{
		static $format = [];

		if( isset( $format[ $table ][ $field ] ) )
		{
			return $format[ $table ][ $field ];
		}


		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		switch( true )
		{
			case(strpos( $eval, 'datetime' ) !== false):
				$format[ $table ][ $field ] = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.datetime' );
				break;
			case(strpos( $eval, 'date' ) !== false):
				$format[ $table ][ $field ] = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.date' );
				break;
			case(strpos( $eval, 'timesec' ) !== false):
				$format[ $table ][ $field ] = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.timesec' );
				break;
			case(strpos( $eval, 'time' ) !== false):
				$format[ $table ][ $field ] = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.time' );
				break;
			default:
				break;
		}
		return $format[ $table ][ $field ];
	}

	/**
	 * _sqlConcat():
	 *
	 * @param array	label
	 * @return string
	 * @access public
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	private static function _sqlConcat( $arrLabel )
	{
		if( count( $arrLabel ) > 1 )
		{
			return 'CONCAT(' . implode( ', ", ", ', $arrLabel ) . ')';
		}

		return $arrLabel[ 0 ];
	}

	/**
	 * _sqlQuoteField():
	 *
	 * @param string	field
	 * @return string	path to the icon file of the given table
	 * @access public
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	private static function _sqlQuoteField( $field )
	{
		return '`' . $field . '`';
	}

	/**
	 * _sqlUnixtimeToDateformat():
	 *
	 * @param string	table
	 * @param string	label
	 * @return string	
	 * @access public
	 * @version 0.9.3
	 * @since 0.9.2
	 */
	private static function _sqlUnixtimeToDateformat( $table, $label )
	{
		$arrDestLabel = null;

		// SELECT `uid`, CONCAT(DATE(FROM_UNIXTIME(`datetime`)), ", ", `title`) AS `title` FROM `tx_org_news`
		// SELECT `uid`, CONCAT(TIME(FROM_UNIXTIME(`datetime`)), ", ", `title`) AS `title` FROM `tx_org_news`
		$arrSrceLabel = ( array ) explode( ',', $label );
		foreach( $arrSrceLabel as $field )
		{
			// #t4446, 200117, 1-/+
			//if( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM' ] )
			if( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'foreign_table' ] )
			{
				continue;
			}
			$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
			switch( true )
			{
				case(strpos( $eval, 'datetime' ) !== false):
					$field = 'DATE(FROM_UNIXTIME(' . self::_sqlQuoteField( $field ) . ')), " ", TIME(FROM_UNIXTIME(' . self::_sqlQuoteField( $field ) . '))';
					break;
				case(strpos( $eval, 'date' ) !== false):
					$field = 'DATE(FROM_UNIXTIME(' . self::_sqlQuoteField( $field ) . '))';
					break;
				case(strpos( $eval, 'timesec' ) !== false):
				case(strpos( $eval, 'time' ) !== false):
					$field = 'TIME(FROM_UNIXTIME(' . self::_sqlQuoteField( $field ) . '))';
					break;
				default:
					$field = '' . self::_sqlQuoteField( $field ) . '';
					break;
			}
			$arrDestLabel[] = $field;
		}
		if( is_array( $arrDestLabel ) )
		{
			return $arrDestLabel;
		}
//		var_dump( __METHOD__, __LINE__, $arrSrceLabel, $arrDestLabel, $label );
//		die();
		$arrDestLabel[] = $label;
		return $arrDestLabel;
	}

	/**
	 * evalDate():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public static function evalDate( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		if( strpos( $eval, 'date' ) !== false )
		{
			return true;
		}
		return false;
	}

	/**
	 * evalDouble2():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public static function evalDouble2( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		if( strpos( $eval, 'double2' ) !== false )
		{
			return true;
		}
		return false;
	}

	/**
	 * evalPassword():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public static function evalPassword( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		if( strpos( $eval, 'password' ) !== false )
		{
			return true;
		}
		return false;
	}

	/**
	 * evalSaltedPassword():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public static function evalSaltedPassword( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		if( strpos( $eval, 'saltedPassword' ) !== false )
		{
			return true;
		}
		return false;
	}

	/**
	 * evalTime():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public static function evalTime( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		if( strpos( $eval, 'time' ) !== false )
		{
			return true;
		}
		return false;
	}


	/**
	 * evalYear():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public static function evalYear( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		if( strpos( $eval, 'year' ) !== false )
		{
			return true;
		}
		return false;
	}
	/**
	 * getColumnConfigEval():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	eval
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public static function getColumnConfigEval( $table, $column )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'eval' ] ) )
		{
			return;
		}

		return $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'eval' ];
	}

	/**
	 * getColumnConfigForeignfield():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	foreigntable
	 * @access public
	 * @version 0.9.14
	 * @since 0.9.5
	 */
	public static function getColumnConfigForeignfield( $table, $column )
	{
		static $foreignfield = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $foreignfield[ $tableField ] ) )
		{
			return $foreignfield[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'foreign_field' ] ) )
		{
			$foreignfield[ $tableField ] = '';
		}
		else
		{
			$foreignfield[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'foreign_field' ];
		}

		return $foreignfield[ $tableField ];
	}

	/**
	 * getColumnConfigForeigntable():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	foreigntable
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.5
	 */
	public static function getColumnConfigForeigntable( $table, $column )
	{
		static $foreigntable = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $foreigntable[ $tableField ] ) )
		{
			return $foreigntable[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'foreign_table' ] ) )
		{
			$foreigntable[ $tableField ] = '';
		}
		else
		{
			$foreigntable[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'foreign_table' ];
		}

		return $foreigntable[ $tableField ];
	}

	/**
	 * getColumnConfigForeigntableWhere():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	foreigntable
	 * @access public
	 * @version 0.9.15
	 * @since 0.9.15
	 */
	public static function getColumnConfigForeigntableWhere( $table, $column )
	{
		static $foreigntableWhere = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $foreigntableWhere[ $tableField ] ) )
		{
			return $foreigntableWhere[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'foreign_table_where' ] ) )
		{
			$foreigntableWhere[ $tableField ] = '';
		}
		else
		{
			$foreigntableWhere[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'foreign_table_where' ];
		}

		return $foreigntableWhere[ $tableField ];
	}

	/**
	 * getColumnConfigMM():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	MM-table
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.5
	 */
	public static function getColumnConfigMM( $table, $column )
	{
		static $MMtable = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $MMtable[ $tableField ] ) )
		{
			return $MMtable[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM' ] ) )
		{
			$MMtable[ $tableField ] = '';
		}
		else
		{
			$MMtable[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM' ];
		}

		return $MMtable[ $tableField ];
	}

	/**
	 * getColumnConfigMMmatchfields():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	MM-table
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.5
	 */
	public static function getColumnConfigMMmatchfields( $table, $column )
	{
		static $MMmatchfields = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $MMmatchfields[ $tableField ] ) )
		{
			return $MMmatchfields[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_match_fields' ] ) )
		{
			$MMmatchfields[ $tableField ] = '';
		}
		else
		{
			$MMmatchfields[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_match_fields' ];
		}

		return $MMmatchfields[ $tableField ];
	}

	/**
	 * getColumnConfigMMmatchfieldsForeign():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	MM match fiels table foreign
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.5
	 */
	public static function getColumnConfigMMmatchfieldsForeign( $table, $column )
	{
		static $MMmatchfieldsForeign = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $MMmatchfieldsForeign[ $tableField ] ) )
		{
			return $MMmatchfieldsForeign[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ] ) )
		{
			$MMmatchfieldsForeign[ $tableField ] = '';
		}
		else
		{
			$MMmatchfieldsForeign[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_match_fields' ][ 'table_foreign' ];
		}

		return $MMmatchfieldsForeign[ $tableField ];
	}

	/**
	 * getColumnConfigMMmatchfieldsLocal():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	MM match fiels table local
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.5
	 */
	public static function getColumnConfigMMmatchfieldsLocal( $table, $column )
	{
		static $MMmatchfieldsLocal = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $MMmatchfieldsLocal[ $tableField ] ) )
		{
			return $MMmatchfieldsLocal[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_match_fields' ][ 'table_local' ] ) )
		{
			$MMmatchfieldsLocal[ $tableField ] = '';
		}
		else
		{
			$MMmatchfieldsLocal[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_match_fields' ][ 'table_local' ];
		}

		return $MMmatchfieldsLocal[ $tableField ];
	}

	/**
	 * getColumnConfigMMoppositeField():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	MM match fiels table local
	 * @access public
	 * @version 0.9.12
	 * @since 0.9.12
	 */
	public static function getColumnConfigMMoppositeField( $table, $column )
	{
		static $MMoppositeField = array();
		$tableField = $table . '.' . $column;

		// RETURN static value
		if( isset( $MMoppositeField[ $tableField ] ) )
		{
			return $MMoppositeField[ $tableField ];
		}

		// Check TCA
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'MM_opposite_field' ] ) )
		{
			$MMoppositeField[ $tableField ] = '';
		}
		else
		{
			$MMoppositeField[ $tableField ] = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'MM_opposite_field' ];
		}

		// RETURN result
		return $MMoppositeField[ $tableField ];
	}

	/**
	 * getColumnConfigType():
	 *
	 * @param string	table
	 * @param string	column
	 * @return string	eval
	 * @access public
	 * @version 0.30.7
	 * @since 0.30.7
	 */
	public static function getColumnConfigType( $table, $column )
	{
		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'type' ] ) )
		{
			return;
		}

		return $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'config' ][ 'type' ];
	}

	/**
	 * getColumnLabel():
	 *
	 * @param string	table
	 * @param string	column
	 * @param bool	bollLabelWiColumn
	 * @return string	label of the given column
	 * @access public
	 * @version 0.20,1
	 * @since 0.9.2
	 */
	public static function getColumnLabel( $table, $column, $boolLabelWiColumn = false )
	{
		$label = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'label' ];
		$label = LanguageUtility::getLabelInContextofBEOrFe( $label );

		if( empty( $label ) )
		{
			$label = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $column ][ 'label' ];
		}
		if( empty( $label ) )
		{
			return $column;
		}

		if( !$boolLabelWiColumn )
		{
			return $label;
		}

		return $label . ' (' . $column . ')';
	}

	/**
	 * getColumnValueRenderedByEval():
	 *
	 * @param string	table
	 * @param string	field
	 * @param string	value
	 * @return string	value rendered in respect on eval configuration
	 * @access public
	 * @version 0.20.13
	 * @since 0.20.13
	 */
	public static function getColumnValueRenderedByEval( $table, $field, $value )
	{
		// 200515, dwildt, -
		//$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		// #t5501, 200702, dwildt, 13+
		switch( true )
		{
			case(strpos( $eval, 'datetime' ) !== false):
			case(strpos( $eval, 'date' ) !== false):
			case(strpos( $eval, 'timesec' ) !== false):
			case(strpos( $eval, 'time' ) !== false):
				$format = self::_dateFormat( $table, $field );
				$value = date( $format, $value );
				break;
			default:
				break;
		}
		return $value;


		// #t5501, 200702, dwildt, 3-
//		$format = self::_dateFormat( $table, $field );
//		$value = date( $format, $value );
//		return $value;
		// 200515, dwildt, -
//		switch( true )
//		{
//			case(strpos( $eval, 'datetime' ) !== false):
//				$format = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.datetime' );
//				$value = date( $format, $value );
//				break;
//			case(strpos( $eval, 'date' ) !== false):
//				$format = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.date' );
//				$value = date( $format, $value );
//				break;
//			case(strpos( $eval, 'timesec' ) !== false):
//				$format = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.timesec' );
//				$value = date( $format, $value );
//				break;
//			case(strpos( $eval, 'time' ) !== false):
//				$format = $GLOBALS[ 'LANG' ]->sL( self::$_LLLpath . 'format.time' );
//				$value = date( $format, $value );
//				break;
//			default:
//				break;
//		}
//		return $value;
	}

	/**
	 * getColumnValueRenderedByEval():
	 *
	 * @param string	table
	 * @param string	field
	 * @return string	format for php date method
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public static function getColumnDateFormat( $table, $field )
	{
		return self::_dateFormat( $table, $field );
	}

	/**
	 * getCtrlDefaultSortBy():
	 *
	 * @param string	table
	 * @return string	path to the icon file of the given table
	 * @access public
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	public static function getCtrlDefaultSortBy( $table )
	{
		$orderBy = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'default_sortby' ];

		if( empty( $orderBy ) )
		{
			return null;
		}

		return ' ' . $orderBy;
	}

	/**
	 * getCtrlSearchfields():
	 *
	 * @param string	table
	 * @param bool fallback	If earchFields are empty, fallback to showRecordFieldList
	 * @return string	
	 * @access public
	 * @version 0.9.5
	 * @since 0.9.5
	 */
	public static function getCtrlSearchfields( $table, $fallback = true )
	{
		$searchfields = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'searchFields' ];
		if( !$fallback )
		{
			return $searchfields;
		}
		if( empty( $searchfields ) )
		{
			$searchfields = $GLOBALS[ 'TCA' ][ $table ][ 'interface' ][ 'showRecordFieldList' ];
		}
		return $searchfields;
	}

	/**
	 * getCtrlSortbyOrDefaultsortby():
	 *
	 * @param string	table
	 * @return string	path to the icon file of the given table
	 * @access public
	 * @version 0.9.4
	 * @since 0.9.4
	 */
	public static function getCtrlSortbyOrDefaultsortby( $table )
	{
		$sortby = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'sortby' ];
		if( !empty( $sortby ) )
		{
			return $sortby;
		}
		return self::getCtrlDefaultSortBy( $table );
	}

	/**
	 * getCtrlDeletedHidden():
	 *
	 * @param string	table
	 * @return string	
	 * @access public
	 * @version 0.9.2
	 * @since 0.9.2
	 */
	public static function getCtrlDeletedHidden( $table )
	{
		$statements[ 0 ] = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'delete' ];
		// 200111, dwildt: display hidden records, 1-/+
		//$statements[ 1 ] = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'enablecolumns' ][ 'hidden' ];
		$statements[ 1 ] = '';

		if( empty( $statements[ 0 ] . $statements[ 1 ] ) )
		{
			return;
		}

		if( empty( $statements[ 0 ] ) )
		{
			unset( $statements[ 0 ] );
		}

		if( empty( $statements[ 1 ] ) )
		{
			unset( $statements[ 1 ] );
		}

		$where = ' WHERE ' . implode( ' = 0,', $statements ) . ' = 0';
		return $where;
	}

	/**
	 * getCtrlIconfile():
	 *
	 * @param string	table
	 * @return string	path to the icon file of the given table
	 * @access public
	 * @version 0.8.10
	 * @since 0.9.2
	 */
	public static function getCtrlIconfile( $table )
	{
		$iconfile = NULL;
		if( isset( $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'iconfile' ] ) )
		{
			$iconfile = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'iconfile' ];
		}

		return $iconfile;
	}

	/**
	 * getCtrlLabel():
	 *
	 * @param string	table
	 * @return string	
	 * @access public
	 * @version 0.10.1
	 * @since 0.9.2
	 */
	public static function getCtrlLabel( $table )
	{
		$label = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label' ];
		return $label;
	}

	/**
	 * getCtrlLabel():
	 *
	 * @param string	table
	 * @return string	
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public static function getCtrlLabelAlt( $table )
	{
		$labelAlt = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label_alt' ];
		return $labelAlt;
	}

	/**
	 * getCtrlLabel():
	 *
	 * @param string	table
	 * @return string	
	 * @access public
	 * @version 0.10.1
	 * @since 0.10.1
	 */
	public static function getCtrlLabelAltForce( $table )
	{
		$labelAltForce = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label_alt_force' ];
		return $labelAltForce;
	}

	/**
	 * getCtrlLabelForSql():
	 *
	 * @param string	table
	 * @return string	
	 * @access public
	 * @version 0.9.5
	 * @since 0.9.2
	 */
	public static function getCtrlLabelForSql( $table )
	{
		$label = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label' ];
		$label = LanguageUtility::getLabelInContextofBEOrFe( $label );
		if( empty( $label ) )
		{
			$label = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label' ];
		}
		$labelAlt = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label_alt' ];
		$labelAltForce = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'label_alt_force' ];

		if( $labelAltForce )
		{
			$csvLabel = $label . ',' . trim( $labelAlt, ' ' );
			$arrLabel = self::_sqlUnixtimeToDateformat( $table, $csvLabel );
			$label = self::_sqlConcat( $arrLabel );
		}
		else
		{
			$label = self::_sqlQuoteField( $label );
		}

		return $label;
	}

	/**
	 * getCtrlTitle():
	 *
	 * @param string	table
	 * @return string	label of the given table
	 * @access public
	 * @version 0.8.21
	 * @since 0.9.2
	 */
	public static function getCtrlTitle( $table )
	{
		$label = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'title' ];
		$label = LanguageUtility::getLabelInContextofBEOrFe( $label );
		if( empty( $label ) )
		{
			return $table;
		}
		// #i0105, 191005, dwildt., 1-/+
		//$label = $table . ' (' . $label . ')';
		// #i0105, 191005, dwildt., 1-
		//$label = $label . ' (' . $table . ')';

		return $label;
	}

	/**
	 * isTimestamp():
	 *
	 * @param		string	table
	 * @param		string	label
	 * @return	boolean	
	 * @access	public
	 * @version	0.9.2
	 * @since		0.9.2
	 */
	public static function isTimestamp( $table, $field )
	{
		$eval = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'eval' ];
		switch( true )
		{
			case(strpos( $eval, 'date' ) !== false):
//			case(strpos( $eval, 'datetime' ) !== false):
			case(strpos( $eval, 'time' ) !== false):
//			case(strpos( $eval, 'timesec' ) !== false):
				return true;
			default:
				return false;
		}
	}

	/**
	 * isTypeInline():
	 *
	 * @param		string	table
	 * @param		string	field
	 * @return	boolean
	 * @access	public
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	public static function isTypeInline( $table, $field )
	{
		static $isTypeInline = array();
		$tableField = $table . '.' . $field;

		// RETURN static value
		if( isset( $isTypeInline[ $tableField ] ) )
		{
			return $isTypeInline[ $tableField ];
		}

		if( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'type' ] == 'inline' )
		{
			$isTypeInline[ $tableField ] = true;
		}
		else
		{
			$isTypeInline[ $tableField ] = false;
		}

		return $isTypeInline[ $tableField ];
	}

	/**
	 * isMM():
	 *
	 * @param		string	table
	 * @param		string	field
	 * @return	boolean	
	 * @access	public
	 * @version	0.30.19
	 * @since		0.30.19
	 */
	public static function isMM( $table, $field )
	{
		static $isMM = array();
		$tableField = $table . '.' . $field;

		// RETURN static value
		if( isset( $isMM[ $tableField ] ) )
		{
			return $isMM[ $tableField ];
		}

		if( !isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'config' ][ 'MM' ] ) )
		{
			$isMM[ $tableField ] = false;
		}
		else
		{
			$isMM[ $tableField ] = true;
		}

		return $isMM[ $tableField ];
	}

}
