<?php

namespace Netzmacher\Xblog\ViewHelpers;

use Netzmacher\Xblog\Utility\SqlUtility;
use Netzmacher\Xblog\Utility\TcaUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 * 
 * @version 0.30.19
 * @since		0.30.17
 */
class ForeigntableViewHelper extends \TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper
{

	/**
	 * @var string	column type: default || individual
	 */
	private $_columntype;

	/**
	 * @var string	the column
	 */
	private $_columnvalue;

	/**
	 * @var array current data row
	 */
	private $_data;

	/**
	 * @var boolean Don't allow HTML
	 */
	private $_dontstriptags;

	/**
	 * @var string current field
	 */
	private $_field;

	/**
	 * @var string current table
	 */
	private $_table;

	/**
	 * @return void
	 * 
	 * @version 0.30.19
	 * @since		0.30.0
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();
//     * @param string $name Name of the argument
//     * @param string $type Type of the argument
//     * @param string $description Description of the argument
//     * @param boolean $required If TRUE, argument is required. Defaults to FALSE.
//     * @param mixed $defaultValue Default value of argument
		$this->registerArgument( 'columntype', 'string', 'Default or individual column type', true );
		$this->registerArgument( 'columnvalue', 'string', 'Individual column', true );
		$this->registerArgument( 'dontstriptags', 'boolean', 'Allow HTML', true );
		$this->registerArgument( 'data', 'array', 'Current row', true );
		$this->registerArgument( 'debugid', 'string', 'Unique id for debugging', true );
		$this->registerArgument( 'field', 'string', 'Field of the localtable', true );
		$this->registerArgument( 'table', 'string', 'Name of the localtable', true );
	}

	/**
	 * render():	Returns the value of the register.
	 * 						If argument test is enabled, it will checked, if a register returns 
	 * 						a value and if the return value matchs the given type.
	 * 						If test failes, the method returns a prompt.
	 * 						If you like the prompt at a different place, please use TestGetregisterViewHelper.
	 * 						TestGetregisterViewHelper will not prompt nothing, if test is succesfull.
	 * 
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	public function render()
	{
		$this->_setVariables();
		$foreign_table = TcaUtility::getColumnConfigForeigntable( $this->_table, $this->_field );

		if( empty( $foreign_table ) )
		{
			return $this->_localtable();
		}

		return $this->_foreigntable();
	}

	/**
	 * _foreigntable():	Returns a value, if current field has a foreogn table configuration
	 * 
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.17
	 */
	private function _foreigntable()
	{
		$mmTable = TcaUtility::getColumnConfigMM( $this->_table, $this->_field );
		switch( true )
		{
			case(!empty( $mmTable )):
				$html = $this->_foreigntableWiMM( $mmTable );
				break;
			default:
				$html = $this->_foreigntableWoMM();
				break;
		}
		return $html;
	}

	/**
	 * _foreigntableWiMM():	
	 * 
	 * @param   string	$mmTable
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _foreigntableWiMM( $mmTable )
	{
		$uid = $this->_data[ 'uid' ];
		$mmTable = TcaUtility::getColumnConfigMM( $this->_table, $this->_field );
		$table_foreign = TcaUtility::getColumnConfigMMmatchfieldsForeign( $this->_table, $this->_field );
		$table_local = TcaUtility::getColumnConfigMMmatchfieldsLocal( $this->_table, $this->_field );

		$rows = SqlUtility::getRecordsFromTableMM( $mmTable, $uid, $table_foreign, $table_local );
		$html = $this->_foreigntableWiMMHtml( $rows, $table_foreign );
		return $html;
	}

	/**
	 * _foreigntableWiMMHtml():	Returns the html from the foreign table
	 * 
	 * @param		array		$rows
	 * @param		string	$table_foreign	label of the foreign table
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.17
	 */
	private function _foreigntableWiMMHtml( $rows, $table_foreign )
	{
		if( empty( $rows ) )
		{
			return;
		}

		$labelField = TcaUtility::getCtrlLabel( $table_foreign );
		foreach( $rows as $row )
		{
			$uid = $row[ 'uid_foreign' ];
			$record = SqlUtility::getRecord( $table_foreign, $uid );
//			$value = $record[ $labelField ];
//			$values[] = $value;
			$values[] = $this->_value( $record, $labelField );
		}
		array_multisort( $values, SORT_ASC );
		$html = implode( ',', $values );
		return $html;
	}

	/**
	 * _foreigntableWoMM():	
	 * 
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _foreigntableWoMM()
	{
		$csvUids = $this->_data[ $this->_field ];
		if( empty( $csvUids ) )
		{
			return;
		}

		$table_foreign = TcaUtility::getColumnConfigForeigntable( $this->_table, $this->_field );

		$rows = $this->_foreigntableWoMMRows( $csvUids, $table_foreign );
		$html = $this->_foreigntableWoMMHtml( $rows, $table_foreign );
		return $html;
	}

	/**
	 * _foreigntableWoMM():	
	 * 
	 * @param		array		$rows
	 * @param		string	$table
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _foreigntableWoMMHtml( $rows, $table )
	{
		$labelField = TcaUtility::getCtrlLabel( $table );

		if( empty( $rows ) )
		{
			return;
		}
		foreach( $rows as $row )
		{
			$values[] = $this->_value( $row, $labelField );
		}

		array_multisort( $values, SORT_ASC );
		$html = implode( ',', $values );
		return $html;
	}

	/**
	 * _value():	
	 * 
	 * @param		array		$rows
	 * @param		string	$table
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _value( $row, $labelField )
	{
		$value = $this->_valueDontstriptags( $row[ $labelField ] );
		if( $this->_columntype != 'individual' )
		{
			return $value;
		}
		if( empty( $this->_columnvalue ) )
		{
			return $value;
		}
		$value = $this->_valueDontstriptags( $row[ $this->_columnvalue ] );
		return $value;
	}

	/**
	 * _value():	
	 * 
	 * @param		string	$value
	 * @return	string	$value
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _valueDontstriptags( $value )
	{
		if( $this->_dontstriptags )
		{
			return $value;
		}
		$valueWoTags = strip_tags( $value );
		//var_dump(__METHOD__, __LINE__, $value, $valueWoTags);
		return $valueWoTags;
	}

	/**
	 * _foreigntableWoMMHtml():	
	 * 
	 * @param		string	$csvUids
	 * @param		string	$table
	 * @return	string	$html
	 * @version 0.30.19
	 * @since		0.30.19
	 */
	private function _foreigntableWoMMRows( $csvUids, $table )
	{
		$pages = null;
		$depth = 0;
		$where = [];
		$where[ 'AND' ][] = [
				'inset' =>
				[
						'uid' => $csvUids
				]
		];

		$rows = SqlUtility::Query( $table, $pages, $depth, $where );
		return $rows;
	}

	/**
	 * _localtable():	Returns a value, if current table is the local table.
	 * 								If it isn't, it returns null
	 * 
	 * @return	string	$html
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	private function _localtable()
	{
		$value = $this->_data[ $this->_field ];
		$html = $this->_localtableEval( $value );
		return $html;
	}

	/**
	 * _localtableEval():	Returns a value, formatted depending on eval
	 * 
	 * @param		sring		$value
	 * @return	string	$html
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	private function _localtableEval( $value )
	{
		$table = $this->_table;
		$field = $this->_field;

		switch( true )
		{
			case(TcaUtility::evalDate( $table, $field )):
				//var_dump( __METHOD__, __LINE__, $table, $field, $value );
				return date( 'y-m-d', $value );
			case(TcaUtility::evalDouble2( $table, $field )):
				return number_format( $value, 2, '.', ',' );
			case(TcaUtility::evalPassword( $table, $field )):
				return '********';
			case(TcaUtility::evalSaltedPassword( $table, $field )):
				return '********';
			case(TcaUtility::evalTime( $table, $field )):
				return date( 'H:i:s', $value );
			case(TcaUtility::evalYear( $table, $field )):
				return date( 'Y', $value );
			default :
				$value = $this->_valueDontstriptags( $value );
				return $value;
		}
	}

	/**
	 * _setVariables():	
	 * 
	 * @return	void
	 * @version 0.30.19
	 * @since		0.30.17
	 */
	private function _setVariables()
	{
		$this->_columntype = $this->arguments[ 'columntype' ];
		$this->_columnvalue = $this->arguments[ 'columnvalue' ];
		$this->_data = $this->arguments[ 'data' ];
		$this->_dontstriptags = $this->arguments[ 'dontstriptags' ];
		$this->_data = $this->arguments[ 'data' ];
		$this->_field = $this->arguments[ 'field' ];
		$this->_table = $this->arguments[ 'table' ];
	}

}
