<?php

namespace Netzmacher\Xblog\ViewHelpers\Publish;

use Netzmacher\Xblog\Utility\SessionFeUserUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 * 
 * @version 0.31.0
 * @since 0.31.0
 */
class AbstractViewHelper extends \TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper
{

	/**
	 * feUserIsOwner(): returns true, if current frontend user and owner of the current record are the same
	 * 
	 * @param array $arguments
	 * @return boolean
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function feUserIsOwner( $arguments )
	{
		$feUserLoggedOn = $GLOBALS[ 'TSFE' ]->fe_user->user[ 'uid' ];
		$feUserOwner = $arguments[ 'fe_cruser_id' ];

		if( $feUserLoggedOn != $feUserOwner )
		{
			return false;
		}
		return true;
	}

	/**
	 * issetAdvancedPublish(): returns value from register pi1__pi_flexform__settings_flexform_pi1_advanced_publish
	 * 
	 * @return boolean
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function issetAdvancedPublish()
	{
		$advanced_publish = (int) $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_advanced_publish' ];
		if( empty( $advanced_publish ) )
		{
			return false;
		}
		return true;
	}

	/**
	 * issetFeUser(): returns true, if frontend user is logged on
	 * 
	 * @return boolean
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function issetFeUser()
	{
		if( empty( $GLOBALS[ 'TSFE' ]->fe_user->user[ 'uid' ] ) )
		{
			return false;
		}
		return true;
	}

	/**
	 * issetPublishPid(): returns value from register pi1__pi_flexform__settings_flexform_pi1_publish_pid
	 * 
	 * @return boolean
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function issetPublishPid()
	{
		$publishPid = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_publish_pid' ];
		if( !$publishPid )
		{
			return false;
		}
		return true;
	}

	/**
	 * issetStoragePid(): returns value from register pi1__pi_flexform__settings_flexform_pi1_publish_storagepid
	 * 
	 * @return boolean
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function issetStoragePid()
	{
		//var_dump( __METHOD__, __LINE__, $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_publish_storagePid' ] );
		$publishPid = $GLOBALS[ 'TSFE' ]->register[ 'pi1__pi_flexform__settings_flexform_pi1_publish_storagepid' ];
		if( !$publishPid )
		{
			return false;
		}
		return true;
	}

	/**
	 * sessionSet(): 
	 * 
	 * @param array $arguments
	 * @return void
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function sessionSet( $arguments )
	{
		$arrSessionData = [];

		$pageIdPublish = $arguments[ 'viewPublish' ];
		unset( $arguments[ 'viewPublish' ] );
		// Next both lines for developing: generate an error for testing.
		//unset( $arguments[ 'extShowUid' ] );
		//$arguments[ 'viewSingle' ] = -123;
		if( empty( $pageIdPublish ) )
		{
			$prompt = 'FATAL ERROR: $pageIdPublish is empty @ ' . __METHOD__ . '#' . __LINE__;
			die( $prompt );
		}

		foreach( ( array ) $arguments as $key => $value )
		{
			$arrSessionData[ $pageIdPublish ][ $key ] = $value;
		}

		$plugin = 'tx_publish_pi1';
		SessionFeUserUtility::Set( $plugin, $arrSessionData );
	}

	/**
	 * sessionUnset(): 
	 * 
	 * @param array $arguments
	 * @return void
	 * 
	 * @version 0.31.0
	 * @since 0.31.0
	 */
	protected static function sessionUnset( $arguments )
	{
		$arrSessionData = [];

		$pageIdPublish = $arguments[ 'viewPublish' ];
		if( empty( $pageIdPublish ) )
		{
			// do nothing
			return;
		}

		$arrSessionData[ $pageIdPublish ] = null;
		$plugin = 'tx_publish_pi1';

		SessionFeUserUtility::Set( $plugin, $arrSessionData );
	}

}
