<?php

namespace Netzmacher\Xblog\ViewHelpers\Publish;

use TYPO3Fluid\Fluid\Core\Rendering\RenderingContextInterface;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 * 
 * @version 0.31.0
 * @since 0.31.0
 */
class IconbarViewHelper extends AbstractViewHelper
{

  protected $escapeOutput = false;

  /**
   * @return void
   * 
   * @version 0.31.0
   * @since 0.31.0
   */
  public function initializeArguments()
  {
//     * @param string $name Name of the argument
//     * @param string $type Type of the argument
//     * @param string $description Description of the argument
//     * @param boolean $required If TRUE, argument is required. Defaults to FALSE.
//     * @param mixed $defaultValue Default value of argument
    $this->registerArgument('fe_cruser_id', 'integer', 'owner/fe_user of the current record', true);
    $this->registerArgument('if', 'string', 'isOwner, isNotOwner', true);
  }

  /**
   * renderStatic():	
   * 
   * @param array $arguments
   * @param \Closure $renderChildrenClosure
   * @param RenderingContextInterface $renderingContext
   * @return boolean	true, if
   * 									* a frontend user is logged on
   * 									and if if in the plugin/flexform
   * 									* is enabled: advanced publish
   * 									* is set: the page id for page with the plugin publish_pi1 
   * @version 0.31.1
   * @since 0.31.0
   */
  public static function renderStatic(array $arguments, \Closure $renderChildrenClosure, RenderingContextInterface $renderingContext)
  {
		unset( $renderChildrenClosure );
		unset( $renderingContext );

    switch ($arguments['if'])
    {
      case('isOwner'):
        return self::_isOwner($arguments);
      case('isNotOwner'):
        return self::_isNotOwner($arguments);
      default:
        $prompt = ''
          . 'Switch: key is undefined: "' . $arguments['if'] . '"<br />' . PHP_EOL
          . '@' . __METHOD__ . '#' . __LINE__;
        die($prompt);
    }
  }

  /**
   * @return boolean
   * 
   * @version 0.31.1
   * @since 0.31.1
   */
  private static function _isNotOwner($arguments)
  {
    switch (true)
    {
      case(!self::issetAdvancedPublish()):
      case(!self::issetFeUser()):
      case(!self::issetPublishPid()):
      case(!self::issetStoragePid()):
      case(self::feUserIsOwner($arguments)):
        return false;
      default:
        return true;
    }
  }

  /**
   * @return boolean
   * 
   * @version 0.31.1
   * @since 0.31.1
   */
  private static function _isOwner($arguments)
  {
    switch (true)
    {
      case(!self::issetAdvancedPublish()):
      case(!self::issetFeUser()):
      case(!self::issetPublishPid()):
      case(!self::issetStoragePid()):
      case(!self::feUserIsOwner($arguments)):
        return false;
      default:
        return true;
    }
  }

}
