<?php

namespace Netzmacher\Xblog\ViewHelpers;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * TestGetregisterViewHelper: See description of thze method render()
 * 
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 * 
 * @version 0.30.17
 * @since 0.30.0
 */
class TestGetregisterViewHelper extends \TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper
{

	protected $escapeOutput = false;
	private $register = null;

	/**
	 * @return void
	 * 
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();
//     * @param string $name Name of the argument
//     * @param string $type Type of the argument
//     * @param string $description Description of the argument
//     * @param boolean $required If TRUE, argument is required. Defaults to FALSE.
//     * @param mixed $defaultValue Default value of argument
		$this->registerArgument( 'mustnotbeempty', 'boolean', 'return value must not be empty', true );
		$this->registerArgument( 'register', 'string', 'name of the register', true );
		$this->registerArgument( 'debugid', 'string', 'some id like @dksvnoeifh', true );
		$this->registerArgument( 'type', 'string', 'type of the return value like string or integer', true );
	}

	/**
	 * render():	Check, if a register returns a value and if the return value matchs the given type.
	 * 						The method returns:
	 * 						* nothing, of test is successfull
	 * 						* a prompt in case of a not proper test
	 * 
	 * @return string
	 * 
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	public function render()
	{
		$this->register = $GLOBALS[ 'TSFE' ]->register[ $this->arguments[ 'register' ] ];

		$prompt = $this->_isset();
		if( $prompt )
		{
			return $prompt;
		}

		$prompt = $this->_mustnotbeempty();
		if( $prompt )
		{
			return $prompt;
		}

		$prompt = $this->_type();
		if( $prompt )
		{
			return $prompt;
		}

		return null;
	}

	/**
	 * _isset(): returns a prompt, if register isn't set
	 * 
	 * @return string
	 * 
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	private function _isset()
	{
		if( isset( $GLOBALS[ 'TSFE' ]->register[ $this->arguments[ 'register' ] ] ) )
		{
			return;
		}
		$prompt = '<div style="background:red;color:white;font-weight:bold;padding:.2em;text-align:center;">'
						. '  Register "' . $this->arguments[ 'register' ] . '" is not set<br />'
						. '  Debug id: ' . $this->arguments[ 'debugid' ] . '<br />'
						. '  @' . __METHOD__ . '#' . __LINE__
						. '</div>'
		;
		return $prompt;
	}

	/**
	 * _mustnotbeempty(): returns a prompt, if register value is null or an empty string
	 * 
	 * @return string
	 * 
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	private function _mustnotbeempty()
	{
		if( !$this->arguments[ 'mustnotbeempty' ] )
		{
			return;
		}

		switch( true )
		{
			case($this->register == null):
			case($this->register == ''):
				$prompt = '<div style="background:red;color:white;font-weight:bold;padding:.2em;text-align:center;">'
								. '  Register ' . $this->arguments[ 'register' ]
								. '  is "' . $this->register . '"'
								. '  but must not be null or ""<br />'
								. '  Debug id: ' . $this->arguments[ 'debugid' ] . '<br />'
								. '  @' . __METHOD__ . '#' . __LINE__
								. '</div>'
				;
				return $prompt;
			default:
				return;
		}
	}

	/**
	 * _type(): returns a prompt, if type of the register value doesn't match the given type
	 * 
	 * @return string
	 * 
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	private function _type()
	{
		switch( true )
		{
			case(!isset( $this->arguments[ 'type' ] )):
				return;
			case($this->arguments[ 'type' ] == 'array'):
				return $this->_typeArray();
			case($this->arguments[ 'type' ] == 'integer'):
				return $this->_typeInteger();
			case($this->arguments[ 'type' ] == 'string'):
				return;
			default:
				$prompt = '<div style="background:red;color:white;font-weight:bold;padding:.2em;text-align:center;">'
								. '  Register "' . $this->register . '" must be of type "' . $this->arguments[ 'type' ] . '",'
								. '  but is of type "' . gettype( $this->register ) . '"<br />'
								. '  Debug id: ' . $this->arguments[ 'debugid' ] . '<br />'
								. '  @' . __METHOD__ . '#' . __LINE__
								. '</div>'
				;
				return $prompt;
		}
	}

	/**
	 * _typeArray(): returns a prompt, if type of the register value does not match array
	 * 
	 * @return string
	 * 
	 * @version 0.30.17
	 * @since		0.30.17
	 */
	private function _typeArray()
	{
		if( is_array($this->register) )
		{
			return;
		}
		$prompt = '<div style="background:red;color:white;font-weight:bold;padding:.2em;text-align:center;">'
						. '  Register "' . $this->register . '" must be of type array,'
						. '  but is of type "' . gettype( $this->register ) . '"<br />'
						. '  Debug id: ' . $this->arguments[ 'debugid' ] . '<br />'
						. '  @' . __METHOD__ . '#' . __LINE__
						. '</div>'
		;
		return $prompt;
	}

	/**
	 * _typeInteger(): returns a prompt, if type of the register value does not match integer
	 * 
	 * @return string
	 * 
	 * @version 0.30.0
	 * @since 0.30.0
	 */
	private function _typeInteger()
	{
		if( ( string ) ( int ) $this->register === ( string ) $this->register )
		{
			return;
		}
		$prompt = '<div style="background:red;color:white;font-weight:bold;padding:.2em;text-align:center;">'
						. '  Register "' . $this->register . '" must be of type integer,'
						. '  but is of type "' . gettype( $this->register ) . '"<br />'
						. '  Debug id: ' . $this->arguments[ 'debugid' ] . '<br />'
						. '  @' . __METHOD__ . '#' . __LINE__
						. '</div>'
		;
		return $prompt;
	}

}
