<?php

namespace Netzmacher\Xblog\ViewHelpers\Xml;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019-2021 - Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 * 
 * @version 0.8.16
 * @since 0.8.16
 */
class TagViewHelper extends \TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper
{

	/**
	 * @return void
	 * 
	 * @version 0.8.16
	 * @since 0.8.16
	 * @internal #i0091
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();
		$this->registerArgument( 'tag', 'string', 'tag', false, '' );
		$this->registerArgument( 'cdata', 'string', 'cdata', false, 'true' );
	}

	/**
	 * @return string
	 * 
	 * @version 0.8.16
	 * @since 0.8.16
	 * @internal #i0091
	 */
	public function render()
	{
		$tag = $this->arguments[ 'tag' ];
		$cdata = $this->arguments[ 'cdata' ];

		if( $tag == 'xml' )
		{
			return '<?xml version="1.0" encoding="utf-8" ?>';
		}

		$tagProperties = '';
		if( $tag == "rdf" )
		{
			$tagProperties = ' xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:content="http://purl.org/rss/1.0/modules/content/" xmlns="http://purl.org/rss/1.0/"';
			$cdata = 'false';
		}
		if( $tag == "rss" )
		{
			$tagProperties = ' version="2.0" xmlns:content="http://purl.org/rss/1.0/modules/content/"';
			$cdata = 'false';
		}

		$tagOpen = '<' . $tag . $tagProperties . '>';
		$tagClose = '</' . $tag . '>';

		$cdataOpen = '<![CDATA[';
		$cdataClose = ']]>';

		if( $cdata != 'true' )
		{
			$cdataOpen = '';
			$cdataClose = '';
		}

		$content = $this->renderChildren();
		$content = $tagOpen . $cdataOpen . trim( $content ) . $cdataClose . $tagClose;
		return $content;
	}

}
